@@ -5,9 +5,11 @@
 # backend_aml.py: support for running jobs under Azure Machine Learning
 import os
 import json
+import uuid
 import shutil
 import logging
 import urllib.request
+from hyperopt import hp
 from interface import implements
 
 from xtlib import utils
@@ -16,6 +18,8 @@ from xtlib import scriptor
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import run_helper
+from xtlib import pc_utils
+from xtlib.hparams.hp_process import HPList, HPDist
 
 from xtlib.console import console
 from xtlib.hparams import hp_helper
@@ -32,6 +36,8 @@ SECTION = "external-services"
 # azure library loaded on demand
 Workspace = None 
 Experiment = None
+Run = None
+ComputeTarget = None
 
 class AzureML(BackendBase):
 
@@ -72,6 +78,7 @@ class AzureML(BackendBase):
         # local or POOL of vm's
         fn_wrapped = None     # we use same script for each box (but with different ARGS)
         username = args["username"]
+        fn_shim = constants.FN_AML_SHIM
 
         for i, box_runs in enumerate(job_runs):
             # wrap the user commands in FIRST RUN of each box (apply data/model actions)
@@ -104,7 +111,6 @@ class AzureML(BackendBase):
 
                 # AML wants a python script, so use our tiny python shim to run __wrapped__.sh 
                 bootstrap_dir = args["bootstrap_dir"]
-                fn_shim = constants.FN_AML_SHIM
                 fn_from = file_utils.get_xtlib_dir() + "/backends/" + constants.FN_AML_SHIM
                 fn_to = bootstrap_dir + "/" + fn_shim
                 shutil.copyfile(fn_from, fn_to)
@@ -136,8 +142,12 @@ class AzureML(BackendBase):
         '''
         ITP subclass overwrites this method.
         '''
-        # we keep the working directory assigned to us by AML because our code is already 
-        # downloaded here, and the log files are handy to access (as subdirs)
+        # we need to move our cwd (default working dir is blob-mapped and slow)
+        # as of 10/21/2020, ITP recommends we use "/var/tmp" until they disable the NFS mount for $HOME
+        # guid = str(uuid.uuid4())
+        # job_home = "$HOME/{}".format(guid)
+        job_home = "/var/tmp"
+        cwd = "{}/.xt/cwd".format(job_home)
 
         use_gpu = args["use_gpu"]
         self.default_docker_image = self.get_default_docker_image(use_gpu)
@@ -147,7 +157,7 @@ class AzureML(BackendBase):
             data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
             is_windows=False, sudo_available=sudo_available, username=username, use_username=False, 
             install_blobfuse=True, setup=setup,  use_allow_other=use_allow_other, 
-            remove_zip=False, homebase=".", cwd=".", args=args)
+            remove_zip=False, homebase=job_home, cwd=cwd, args=args, copy_code=True)
 
         return fn_wrapped
 
@@ -342,7 +352,7 @@ class AzureML(BackendBase):
 
         return {"workspace": ws_name, "run_name": run_name, "cancelled": killed, "status": status}
         
-    def cancel_runs(self, exper_name, run_names):
+    def cancel_runs(self, ws_name, exper_name, run_names):
         results = []
 
         for ws_run_name in run_names:
@@ -353,6 +363,7 @@ class AzureML(BackendBase):
 
             if run_helper.is_run_name(run_name):
                 run_name = exper_name + "." + run_name
+
             result = self.cancel_run(ws_name, run_name)
             results.append(result)
 
@@ -361,7 +372,7 @@ class AzureML(BackendBase):
 
     def get_active_jobs(self, ws_name):
         ''' return a list of job_id's running on this instance of Azure Batch '''
-        mongo = self.store.get_mongo()
+        db = self.store.get_database()
 
         filter_dict = {
             "username": self.username,
@@ -373,7 +384,7 @@ class AzureML(BackendBase):
 
         fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
 
-        job_records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
+        job_records = db.get_info_for_jobs(ws_name, filter_dict, fields_dict)
 
         return job_records
 
@@ -482,7 +493,7 @@ class AzureML(BackendBase):
         return ws
 
     def get_aml_ws_sizes(self, aml_ws_name):
-        ws = get_aml_ws(self.config, aml_ws_name)
+        ws = self.get_aml_ws(self.config, aml_ws_name)
 
         # TODO: make this an xt cmd: xt list sizes
         from azureml.core.compute import ComputeTarget, AmlCompute
@@ -498,7 +509,7 @@ class AzureML(BackendBase):
 
         for name, value in hp_sets.items():
             if isinstance(value, HPList):
-                hd[name] = choice(*value.values)
+                hd[name] = hp.choice(*value.values)
             elif isinstance(value, HPDist):
                 # convert from comma sep. string to list of float values
                 values = utils.get_number_or_string_list_from_text(value.values)
@@ -670,8 +681,8 @@ class AzureML(BackendBase):
 
         provider_code_path = config.get_storage_provider_code_path(store_creds)
         
-        mongo_creds, mongo_name = self.config.get_mongo_creds()
-        mongo_conn_str = mongo_creds["mongo-connection-string"]
+        mongo_creds = self.config.get_database_creds()
+        mongo_conn_str = mongo_creds["connection-string"]
 
         username = args["username"]
         description = args["description"]
@@ -842,11 +853,17 @@ class AzureML(BackendBase):
                     user_managed=user_managed, environment_variables=env_vars, node_count=node_count, distributed_training=distributed_obj,
                     max_run_duration_seconds=max_secs, custom_docker_image=docker_image)
 
+            # supress AML/ITP warnings about duplicate script/arguments values
+            # estimator.run_config.arguments = []
+            # estimator.run_config.script = None
+            # estimator.arguments = []
+            # estimator.script = None
+
             # update estimator for ITP 
             sku = utils.safe_value(self.compute_def, "sku", "g1")
             gpu_count = int(sku[1:]) if isinstance(sku, str) else 0
 
-            low_pri = utils.safe_value(self.compute_def, "low_pri", False)
+            low_pri = utils.safe_value(self.compute_def, "low-pri", False)
 
             self.update_estimator(estimator, gpu_count, low_pri)
 
@@ -860,8 +877,10 @@ class AzureML(BackendBase):
 
         username = args["username"]
         aml_exper_name = "{}__{}__{}".format(username, workspace, experiment)
+
         cwd = os.getcwd()
-        bootstrap_dir = args["bootstrap_dir"]
+
+        #bootstrap_dir = args["bootstrap_dir"]
 
         compute = args["target"]
         compute_def = args["compute_def"]
@@ -875,13 +894,15 @@ class AzureML(BackendBase):
         service_info_by_node = {}
 
         for i, node_runs in enumerate(job_runs):
-            node_info = self.submit_node_runs(job_id, node_runs, workspace, aml_ws_name, experiment, aml_exper_name, compute_def, resume_name, 
-                repeat_count, using_hp, compute, runs_by_box, bootstrap_dir, i, show_aml_run_name, nodes, args)
+            node_info = self.submit_node_runs(job_id, node_runs, workspace, aml_ws_name, experiment, aml_exper_name, 
+                compute_def, resume_name, repeat_count, using_hp, compute, runs_by_box, snapshot_dir, i, 
+                show_aml_run_name, nodes, args)
 
             node_id = "node" + str(i)
             service_info_by_node[node_id] = node_info
 
-        fb.feedback("job submitted", is_final=True)
+        fb.reset_feedback()
+        fb.feedback("submitted")
 
         service_run_info = {}
         return service_run_info, service_info_by_node     
@@ -893,9 +914,10 @@ class AzureML(BackendBase):
         first_run = node_runs[0]
         first_run_name = first_run["run_name"]
         fake_submit = args["fake_submit"]
+        trainer = None
 
         # this indicates we should make serializable versions of estimator and trainer
-        self.submit_logs = True or fake_submit  # must be true if we are using fake_submit
+        self.submit_logs = True   #  fake_submit  # must be true if we are using fake_submit
 
         self.serializable_estimator = None
         self.serializable_trainer = None
@@ -931,6 +953,9 @@ class AzureML(BackendBase):
         if not compute_target:
             errors.config_error("'compute' property missing on compute target '{}' in XT config file".format(compute))
 
+        # append job/run name to AML experiment name (make locating XT runs in AML portal easier)
+        aml_exper_name += "__{}__{}".format(job_id, first_run_name)
+
         estimator, experiment = self.create_estimator(job_id, workspace, aml_ws_name, xt_exper_name, aml_exper_name, first_run_name, 
             code_dir, target_fn, arg_dict, compute_target, node_id, nodes, fake_submit, args)
 
@@ -960,6 +985,8 @@ class AzureML(BackendBase):
                 #max_minutes = 43200   # aml workaround: None not supported, either is -1 or 0, so use max value
                 max_minutes = 10080   # aml workaround: documented max not supported
 
+            hp_sets = None     # where is this supposed to come from?
+
             if hp_sets:
                 hd_dict = self.build_hyperdrive_dict(hp_sets)
             else:
@@ -973,8 +1000,8 @@ class AzureML(BackendBase):
                     early_term = {"policy_type": policy_name, "eval_interval": eval_interval, "delay_eval": delay_eval, 
                         "truncation_percentage": truncation_percentage, "slack_factor": slack_factor, "slack_amount": slack_amount}
 
-                    self.serializable_trainer = {"estimator": serializable_estimator, "hd_dict": hd_dict, "search_type": search_type, "primary_metric": primary_metric, 
-                        "maximize_metric": maximize_metric, "early_term": serializable_early_term, "max_total_runs": max_runs, "max_concurrent_runs": max_concurrent_runs, 
+                    self.serializable_trainer = {"estimator": self.serializable_estimator, "hd_dict": hd_dict, "search_type": search_type, "primary_metric": primary_metric, 
+                        "maximize_metric": maximize_metric, "early_term": self.serializable_early_term, "max_total_runs": max_runs, "max_concurrent_runs": max_concurrent_runs, 
                         "max_minutes": max_minutes}
 
                 if fake_submit:
@@ -994,11 +1021,10 @@ class AzureML(BackendBase):
                 box_name, node_index, repeat_count, fake_submit, args)
 
         if show_aml_run_name:
-            fb.feedback("[aml: {}/Run {}], xt: {}/{} ".format(aml_exper_name, aml_run_number, workspace, run_name), is_final=True)
+            console.print("  {} (AML: {}/Run {})".format(run_name, aml_exper_name, aml_run_number))
         else:
-            fb.feedback("{}/{}".format(aml_exper_name, aml_run_number, workspace, run_name))
+            fb.feedback("{}".format(run_name))
 
-        mongo = self.store.get_mongo()
         run_names = []
         for run in node_runs:
             run_name = run["run_name"]
@@ -1016,6 +1042,7 @@ class AzureML(BackendBase):
             # update mongo db info for run with cluster and service_job_id
             # DO WE STILL NEED THIS?  Aug-08-2020
             # I think not, since this is now covered by service_node_info 
+            #db = self.store.get_database()
             #mongo.update_run_info(workspace, run_name, {"aml_exper_name": aml_exper_name, "aml_run_number": aml_run_number})
 
         if monitor_cmd:
@@ -1080,22 +1107,22 @@ class AzureML(BackendBase):
         '''
         errors.general_error("support for talking to XT controller for AML jobs not yet supported")
 
-    def mount(self, storage_name, storage_key, container):
+    # def mount(self, storage_name, storage_key, container):
 
-        ws = Workspace(subscription_id, resource_group, ws_name)     # , auth=svc_pr)
+    #     ws = Workspace(subscription_id, resource_group, ws_name)     # , auth=svc_pr)
 
-        from azureml.core import Datastore
-        datastore = Datastore.register_azure_blob_container(workspace=ws, 
-            datastore_name=container, container_name=container,
-            account_name=storage_name, account_key=storage_key,
-            create_if_not_exists=True)
+    #     from azureml.core import Datastore
+    #     datastore = Datastore.register_azure_blob_container(workspace=ws, 
+    #         datastore_name=container, container_name=container,
+    #         account_name=storage_name, account_key=storage_key,
+    #         create_if_not_exists=True)
 
-        console.print("datastore=", datastore)
+    #     console.print("datastore=", datastore)
 
-        dataref = datastore.as_mount()
-        dir_name = dataref.path_on_compute
-        console.print("datastore MOUNT dir_name=", dir_name)
-        return dir_name
+    #     dataref = datastore.as_mount()
+    #     dir_name = dataref.path_on_compute
+    #     console.print("datastore MOUNT dir_name=", dir_name)
+    #     return dir_name
 
     # API call
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
@@ -1235,9 +1262,11 @@ class AzureML(BackendBase):
         return result
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
 
         self.append(cmds, "mkdir -p {}".format(dest_dir))
         #self.append(cmds, "cp -r -v $AZUREML_LOGDIRECTORY_PATH/* {}".format(dest_dir))
         #self.append(cmds, "cp -r -v $AZ_BATCHAI_STDOUTERR_DIR/* {}".format(dest_dir))
-        self.append(cmds, "cp -r azureml_compute_logs/* {}".format(dest_dir))
+        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml_compute_logs/* {}".format(dest_dir))
+        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml-logs/* {}".format(dest_dir))
+        self.append(cmds, "cp -r $XT_ORIG_WORKDIR/logs/* {}".format(dest_dir))
