@@ -5,6 +5,7 @@
 # backend_base.py: provides a baseclass for backend classes, and defines the API they implement.
 import os
 import inspect
+from typing import List
 from interface import implements
 
 from xtlib.console import console
@@ -13,6 +14,7 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import scriptor
 from xtlib import pc_utils
+from xtlib import run_errors
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import store_utils
@@ -29,9 +31,11 @@ class BackendBase(implements(BackendInterface)):
         self.blobfuse_installed = False
         self.capture_setup_cmds = False
         self.echo_cmds = True
-        self.add_time = self.get_name() in ["pool", "batch"]
         self.default_docker_image = ""
 
+        # TODO: change this to rely on API for this info
+        self.add_time = self.get_name() in ["pool", "batch", "aml"]
+
         # if these are set to constant values, backend must use unique scripts for each node
         # default for backends is to pass these 2 args to their __wrapped__.xx script (philly is different)
         self.node_index = "$1"
@@ -137,8 +141,8 @@ class BackendBase(implements(BackendInterface)):
             echo = self.echo_cmds
 
         if echo:
-            # avoid echo of super long (mutiple line) commands
-            max_echo_len = 100        
+            # avoid echo of super long (multiple line) commands
+            max_echo_len = 300        
             cmd_text = cmd
             if len(cmd_text) > max_echo_len:
                 cmd_text = cmd_text[0:max_echo_len-3] + "..."
@@ -149,8 +153,11 @@ class BackendBase(implements(BackendInterface)):
                 cmd_text = cmd_text.replace(">", "^>")    # must escape the ">" to prevent cmd redirection
 
                 if self.add_time:
+                    # %date% is subject to local formatting, etc. so we use python instead
+                    # cmd = '''python -c "import datetime as dt; print(dt.date.today().strftime('%m/%d/%Y'))"'''
                     # we use sed (.git install required) to remove double quotes here so that cmd piping can be echoed
-                    cmds.append('''echo "@%date% %time%     ++ {}" | sed 's/"//g' '''.format(cmd_text))
+                    #cmds.append('''echo "@%date% %time%     ++ {}" | sed 's/"//g' '''.format(cmd_text))
+                    cmds.append('''echo "%date% %time%     ++ {}" | sed 's/^...../@/' '''.format(cmd_text))
                 else:
                     cmds.append('''echo ++ {}'''.format(cmd_text))
 
@@ -207,6 +214,7 @@ class BackendBase(implements(BackendInterface)):
         
     def append_install_blobfuse_cmds(self, cmds, sudo_available):
         # only install it once per backend instance
+
         if not self.blobfuse_installed:
             self.append_title(cmds, "INSTALL BLOBFUSE:")
             sudo = "sudo " if sudo_available else ""
@@ -229,7 +237,7 @@ class BackendBase(implements(BackendInterface)):
             self.blobfuse_installed = True
 
     def create_blobfuse_commands(self, storage_name, storage_key, sudo_available, mount_requests, install_blobfuse, 
-            use_username=True, use_allow_other=True, nonempty=False):
+            use_username=True, use_allow_other=True, nonempty=False, tmpbase = "$HOME"):
         username = "$USER"
         cmds = []
         sudo = "sudo " if sudo_available else ""
@@ -241,9 +249,11 @@ class BackendBase(implements(BackendInterface)):
             readonly = md["readonly"]
 
             self.blobfuse_index += 1
+            #console.print("tmpbase=", tmpbase)
+
             #tmp_dir = "/mnt/resource/blobfusetmp{}".format(self.blobfuse_index)
-            tmp_dir = "$HOME/blobfusetmp{}".format(self.blobfuse_index)
-            fn_config = "$HOME/fuse{}.cfg".format(self.blobfuse_index)
+            tmp_dir = tmpbase + "/blobfusetmp{}".format(self.blobfuse_index)
+            fn_config = tmpbase + "/fuse{}.cfg".format(self.blobfuse_index)
             readonly_opt = "-o ro" if readonly else ""
             nonempty_opt = "-o nonempty" if nonempty else ""
 
@@ -255,7 +265,7 @@ class BackendBase(implements(BackendInterface)):
             allow_other = "-o allow_other" if use_allow_other else ""
 
             # create temp dir (required by blobfuse)
-            self.append(cmds, "mkdir {} -p".format(tmp_dir))
+            self.append(cmds, "mkdir -p {}".format(tmp_dir))
 
             # create fuse config file (clunky but it works)
             self.append(cmds, "echo accountName {} > {}".format(storage_name, fn_config))
@@ -301,7 +311,7 @@ class BackendBase(implements(BackendInterface)):
                 else:
                     sub_cmds = \
                     [ 
-                        "{}mkdir {} -p".format(sudo, dest_dir),
+                        "{}mkdir -p {}".format(sudo, dest_dir),
                     ]
                     if use_username:
                         sub_cmds.append("{}chown {} {}".format(sudo, username, dest_dir))
@@ -315,15 +325,17 @@ class BackendBase(implements(BackendInterface)):
 
     def emit_mount_cmds(self, cmds, storage_name, storage_key, container, mnt_path, is_writable, 
         install_blobfuse, sudo_available, use_username, use_allow_other, 
-        nonempty=False, cleanup_needed=False):
+        nonempty=False, cleanup_needed=False, tmpbase="$HOME", args=None):
 
         if self.mounting_enabled:
-            if install_blobfuse:
+            user_install_blobfuse = args["install_blobfuse"]
+            if install_blobfuse and user_install_blobfuse:
                 self.append_install_blobfuse_cmds(cmds, sudo_available)
 
             if self.gen_for_windows:
                 # TODO: provide pseudo-mount for local machine by using data-local and store-local config properties
-                errors.combo_error("Mounting of Azure storage (for '{}') not supported by target OS (Windows)".format(store_data_dir))
+                errors.combo_error("Mounting of Azure storage (for '{}') not supported by target OS (Windows)".\
+                    format(storage_name))
 
             if cleanup_needed:
                 # on pool machines, for any action, always UNMOUNT mnt_dir 
@@ -338,12 +350,12 @@ class BackendBase(implements(BackendInterface)):
 
             requests = [ {"container": container, "mnt_dir": mnt_path, "readonly": not is_writable} ]
             sub_cmds = self.create_blobfuse_commands(storage_name, storage_key, sudo_available, requests, install_blobfuse=install_blobfuse,
-                use_username=use_username, use_allow_other=use_allow_other, nonempty=nonempty)
+                use_username=use_username, use_allow_other=use_allow_other, nonempty=nonempty, tmpbase=tmpbase)
             cmds += sub_cmds
 
     def process_action(self, cmds, action, mnt_path, container, store_data_dir, env_var_name, is_writable,
         storage_name, storage_key, sudo_available=True, cleanup_needed=False, is_windows=False, use_username=True, 
-        install_blobfuse=False, use_allow_other=True, nonempty=False, args=None):
+        install_blobfuse=False, use_allow_other=True, nonempty=False, tmpbase=None, args=None):
 
         gen_for_windows = self.set_gen_for_windows(is_windows, args)
 
@@ -351,7 +363,7 @@ class BackendBase(implements(BackendInterface)):
             self.emit_mount_cmds(cmds, storage_name, storage_key, container, mnt_path=mnt_path, 
                 is_writable=is_writable, install_blobfuse=install_blobfuse, sudo_available=sudo_available, 
                 use_username=use_username, use_allow_other=use_allow_other, 
-                nonempty=nonempty, cleanup_needed=cleanup_needed)
+                nonempty=nonempty, cleanup_needed=cleanup_needed, tmpbase=tmpbase, args=args)
 
             self.append_export(cmds, env_var_name, "{}/{}".format(mnt_path, store_data_dir))
 
@@ -410,7 +422,7 @@ class BackendBase(implements(BackendInterface)):
             - cmds: the set of commands to append the export cmd to
             - name: the name of the environment var to set/export
             - value: the string value (could have $xx or %xx% variables in a list)
-            - value_is_windows: if value is windows stye (vs. linux style)
+            - value_is_windows: if value is windows style (vs. linux style)
         '''
         # ensure value is a str
         value = str(value)
@@ -457,22 +469,28 @@ class BackendBase(implements(BackendInterface)):
         
         return cmd
 
-    def add_first_cmds(self, cmds, homebase, cwd, args):
+    def add_first_cmds(self, cmds, homebase, cwd, args, copy_code=False):
 
         # don't add this as a cmd (breaks our script on philly)
         # if not self.gen_for_windows:
         #     self.append(cmds, "#!/bin/sh")
 
         self.append_title(cmds, "FIRST CMDS:")
-    
+
+        self.export_now_to_var(cmds, "XT_PREP_START_TIME")
+
+        # remember original dir for service log files at end
+        pwd_cmd = "%CD%" if self.gen_for_windows else "$(pwd)"
+        self.append_export(cmds, "XT_ORIG_WORKDIR", pwd_cmd, fix_value=False)
+
         if homebase == ".":
-            value = "%CD%" if self.gen_for_windows else "$(pwd)"
-            self.append_export(cmds, "XT_HOMEBASE", value, fix_value=False)
+            self.append_export(cmds, "XT_HOMEBASE", pwd_cmd, fix_value=False)
         else:
             self.append_export(cmds, "XT_HOMEBASE", homebase, fix_value=False)
 
         if cwd == ".":
             value = "%CD%" if self.gen_for_windows else "$(pwd)"
+            cwd = value
             self.append_export(cmds, "XT_CWD", value, fix_value=False)
         else:
             self.append_export(cmds, "XT_CWD", cwd, fix_value=False)
@@ -481,26 +499,48 @@ class BackendBase(implements(BackendInterface)):
         self.append_export(cmds, "XT_RUN_NAME", self.run_name)
         self.append_export(cmds, "XT_MOUNTING_ENABLED", self.mounting_enabled)
 
+        job_id = args["job_id"]
+        ws_name = args["workspace"]
+        self.append_export(cmds, "XT_JOB_ID", job_id)
+        self.append_export(cmds, "XT_WORKSPACE_NAME", ws_name)
+
         # for debugging version 1 vs. 2 issues
         self.append_export(cmds, "XT_STORE_NAME", self.config.get("store"))
         self.append_export(cmds, "XT_JOBS_CONTAINER_NAME", store_utils.get_jobs_container(args["workspace"]))
         self.append_export(cmds, "XT_STORAGE_FORMAT", store_utils.STORAGE_FORMAT)
-        
+
+        # workaround for Pytorch 1.5/1.6 issue
+        # note: this is set in the std xtlib docker files but something in AML must be changing it
+        # so we reset it to GNU here (to avoid MKL NOT COMPATIBLE error in AML runs)
+        self.append(cmds, "echo before setting mkl is $MKL_THREADING_LAYER", echo=False)
+        self.append_export(cmds, "MKL_THREADING_LAYER", "GNU")
+         
         self.add_info_cmds(cmds, args)
 
         # debug
         self.append_dir(cmds)
 
-        if homebase != ".":
-            self.append_title(cmds, "CHANGE TO CONTROLLER DIR:")
-            cwd = "%HOME%/.xt/cwd" if self.gen_for_windows else "$HOME/.xt/cwd"
-            cwd = file_utils.fix_slashes(cwd, is_linux=not self.gen_for_windows)
+        if cwd != ".":
+            #cwd = homebase
+            cwd = file_utils.fix_slashes(cwd, is_linux=not self.gen_for_windows, protect_ws_run_name=False)
+
+            if copy_code:
+                self.append_title(cmds, "COPY/UNZIP CODE TO CWD:")
+            else:
+                self.append_title(cmds, "CHANGE TO CONTROLLER DIR:")
 
             if self.gen_for_windows:
                 self.append(cmds, "mkdir {} 2>nul".format(cwd))
             else:
-                # echo commands as they are executed
-                self.append(cmds, "mkdir {} -p".format(cwd))
+                self.append(cmds, "mkdir -p {}".format(cwd))
+
+            if copy_code:
+                self.append(cmds, "cp __* {}".format(cwd))
+                # self.append(cmds, "cp -r code {}".format(cwd))
+                # self.append(cmds, "cp -r xtlib {}".format(cwd))
+
+                # unzip code
+                self.append_unzip(cmds, "xt_code.zip", cwd, echo=True)
 
             self.append(cmds, "cd {}".format(cwd))
             self.append(cmds, 'echo current dir: {}'.format("%cd%" if self.gen_for_windows else "$PWD"), echo=False)
@@ -525,7 +565,7 @@ class BackendBase(implements(BackendInterface)):
 
         self.append(cmds, cmd, echo=False)
 
-    def append_dir(self, cmds, path="."):
+    def append_dir(self, cmds, path=".", recursive=False):
         # limit to 30 lines
 
         display_path = path
@@ -539,7 +579,8 @@ class BackendBase(implements(BackendInterface)):
         else:
             # grep is used to supress the distracting "total" line
             # head is used to limit to 30 lines
-            cmd = "ls -lt {} | grep -vh '^total' | head -n 30".format(path)
+            opts = "-R " if recursive else ""
+            cmd = "ls -lt {}{} | grep -vh '^total' | head -n 30".format(opts, path)
 
         self.append(cmds, cmd, echo=True)
 
@@ -569,9 +610,6 @@ class BackendBase(implements(BackendInterface)):
     def add_info_cmds(self, cmds, args):
         self.append_title(cmds, "BEFORE environment:", echo=False)
 
-        # for debugging AML docker usage
-        #self.append(cmds, '''python -c "import torch; print('PyTorch: \t' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"''')
-
         target = args["target"]
         docker_name = args["docker"]
         docker_image, login_server, docker_registry = self.config.get_docker_info(target, docker_name, required=False)
@@ -599,7 +637,8 @@ class BackendBase(implements(BackendInterface)):
             self.add_info(cmds, "OS version", "%xt_tmp%")
             self.add_info(cmds, "Conda env", "$CONDA_DEFAULT_ENV", "%CONDA_DEFAULT_ENV%")
             self.add_info(cmds, "In docker", "$IN_DOCKER", "False")
-            self.add_info(cmds, "Docker img", docker_image)
+            self.add_info(cmds, "Image request", docker_image)
+            self.add_info(cmds, "Image name", "$DOCKER_IMAGE_NAME", "%DOCKER_IMAGE_NAME%")
             cmds.append("wmic path win32_VideoController get name | findstr -V Name | head -n 1 > __t__ && set /p xt_tmp= < __t__")
             self.add_info(cmds, "GPU type", "%xt_tmp%")
         else:
@@ -609,11 +648,20 @@ class BackendBase(implements(BackendInterface)):
             self.add_info(cmds, "Conda env", "$CONDA_DEFAULT_ENV", "%CONDA_DEFAULT_ENV%")
             self.append(cmds, "if [ -f /.dockerenv ]; then export IN_DOCKER=True; else export IN_DOCKER=False; fi")
             self.add_info(cmds, "In docker", "$IN_DOCKER", "False")
-            self.add_info(cmds, "Docker img", docker_image)
+            self.add_info(cmds, "Image request", docker_image)
+            self.add_info(cmds, "Image name", "$DOCKER_IMAGE_NAME", "%DOCKER_IMAGE_NAME%")
             self.add_info(cmds, "GPU type", "$(nvidia-smi -L | cut -d'(' -f1)")
 
         self.add_python_info(cmds)
 
+        # debug
+        #self.append(cmds, "which python")
+        #self.append(cmds, "pip show torch")
+
+        # TORCH version (before pip install)
+        target_cmd = '''python -c "import torch; print('PyTorch:'.ljust(15) + ' ' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"'''
+        self.append_package_test(cmds, "torch", target_cmd, "PyTorch")
+
         self.add_info(cmds, "running", "$(basename $0)", "%0")
         self.add_info(cmds, "current dir", "$(pwd)", "%CD%")
         self.add_info(cmds, "username", "$(whoami)", "%USERNAME%")
@@ -695,7 +743,7 @@ class BackendBase(implements(BackendInterface)):
             for cmd in pp:
                 self.append(cmds, cmd)
 
-    def set_gen_for_windows(self, for_windows: bool, args: [str]):
+    def set_gen_for_windows(self, for_windows: bool, args: List[str]):
         docker_cmd = args["docker_cmd"]
         docker_is_windows = args["docker_is_windows"]
 
@@ -711,7 +759,8 @@ class BackendBase(implements(BackendInterface)):
         store_model_dir, model_action, model_writable, storage_name, storage_key, actions, is_windows, 
         sudo_available=True, username=None, use_username=True, install_blobfuse=False, pre_setup_cmds=None,
         setup=None, post_setup_cmds=None, args=None, nonempty=False, use_allow_other=True, 
-        remove_zip=True, node_index="$1", run_name="$2", homebase="$HOME", cwd="$HOME/.xt/cwd"):
+        remove_zip=True, node_index="$1", run_name="$2", homebase="$HOME", cwd="$HOME/.xt/cwd", 
+        copy_code=False, mountbase=None, tmpbase=None):
         '''
         we need to run several commands to configure the target machine for our run, and then run the user's commands (user_parts).
         we do this by writing all the commands (config cmds and user_parts) to shell script or batch file that resides in the 
@@ -746,7 +795,7 @@ class BackendBase(implements(BackendInterface)):
         if pre_setup_cmds:
             cmds += pre_setup_cmds
 
-        self.add_first_cmds(cmds, homebase, cwd, args=args)
+        self.add_first_cmds(cmds, homebase, cwd, args=args, copy_code=copy_code)
 
         if self.get_service_name() == "pool" and docker_cmd:
             # for pool jobs running under docker, copy source code into docker CWD 
@@ -763,10 +812,17 @@ class BackendBase(implements(BackendInterface)):
         if username:
             self.append(cmds, "$export USER=" + username, expand=True)
 
+        if not mountbase:
+            mountbase = homebase
+
+        if not tmpbase:
+            tmpbase = homebase
+
         self.add_mount_cmds(cmds, sudo_available, storage_name, storage_key, 
             actions, data_action, model_action, store_data_dir, store_model_dir, data_writable, model_writable,
             install_blobfuse=install_blobfuse, use_username=use_username, 
-            use_allow_other=use_allow_other, nonempty=nonempty, homebase=homebase, args=args)
+            use_allow_other=use_allow_other, nonempty=nonempty, homebase=homebase, mountbase=mountbase, 
+            tmpbase=tmpbase, args=args)
 
         # add post setup commands now
         if post_setup_cmds:
@@ -788,7 +844,7 @@ class BackendBase(implements(BackendInterface)):
     def add_mount_cmds(self, cmds, sudo_available, storage_name, storage_key, 
         actions, data_action, model_action, store_data_dir, store_model_dir, data_writable=True, model_writable=False, 
         use_username=None, use_allow_other=None, nonempty=False, install_blobfuse=True, homebase="$HOME",
-        args=None):
+        mountbase="$HOME", tmpbase="$HOME", args=None):
 
         gen_for_windows = self.gen_for_windows
         workspace = args["workspace"]
@@ -796,21 +852,21 @@ class BackendBase(implements(BackendInterface)):
 
         # put mnt paths in user's home dir so sudo isn't needed to create/mount
         if self.mounting_enabled:
-            jobs_mount_dir = "$XT_HOMEBASE/.xt/mnt/jobs_container"
-            workspace_mount_dir = "$XT_HOMEBASE/.xt/mnt/workspace_container"
-            data_mount_dir = "$XT_HOMEBASE/.xt/mnt/data_container"
-            model_mount_dir = "$XT_HOMEBASE/.xt/mnt/models_container"
+            jobs_mount_dir = mountbase + "/.xt/mnt/jobs_container"
+            workspace_mount_dir = mountbase + "/.xt/mnt/workspace_container"
+            data_mount_dir = mountbase + "/.xt/mnt/data_container"
+            model_mount_dir = mountbase + "/.xt/mnt/models_container"
         else:
             if self.gen_for_windows:
-                jobs_mount_dir = "%XT_HOMEBASE%\\.xt\\local\\jobs_container"
-                workspace_mount_dir = "%XT_HOMEBASE%\\.xt\\local\\workspace_container"
-                data_mount_dir = "%XT_HOMEBASE%\\.xt\\local\\data_container"
-                model_mount_dir = "%XT_HOMEBASE%\\.xt\\local\\models_container"
+                jobs_mount_dir = mountbase + "\\.xt\\local\\jobs_container"
+                workspace_mount_dir = mountbase + "\\.xt\\local\\workspace_container"
+                data_mount_dir = mountbase + "\\.xt\\local\\data_container"
+                model_mount_dir = mountbase + "\\.xt\\local\\models_container"
             else:
-                jobs_mount_dir = "$XT_HOMEBASE/.xt/local/jobs_container"
-                workspace_mount_dir = "$XT_HOMEBASE/.xt/local/workspace_container"
-                data_mount_dir = "$XT_HOMEBASE/.xt/local/data_container"
-                model_mount_dir = "$XT_HOMEBASE/.xt/local/models_container"
+                jobs_mount_dir = mountbase + "/.xt/local/jobs_container"
+                workspace_mount_dir = mountbase + "/.xt/local/workspace_container"
+                data_mount_dir = mountbase + "/.xt/local/data_container"
+                model_mount_dir = mountbase + "/.xt/local/models_container"
 
         # emit cmds to MOUNT WORKSPACE and export/set releated environment variables
         if not gen_for_windows:
@@ -818,7 +874,8 @@ class BackendBase(implements(BackendInterface)):
             workspace = args["workspace"]
 
             if self.mounting_enabled:
-                if install_blobfuse:
+                user_install_blobfuse = args["install_blobfuse"]
+                if install_blobfuse and user_install_blobfuse:
                     self.append_install_blobfuse_cmds(cmds, sudo_available)
 
                 # mount workspace to /mnt/xt_workspace
@@ -826,7 +883,7 @@ class BackendBase(implements(BackendInterface)):
                 self.emit_mount_cmds(cmds, storage_name, storage_key, container=workspace, 
                     mnt_path=workspace_mount_dir, is_writable=True, install_blobfuse=False, 
                     sudo_available=sudo_available, use_username=use_username, use_allow_other=use_allow_other, 
-                    nonempty=nonempty, cleanup_needed=True)
+                    nonempty=nonempty, cleanup_needed=True, tmpbase=tmpbase, args=args)
 
             # on linux, always mount JOBS container
             jobs_container = store_utils.get_jobs_container(workspace)
@@ -838,23 +895,20 @@ class BackendBase(implements(BackendInterface)):
                 self.emit_mount_cmds(cmds, storage_name, storage_key, container=jobs_container, 
                     mnt_path=jobs_mount_dir, is_writable=True, install_blobfuse=False, 
                     sudo_available=sudo_available, use_username=use_username, use_allow_other=use_allow_other, 
-                    nonempty=nonempty, cleanup_needed=True)
+                    nonempty=nonempty, cleanup_needed=True, tmpbase=tmpbase, args=args)
 
             self.append_title(cmds, "EXPORT related environment variables:")
 
-            # XT_NODE_DIR
-            store_path = "{}/jobs/{}/node-{}".format(jobs_mount_dir, job_id, self.node_index)
-            self.append_export(cmds, "XT_NODE_DIR", store_path)
-
-        # on windows or linux, always define the output dir (even if it is not mounted)
+        # on windows or linux, always define XT_OUTPUT_DIR  (even if it is not mounted)
         # XT_OUTPUT_DIR
         run_path = store_utils.get_run_path(job_id, self.run_name) 
         store_path = "{}/{}/output".format(workspace_mount_dir, run_path)
-        self.append_export(cmds, "XT_OUTPUT_DIR", store_path)
-    
-        # create it so the user doesn't trip over missing dir
-        # surround dir with quotes so windows doesn't complain about forward slashes
-        self.append(cmds, 'mkdir "{}"'.format(store_path))
+        self.define_xt_dir(cmds, "XT_OUTPUT_DIR", store_path)
+
+        # on windows or linux, always define XT_NODE_DIR  (even if it is not mounted)
+        # XT_NODE_DIR
+        store_path = "{}/jobs/{}/nodes/node{}".format(jobs_mount_dir, job_id, self.node_index)
+        self.define_xt_dir(cmds, "XT_NODE_DIR", store_path)
 
         # emit cmds to MOUNT or DOWNLOAD data
         if "data" in actions:
@@ -862,7 +916,7 @@ class BackendBase(implements(BackendInterface)):
             self.process_action(cmds, data_action, data_mount_dir, store_utils.DATA_STORE_ROOT, store_data_dir, "XT_DATA_DIR",
                 data_writable, storage_name, storage_key, sudo_available=sudo_available, cleanup_needed=True, 
                 is_windows=gen_for_windows, use_username=use_username, install_blobfuse=False, nonempty=nonempty,
-                use_allow_other=use_allow_other, args=args)
+                use_allow_other=use_allow_other, tmpbase=tmpbase, args=args)
 
         # emit cmds to MOUNT or DOWNLOAD model
         if "model" in actions:
@@ -870,8 +924,23 @@ class BackendBase(implements(BackendInterface)):
             self.process_action(cmds, model_action, model_mount_dir, store_utils.MODELS_STORE_ROOT, store_model_dir, "XT_MODEL_DIR",
                 model_writable, storage_name, storage_key, sudo_available=sudo_available, cleanup_needed=True, 
                 is_windows=gen_for_windows, use_username=use_username, install_blobfuse=False, nonempty=nonempty,
-                use_allow_other=use_allow_other, args=args)
+                use_allow_other=use_allow_other, tmpbase=tmpbase, args=args)
 
+    def define_xt_dir(self, cmds, name, path):
+
+        # ensure dir is empty but present
+        if self.gen_for_windows:
+            path = file_utils.fix_slashes(path, False, protect_ws_run_name=False)
+            self.append_export(cmds, name, path, fix_value=False)
+            self.append(cmds, "rd /s /q {}".format(path))
+            self.append(cmds, 'mkdir "{}"'.format(path))
+        else:
+            path = file_utils.fix_slashes(path, True, protect_ws_run_name=False)
+            self.append_export(cmds, name, path)
+            self.append(cmds, "rm -rf {}".format(path))
+            self.append(cmds, 'mkdir -p "{}"'.format(path))
+
+        self.append_dir(cmds, path)
 
     def add_log_upload_cmds(self, cmds, args):
         '''
@@ -883,38 +952,87 @@ class BackendBase(implements(BackendInterface)):
         we cannot rely on blobfuse mapping, and we know we have xtlib installed.
         '''
 
-        # # debug (ensure azureml-logs is present on ITP runs)
-        # self.append_dir(cmds)
-        # self.append_dir(cmds, "logs/azureml")
-        # self.append(cmds, "cp -r logs  {}".format(constants.AFTER))
+        fn_stdout_aml = "azureml-logs/00_f5cca3f8f56b6f68d8499ecd4520d4a8-master-0_stdout.txt"
+        fn_stdout_aml = "azureml-logs/00_{}_stdout.txt"
+
+        # debug (ensure azureml-logs is present on ITP runs)
+        self.append(cmds, "cd $XT_ORIG_WORKDIR")
+        self.append_dir(cmds, "/var/log/compute", True)
+        #self.append_dir(cmds, "logs", True)
+
+        # these are empty for ITP jobs
+        #self.append_dir(cmds, "$AZUREML_RUN_CONFIGURATION")
+        #self.append_dir(cmds, "./azureml-logs", True)
+        #self.append_dir(cmds, "./outputs", True)
 
         self.append_title(cmds, "UPLOAD service and XT logs to job storage:")
 
+        # workaround for "$jobs" not resolving correctly on compute node
         job_id = args["job_id"]
         workspace = args["workspace"]
-
-        # workaround for "$jobs" not resolving correctly on compute node
-        #store_path = "$jobs/{}/node-{}/after".format(job_id, self.node_index)
         jobs_container = store_utils.get_jobs_container(workspace)
-        store_path = "/{}/jobs/{}/node-{}/after".format(jobs_container, job_id, self.node_index)
+        store_after = "/{}/jobs/{}/nodes/node{}/after".format(jobs_container, job_id, self.node_index)
 
         if self.gen_for_windows:
-            # have backend add its needed cmds to copy the service logs
-            self.add_service_log_copy_cmds(cmds, constants.WINDOWS_SERVICE_LOGS, store_path, args)         
 
-            # upload both XT logs and service logs to job after dir
-            # specify workspace since our config file isn't present on the compute node
-            self.append(cmds, "xt upload {} {} --feedback=0".format(constants.AFTER, store_path))
+            # copy from __after__/xt_logs to output node dir
+            dest = "%XT_NODE_DIR%\\after\\xt_logs"
+            self.append(cmds, "xcopy {} {}\\".format(constants.WINDOWS_XT_LOGS, dest))
+
+            # copy node run_errors directory to output node dir
+            dest = "%XT_NODE_DIR%\\after\\run_errors"
+            self.append(cmds, "xcopy {} {}\\ /s".format(run_errors.WINDOWS_RUN_ERRORS_DIR, dest))
+
+            # have backend add its needed cmds to copy the service logs
+            dest = "%XT_NODE_DIR%\\after\\service_logs"
+            self.add_service_log_copy_cmds(cmds, dest, args)         
+
+            if not self.mounting_enabled:
+                # specify workspace since our config file isn't present on the compute node
+                src_after = "%XT_NODE_DIR%\\after"
+                self.append(cmds, "xt upload {} {} --feedback=0".format(src_after, store_after))
 
         else:
-            # copy xt_logs to output node dir
+            # copy from __after__/xt_logs to output node dir
             dest = "$XT_NODE_DIR/after/xt_logs"
             self.append(cmds, "mkdir -p {} && cp {}/* {}".format(dest, constants.LINUX_XT_LOGS, dest))
 
+            # copy node run_errors directory to output node dir (but avoid err msg if no files)
+            dest = "$XT_NODE_DIR/after/run_errors"
+            self.append(cmds, "find {}/* > /dev/null 2>&1 && mkdir -p {} && cp -r {}/* {}".\
+                format(run_errors.LINUX_RUN_ERRORS_DIR, dest, run_errors.LINUX_RUN_ERRORS_DIR, dest))
+
             # have backend add its needed cmds to copy the service logs
-            logs_dir = "$XT_NODE_DIR/after/service_logs"
-            self.add_service_log_copy_cmds(cmds, logs_dir, store_path, args)         
+            dest = "$XT_NODE_DIR/after/service_logs"
+            self.add_service_log_copy_cmds(cmds, dest, args)         
+
+            if not self.mounting_enabled:
+                # if blobfuse is unavailable, we manually upload the logs using XT (assumes XT has been successfully loaded)
+                src_after = "$XT_NODE_DIR/after"
+                cmd = "xt upload {} {} --feedback=0".format(src_after, store_after)
+                self.append(cmds, cmd)
         
+    def export_now_to_var(self, cmds, var_name):
+        '''
+        export current date/time in an arrow-compatible format, including timezone, to specified var_name.
+        '''
+
+        # don't use arrow (early setup - it may not be installed)
+        #now_cmd = 'python -c "import arrow; print(arrow.now())"'
+
+        # don't use datetime (early python versions may not support astimezone() with no arg)
+        #now_cmd = 'python -c "import datetime; print(datetime.datetime.now().astimezone())"'
+
+        # time gives us pretty much what we need (except for fractions of a second)
+        now_cmd = '''python -c "import time; print(time.strftime('%Y-%m-%d %H:%M:%S.0%z', time.localtime()))"'''
+
+        if self.gen_for_windows:
+            now_cmd = now_cmd.replace("%", "%%")      # windows will strip first "%" away
+            cmd = "{} > __t__ && set /p {}= < __t__".format(now_cmd, var_name)
+            self.append(cmds, cmd)   
+        else:
+            self.append_export(cmds, var_name, '$({})'.format(now_cmd), fix_value=False)
+
     def create_wrapper_and_inner(self, cmds, user_cmd, args):
 
         # debug
@@ -924,10 +1042,16 @@ class BackendBase(implements(BackendInterface)):
 
         self.append(cmds, user_cmd)
 
+        self.export_now_to_var(cmds, "XT_POST_START_TIME")
+
         # append cmd(s) to download service and xt logs
         bootstrap_dir = args["bootstrap_dir"]
         self.add_log_upload_cmds(cmds, args)
 
+        # log the POST end_time and duration
+        cmd = 'python -c "from xtlib import node_post_wrapup; node_post_wrapup.main()" '
+        self.append(cmds, cmd)
+
         self.append_title(cmds, 'END of XT-level processing', double=True)
 
         # remove empty cmds
@@ -1037,6 +1161,12 @@ class BackendBase(implements(BackendInterface)):
     def add_setup_cmds(self, cmds, remove_zip=True, 
             gen_for_windows=False, args=None):
 
+        pre_cmds = args["pre_cmds"]
+        if pre_cmds:
+            self.append_title(cmds, "USER pre-setup commands:")
+            for cmd in pre_cmds:
+                self.append(cmds, cmd)
+
         # KISS: just insist that user specify PYTHONPATH in SETUP (not env vars)
         preserve_python_path=False
 
@@ -1093,18 +1223,16 @@ class BackendBase(implements(BackendInterface)):
         pp = args["python_path"]     # a list of strings from SETUP in config file
 
         if not pp:
-            # by default, we add a "." to path so xtlib and other libraries usually just work
-            self.append_export(cmds, "PYTHONPATH", ".")
+            # by default, we add XT_CWD to path so xtlib and other libraries usually just work
+            self.append_export(cmds, "PYTHONPATH", "$XT_CWD")
         else:
             # user has specified the PYTHONPATH; give them complete control
             pp_str = ":".join(pp)
-            path = file_utils.fix_slashes(pp_str, is_linux=not self.gen_for_windows)
+            path = file_utils.fix_slashes(pp_str, is_linux=not self.gen_for_windows, protect_ws_run_name=False)
             self.append_export(cmds, "PYTHONPATH", path)
 
         self.append_python_path(cmds)
 
-        #self.append(cmds, "xt --version")
-
     # API call
     def get_node_status(self, service_node_info):
         pass
@@ -1148,5 +1276,5 @@ class BackendBase(implements(BackendInterface)):
         return result
 
     # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
         pass        
\ No newline at end of file
