@@ -7,10 +7,12 @@
 import os
 import io
 import sys
+import math
 import time
 import datetime
 import numpy as np
 from interface import implements
+from threading import Lock
 
 from xtlib import utils
 from xtlib import errors
@@ -32,6 +34,7 @@ from .backend_base import BackendBase
 batch = None   
 azureblob = None
 azuremodels = None
+batchmodels = None
 
 class AzureBatch(BackendBase):
     ''' 
@@ -60,7 +63,7 @@ class AzureBatch(BackendBase):
         # CAUTION: "batchmodels" is NOT the same as batch.models
 
         if not compute_def:
-            compute_def = config.get_compute_def(compute)
+            compute_def = config.get_target_def(compute)
 
         self.compute = compute
         self.compute_def = compute_def
@@ -145,8 +148,8 @@ class AzureBatch(BackendBase):
                 # recompose this command to pass along all script arguments
                 wrapped_parts = ['/bin/bash', '--login', '{} $*'.format(os.path.basename(fn_wrapped))]
     
-                # create an STDBOTH.txt that merged stdout and stderr
-                node_cmd = "{} > ../stdboth.txt 2>&1".format(" ".join(wrapped_parts))
+                # create a STDBOTH.tmp that merged stdout and stderr
+                node_cmd = "{} > ../stdboth.tmp 2>&1 && cp ../stdboth.tmp ../stdboth.txt".format(" ".join(wrapped_parts))
 
                 # create a helper file to call our node cmd
                 fn_helper = snapshot_dir + "/" + constants.FN_BATCH_HELPER
@@ -214,9 +217,15 @@ class AzureBatch(BackendBase):
 
         actions = ["data", "model"]
 
-        self.add_mount_cmds(cmds, False, storage_name, storage_key, 
+        # try to install blobfuse, but it may already be installed on some images
+        install_blobfuse = True
+
+        # sudo is available, but causes problems with subsequent MNT directory access
+        sudo_available = False     
+
+        self.add_mount_cmds(cmds, sudo_available, storage_name, storage_key, 
             actions, data_action, model_action, store_data_dir, store_model_dir, data_writable, model_writable,
-            use_username=False, use_allow_other=False, args=args)
+            use_username=False, use_allow_other=False, install_blobfuse=install_blobfuse, args=args)
 
         fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, args)
         return fn_wrapped
@@ -317,37 +326,65 @@ class AzureBatch(BackendBase):
         node_records = []
         service_info_by_node = {}
 
-        for n, run_data_list in enumerate(job_runs):    # run_data_list_by_node):
-
-            node_record = self.prep_node(job_id, n, run_data_list, num_nodes, use_low_pri, 
-                using_hp=using_hp, exper_name=experiment, static_run_cmds=static_run_cmds, args=args)
-
-            node_records.append(node_record)
-
-            if not static_run_cmds:
-                first_run = run_data_list[0]
-                run_name = first_run["run_name"]
-                fb.feedback("{}/{}".format(workspace, run_name))
-
-            node_id = "node" + str(n)
-            batch_job_id = self.make_batch_job_id(ws_name, job_id)
-
-            service_info_by_node[node_id] = {"workspace": ws_name, "job_id": job_id, "batch_job_id": batch_job_id, 
-                "node_id": node_id, "task_index": n, "merge_batch_logs": merge_batch_logs}
+        self.prep_all_nodes(job_id, job_runs, num_nodes, use_low_pri, using_hp, experiment, 
+            static_run_cmds, node_records, workspace, service_info_by_node, merge_batch_logs, args)
 
         # "hold" is now used to hold open the xt controller task AND the created pool
         auto_pool = not args["hold"]
 
+        fb.feedback("submitting job to batch")
+
         # finally, launch the job on AZURE BATCH
         pool_id = self.launch(job_id, node_records, auto_pool=auto_pool, ws_name=ws_name, vm_size=vm_size, 
             vm_image=vm_image, num_nodes=num_nodes, use_low_pri=use_low_pri, is_distributed=is_distributed, args=args)
 
-        fb.feedback("job submitted", is_final=True)
-
         service_run_info = {"batch_job_id": self.batch_job_id, "pool_id": pool_id}
 
         return service_run_info, service_info_by_node    
 
+    def prep_all_nodes(self, job_id, job_runs_list, num_nodes, use_low_pri, using_hp, exper_name, 
+        static_run_cmds, node_records, workspace, service_info_by_node, merge_batch_logs, args):
+
+        # create each run on a worker thread
+        next_progress_num = 1
+        job_count = len(job_runs_list)
+        thread_lock = Lock()
+
+        def thread_worker(job_runs, job_id, num_nodes, use_low_pri, 
+                using_hp, exper_name, static_run_cmds, args):
+
+            # process a subset of all jobs on this thread
+            for run_data_list in job_runs:
+                
+                node_index = run_data_list[0]["node_index"]
+
+                node_record = self.prep_node(job_id, node_index, run_data_list, num_nodes, use_low_pri, 
+                    using_hp=using_hp, exper_name=exper_name, static_run_cmds=static_run_cmds, args=args)
+
+                with thread_lock:
+                    nonlocal next_progress_num
+
+                    node_records.append(node_record)
+
+                    node_msg = "building nodes: {}/{}".format(next_progress_num, job_count)
+
+                    fb.feedback(node_msg, id="node_msg")  # , add_seperator=is_last)
+                    next_progress_num += 1
+
+                    node_id = "node" + str(node_index)
+                    batch_job_id = self.make_batch_job_id(workspace, job_id)
+
+                    service_info_by_node[node_id] = {"workspace": workspace, "job_id": job_id, "batch_job_id": batch_job_id, 
+                        "node_id": node_id, "task_index": node_index, "merge_batch_logs": merge_batch_logs}
+
+        # create each Batch Task on a worker thread
+        max_run_workers = args["max_workers"]
+        utils.run_on_threads(thread_worker, job_runs_list, max_run_workers, [job_id, num_nodes, use_low_pri, 
+                using_hp, exper_name, static_run_cmds, args])
+
+        # sort node_records to normal order
+        node_records.sort(key=lambda r: r["node_index"])
+
     def prep_node(self, job_id, node_index, run_data_list, num_nodes, use_low_pri, using_hp,  
             exper_name, static_run_cmds, args):
         '''prepare runs for node node_index'''
@@ -363,7 +400,7 @@ class AzureBatch(BackendBase):
 
         # build RESOURCE FILES for each file we have uploaded into "before"  directory
         before_path = "before/code"      # job store
-        after_path = "node-{}/after/service_logs".format(node_index)         # run store
+        after_path = "nodes/node{}/after/service_logs".format(node_index)         # run store
 
         # use blob service from store 
         store_provider = self.store.helper.provider
@@ -399,17 +436,18 @@ class AzureBatch(BackendBase):
         output_file_list = ["controller.log", "../*.txt"]
 
         after_blob_path = constants.JOBS_DIR + "/" + job_id + "/" + after_path
-        node_output_files = self.build_output_files(workspace, after_blob_path, output_file_list)
+        job_container = workspace if store_utils.STORAGE_FORMAT == "2" else constants.INFO_CONTAINER_V1
+        node_output_files = self.build_output_files(job_container, after_blob_path, output_file_list)
 
-        if static_run_cmds:
-            fb.feedback("  adding node: {}, run(s): {}".format(box_name.upper(), run_names.upper()), is_final=True)
+        # if static_run_cmds:
+        #     fb.feedback("  adding node: {}, run(s): {}".format(box_name.upper(), run_names.upper()), is_final=True)
 
         node_specs = first_run["run_specs"]
         box_secret = first_run["box_secret"]
         node_cmd = " ".join(node_specs["cmd_parts"])
 
         return {"node_cmd": node_cmd, "node_res_files": node_res_files, "node_output_files": node_output_files, 
-            "box_secret": box_secret}
+            "box_secret": box_secret, "node_index": node_index}
 
     def launch(self, job_id, node_records, auto_pool=True, description=None, ws_name=None, 
             vm_size=None, vm_image=None, num_nodes=1, use_low_pri=True, is_distributed=False, args=None):
@@ -477,6 +515,7 @@ class AzureBatch(BackendBase):
 
     def get_external_port(self, port_name, node):
         port = None
+        ip_addr = None
 
         for ep in node.endpoint_configuration.inbound_endpoints:
             if ep.name.startswith(port_name):
@@ -1197,6 +1236,7 @@ class AzureBatch(BackendBase):
         job_id, service_name, node_index = box_name.split("-")
         node_index = int(node_index)
         cancelled_by_controller = False
+        cancel_results = None
 
         # is the controller available?
         azure_batch_state, ip_addr, controller_port, tensorboard_port = \
@@ -1221,7 +1261,7 @@ class AzureBatch(BackendBase):
 
                 cs, box_secret = run_helper.get_client_cs(self.core, workspace, first_run_name)
                 if not cs:
-                    console.print("could not find info for run: {}/{}".format(actual_ws, first_run_name))
+                    console.print("could not find info for run: {}/{}".format(workspace, first_run_name))
                 else:
                     with XTClient(self.config, cs, box_secret) as xtc:
                         if xtc.connect():
@@ -1315,7 +1355,7 @@ class AzureBatch(BackendBase):
 
         if job_ids:
             # get runs_by_job data from MONGO
-            mongo = self.store.get_mongo()
+            db = self.store.get_database()
 
             filter_dict = {}
             filter_dict["job_id"] = {"$in": job_ids}
@@ -1323,7 +1363,7 @@ class AzureBatch(BackendBase):
 
             fields_dict = {"runs_by_box": 1}
 
-            job_records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
+            job_records = db.get_info_for_jobs(ws_name, filter_dict, fields_dict)
             console.diag("after get_info_for_jobs()")
 
             # cancel each ACTIVE job
@@ -1356,99 +1396,6 @@ class AzureBatch(BackendBase):
         task = self.get_node_task(service_node_info)
         return task.state.value
 
-   # BACKEND API
-    def log_file_iterator(self, service_node_info, log_name, start_offset=0, end_offset=None, 
-        encoding='utf-8', use_best_log=True):
-        '''
-        returns an iterator for the specified log file.
-        '''
-        job_id = service_node_info["job_id"]
-        merge_batch_logs = utils.safe_value(service_node_info, "merge_batch_logs")
-        node_id = service_node_info["node_id"]
-
-        new_text = None
-        batch_status = None
-        simple_status = None
-        got_text = False
-
-        if log_name is None or log_name in ["primary", "stdout", "stdout.txt"]:
-            file_path = "stdboth.txt" if merge_batch_logs else "stdout.txt"
-        elif log_name in ["secondary", "error", "stderr", "stderr.txt"]:
-            file_path = "stderr.txt"
-        else:
-            file_path = log_name
-
-        if start_offset == 0:
-            # try to first read from job storage (if task has completed)
-            node_index = utils.node_index(node_id)
-            job_path = "node-{}/after/service_logs/{}".format(node_index, file_path)
-            if self.store.does_job_file_exist(ws_name, job_id, job_path):
-                console.print("==> reading log from job storage...")
-
-                new_text = self.store.read_job_file(workspace, job_id, job_path)
-                batch_status = "completed"
-                simple_status = "completed"
-                got_text = True
-
-                result = {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, 
-                    "service_status": batch_status, "file_path": file_path}
-
-                # here we just yield a single result
-                yield result                
-
-        if not got_text:
-            # this call can take up to 15 secs
-            task_index = service_node_info["task_index"]
-            task_id = "task" + str(task_index)
-            task = None     # delay creation until needed
-
-            if not self.batch_client:
-                self.create_batch_client()
-
-            next_offset = start_offset
-            if not end_offset:
-                end_offset = 1024*1024*1024*16     # 16 GB should be big enough for a log file
-
-            while True:
-                try:
-                    gft_opts = batchmodels.FileGetFromTaskOptions(ocp_range='bytes={}-{}'.format(next_offset, end_offset))
-
-                    # read currently available data
-                    stream = self.batch_client.file.get_from_task(job_id, task_id, file_path=file_path, 
-                        file_get_from_task_options=gft_opts)
-
-                    output = io.BytesIO()
-                    for data in stream:
-                        output.write(data)
-
-                    new_bytes = output.getvalue()
-                    new_text = new_bytes.decode(encoding)
-                    next_offset += len(new_bytes)
-
-                    # if we are streaming output, assume we are running
-                    batch_status == "running"
-                    simple_status == "running"
-
-                except (batchmodels.BatchErrorException) as ex:
-                    # interpret this error as task has terminated
-                    console.diag("Exception reading batch log file: {}".format(ex))
-                    next_text = ""
-
-                if not new_text:
-                    # is task done?  update status 
-                    # CAUTION: this is a 15-30 second call
-                    if task is None:
-                        console.print("==> creating azure batch client...")
-                    task = self.get_node_task(service_node_info)
-                    batch_status = task.state.value
-                    simple_status = self.get_simple_status(batch_status)
-
-                result = {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, 
-                    "service_status": batch_status, "file_path": file_path}
-
-                # we never stop yielding; its up to caller to examine status and stop iteration
-                yield result
-
     # BACKEND API
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
         encoding='utf-8', use_best_log=True, service_context=None):
@@ -1476,19 +1423,20 @@ class AzureBatch(BackendBase):
         # try to first read from job storage (if task has completed)
         node_index = utils.node_index(node_id)
 
-        job_path = "node-{}/after/service_logs/{}".format(node_index, file_path)
+        job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
         if self.store.does_job_file_exist(workspace, job_id, job_path):
             new_text = self.store.read_job_file(workspace, job_id, job_path)
             batch_status = "completed"
             simple_status = "completed"
             found_file = True
 
-        job_path = "node-{}/after/xt_logs/{}".format(node_index, file_path)
-        if self.store.does_job_file_exist(workspace, job_id, job_path):
-            new_text = self.store.read_job_file(workspace, job_id, job_path)
-            batch_status = "completed"
-            simple_status = "completed"
-            found_file = True
+        if not found_file:
+            job_path = "nodes/node{}/after/xt_logs/{}".format(node_index, file_path)
+            if self.store.does_job_file_exist(workspace, job_id, job_path):
+                new_text = self.store.read_job_file(workspace, job_id, job_path)
+                batch_status = "completed"
+                simple_status = "completed"
+                found_file = True
 
         if not found_file:
 
@@ -1506,7 +1454,10 @@ class AzureBatch(BackendBase):
             gft_opts = batchmodels.FileGetFromTaskOptions(ocp_range='bytes={}-{}'.format(start_offset, end_offset))
 
             try:
-                stream = self.batch_client.file.get_from_task(batch_job_id, task.id, file_path=log_name, 
+                # stdboth is a .TMP file when it is read live
+                live_log_name = "stdboth.tmp" if log_name == "stdboth.txt" else log_name
+
+                stream = self.batch_client.file.get_from_task(batch_job_id, task.id, file_path=live_log_name, 
                     file_get_from_task_options=gft_opts)
 
                 # put streamed output into "output"
@@ -1579,7 +1530,7 @@ class AzureBatch(BackendBase):
         return result
 
     # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
         # this is done by the batch service at the end of our batch task
-        pass
+        pass            
 
