@@ -5,6 +5,7 @@
 # backend_itp.py: support for running jobs under the Microsoft ITP (Integrated Training Platform)
 
 import os
+import uuid
 from xtlib.backends.backend_aml import AzureML
 
 class ITP(AzureML):
@@ -12,8 +13,9 @@ class ITP(AzureML):
     def __init__(self, compute, compute_def, core, config, username=None, arg_dict=None, disable_warnings=True):
         super(ITP, self).__init__(compute, compute_def, core, config, username, arg_dict, disable_warnings)
 
-        # still busted, as of 9/2/2020
-        self.mounting_enabled = False
+        # blobfuse is still busted if we are using their default docker images
+        # for now, let's assume we are using latest good pytorch-xtlib docker image
+        self.mounting_enabled = True   # False
 
     def get_name(self):
         return "itp"
@@ -41,15 +43,12 @@ class ITP(AzureML):
         estimator.run_config.cmk8scompute = k8sconfig
 
   # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
 
         self.append(cmds, "mkdir -p {}".format(dest_dir))
-        self.append(cmds, "cp -r -v azureml-logs/* {}".format(dest_dir))
-
-        # currently, blobfuse is busted for ITP, so we manually upload the logs
-        workspace = args["workspace"]
-        cmd = "xt upload {} {} --feedback=0".format(dest_dir, store_path)
-        self.append(cmds, cmd)
+        self.append(cmds, "cp -r -v /var/log/compute/00_stdout.txt {}".format(dest_dir))
+        #self.append(cmds, "cp -r -v /var/log {}".format(dest_dir))
+        self.append(cmds, "cp -r -v $XT_ORIG_WORKDIR/logs {}".format(dest_dir))
 
     def create_wrapper_script(self, cmd_parts, snapshot_dir, store_data_dir, data_action, 
             data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
@@ -65,27 +64,56 @@ class ITP(AzureML):
         self.build_other_cmds(other_cmds)
         args["other_cmds"] = other_cmds
 
+        # we need to move our cwd (default working dir is blob-mapped and slow)
+        # as of 10/21/2020, ITP recommends we use "/var/tmp" until they disable the NFS mount for $HOME
+        use_var_tmp = True
+
+        if use_var_tmp:
+            job_home = "/var/tmp"
+        else:
+            guid = str(uuid.uuid4())
+            job_home = "$HOME/{}".format(guid)
+        
+        cwd = "{}/.xt/cwd".format(job_home)
+
+        # see if this helps our new blobfuse problem on ITP
+        mountbase = "$XT_ORIG_WORKDIR"
+        tmpbase = job_home
+
         # we only do this once (for the first box/job)
         fn_wrapped = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
             data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
             is_windows=False, sudo_available=sudo_available, username=username, use_username=False, 
             install_blobfuse=True, setup=setup, use_allow_other=use_allow_other, 
-            remove_zip=False, homebase=".", cwd=".", args=args)
+            remove_zip=False, homebase=job_home, cwd=cwd, args=args, copy_code=True, mountbase=mountbase, 
+            tmpbase=tmpbase)
 
         return fn_wrapped
 
     def build_other_cmds(self, cmds):
         self.append_title(cmds, "CORRECT python path used for XT on ITP")
 
-        # following code removes any lines containing "azureml-envs" (lines at top of file)
+        # following code removes any lines containing "/opt/conda/" (lines at top of file)
         # and adds a new line (#!path-to-python) to the xt file 
         # since ITP installs in incorrectly thru conda
-        self.append(cmds, "xt_file=$(which xt)")
+
+        # try as is
+        self.append(cmds, "xt --version")
 
         # show the problem
+        self.append(cmds, "xt_file=$(which xt)")
         self.append(cmds, "echo xt script: $xt_file")
-        self.append(cmds, "cat $xt_file")
+        self.append(cmds, "head -2 $xt_file")
 
-        # fix the problem
-        self.append(cmds, "{ echo '#!'$(which python); grep -v azureml-envs  $xt_file; } > tmp")
-        self.append(cmds, "sudo mv tmp $xt_file")
+        # # fix the problem
+        # self.append(cmds, "{ echo '#!'$(which python); grep -v /opt/conda/  $xt_file; } > tmp")
+        # self.append(cmds, "sudo mv tmp $xt_file")
+
+        # # show the fix
+        # self.append(cmds, "head -2 $xt_file")
+
+        # # test the fix
+        # self.append(cmds, "xt --version")
+        # self.append(cmds, "sudo chmod +x $xt_file")
+        # self.append(cmds, "xt --version")
+        # self.append(cmds, "python $xt_file --version")
