@@ -73,7 +73,7 @@ class Philly(BackendBase):
         job_id = service_node_info["job_id"]
         ws_name = service_node_info["ws_name"]
         filter = {"_id": job_id}
-        jobs = self.core.store.mongo.get_info_for_jobs(ws_name, filter, None)
+        jobs = self.core.store.database.get_info_for_jobs(ws_name, filter, None)
         if not jobs:
             errors.store_error("unknown job_id: {}", job_id)
 
@@ -229,7 +229,7 @@ class Philly(BackendBase):
 
         # create environment variables for user
         job_id = args["job_id"]
-        node_store_path = "jobs/{}/node-{}".format(job_id, self.node_index)
+        node_store_path = "jobs/{}/nodes/node{}".format(job_id, self.node_index)
         code_store_path = "jobs/{}/before/code".format(job_id, self.node_index)
 
         self.append(cmds, 'export XT_OUTPUT_DIR={}/{}'.format(WORKSPACE_MOUNT_PATH, store_output_dir))
@@ -270,8 +270,15 @@ class Philly(BackendBase):
 
         self.append_dir(cmds)
         self.append(cmds, user_cmd)
+
+        self.export_now_to_var(cmds, "XT_POST_START_TIME")
+
         self.add_log_upload_cmds(cmds, args)
 
+        # log the POST end_time and duration
+        cmd = 'python -c "from xtlib import node_post_wrapup; node_post_wrapup.main()" '
+        self.append(cmds, cmd)
+
         self.append_title(cmds, 'END of XT-level processing', double=True)
         self.append_title(cmds, 'START of XT-level processing', double=True, index=0)
 
@@ -341,7 +348,7 @@ class Philly(BackendBase):
         vc = utils.safe_value(compute_def, "vc")
         self.username = args["username"]
 
-        fb.feedback("submitting to philly.{}.{}: ".format(cluster, vc), add_seperator=False)
+        fb.feedback("submitting to philly.{}.{}".format(cluster, vc), add_seperator=False)
 
         for i, node_runs in enumerate(job_runs):
             node_id = "node" + str(i)
@@ -356,7 +363,7 @@ class Philly(BackendBase):
         for fn in tmp_files:
             os.remove(fn)
 
-        fb.feedback("done", is_final=True)
+        fb.feedback("")
         
         monitor_url = "https://philly/#/jobSummary/{}/{}/{}".format(self.cluster, self.vc, self.username)
         service_job_info = {"monitor_url": monitor_url}
@@ -422,6 +429,7 @@ class Philly(BackendBase):
 
         wrapped_cmds = []
         run_names = []
+        run_name = None
 
         for run_data in node_runs:
             # run_data = {"run_name": run_name, "run_specs": run_specs, "box_name": box_name, "box_info": box_info, "repeat": repeat_count}
@@ -533,17 +541,17 @@ class Philly(BackendBase):
                 errors.service_error("Unexpected result from Philly job submit: " + str(result))
             service_job_id = result["jobId"]
 
-            mongo = self.store.get_mongo()
+            db = self.store.get_database()
 
             for run_name in run_names:
                 # update mongo db info for run with cluster and service_job_id
 
                 # add newly acquired info (service_run_id)
                 rd = {"cluster": cluster, "vc": vc, "service_run_id": service_job_id}
-                mongo.update_run_info(workspace, run_name, rd, update_primary=True)
+                db.update_run_info(workspace, run_name, rd, update_primary=True)
 
                 #fb.feedback("{}/{}".format(workspace, run_name), id="run_name", add_seperator=False)
-                fb.feedback("{}/{}".format(workspace, run_name))
+                #fb.feedback("{}/{}".format(workspace, run_name))
                 #console.print("Philly job id:", service_job_id)
 
         # add service_job_id to runs_by_box entries for this node
@@ -830,7 +838,7 @@ class Philly(BackendBase):
 
             try:
                 result = self.run_curl_cmd(curl_cmd, parse_result=True)
-            except:
+            except BaseException as ex:
                 console.print("Exception in read_log_file: run_curl_cmd ex=" + str(ex))
                 philly_status = "unknown"
                 result = ""
@@ -853,7 +861,7 @@ class Philly(BackendBase):
         # translates an Philly status to a simple status (queued, running, completed)
 
         queued = ["Queued"]
-        running = ["Running", "unknown"]
+        running = ["active", "Running", "unknown"]
         completed = ["Pass", "Failed", "Killed"]
 
         if status in queued:
@@ -922,7 +930,7 @@ class Philly(BackendBase):
         file_utils.write_text_file(fn_result, result)
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/logs/1")
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/stdout/1")
         # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/amlogs/1")
@@ -932,9 +940,65 @@ class Philly(BackendBase):
         self.append(cmds, "echo wait 30 secs for our STDOUT to flush to file", echo=False)
         self.append(cmds, "sleep {}".format(sleep_secs))
 
-
         self.append(cmds, "tail $PHILLY_JOB_DIRECTORY/stdout/1/stdout.txt")
 
         cmd = "cp -r $PHILLY_JOB_DIRECTORY/stdout/* {}".format(dest_dir)
         self.append(cmds, cmd)
 
+    # NEW API call
+    def get_queue_jobs(self, target_def, username, queues, workspace):
+
+        cluster = utils.safe_value(target_def, "cluster")
+        vc = utils.safe_value(target_def, "vc")
+        max_finished = 100
+
+        if not username:
+            username = self.username
+
+        results_by_queue = {}
+        if not queues:
+            queues = ["queued", "running", "completed"]
+
+        result = self.list_jobs(cluster, vc, username=username, status=None, max_finished=max_finished)
+
+        columns = ["job_id", "ws_name", "run_name", "appID", "status", "vc", "cluster", "elapsedTime", "username", "gpus", "retries", "appID"]
+
+        for queue in queues:
+
+            if queue == "completed":
+                status = "finishedJobs"
+            else:
+                status = queue + "Jobs"
+
+            jobs = result[status]
+
+            for job in jobs:
+                name = job["name"]
+                job_id = name.split("/")[-3]
+                ws_name = name.split("/")[-4]
+                run_name = name.split("/")[-2]
+
+                if workspace and workspace != ws_name:
+                    continue
+
+                job["job_id"] = job_id
+                job["ws_name"] = ws_name
+                job["run_name"] = run_name
+                job["cluster"] = cluster
+
+            results_by_queue[queue] = jobs
+
+        return results_by_queue, columns
+
+        # title = "target={} jobs:".format(self.compute)
+
+    def cancel_job_from_service_dict(self, job, workspace):
+
+        cluster = job["cluster"]
+        vc = job["vc"]
+        run_name = job["run_name"]
+        app_id = job["appID"]
+
+        cancel_result = self.abort_run(workspace, cluster, vc, run_name, app_id)
+        return cancel_result
+
