@@ -138,9 +138,9 @@ class PoolBackend(BackendBase):
                 install_blobfuse = bool(docker_cmd)
 
                 if self.gen_for_windows:
-                    node_index = "%1%"
-                    run_name = "%2%"
-                    homebase = os.getenv("HOMEDRIVE")
+                    node_index = "%1"
+                    run_name = "%2"
+                    homebase = os.getenv("HOMEDRIVE") + "\\."
                     cwd = homebase + "\\.xt\\cwd"
                 else:
                     node_index = "$1"
@@ -227,7 +227,7 @@ class PoolBackend(BackendBase):
         service_node_info = {"fn_entry": fn_entry, "box_addr": box_addr, "box_os": box_os, 
             "box_name": box_name, "job_id": job_id, "run_name": run_name}
 
-        fb.feedback("submitted", is_final=True) 
+        fb.feedback("submitted") 
 
         return service_node_info
 
@@ -290,7 +290,7 @@ class PoolBackend(BackendBase):
                 self.client.change_box(box_name)
 
                 if run_name in ["tensorboard", "mirror"]:
-                    text += self.get_box_process_status_inner(box_name, wworkspaces, run_name)
+                    text += self.get_box_process_status_inner(box_name, workspace, run_name)
                 else:
                     text += self.core.get_box_run_status_inner(box_name, workspace, run_name, stage_flags)
 
@@ -414,6 +414,7 @@ class PoolBackend(BackendBase):
         #utils.report_elapsed(start, "read log")
 
         new_text = new_bytes.decode(encoding)
+        service_status = None
 
         if new_text:
             service_status = "running"
@@ -498,7 +499,7 @@ class PoolBackend(BackendBase):
 
     def get_active_jobs(self, ws_name):
         ''' return a list of job_id's running on this instance of Azure Batch '''
-        mongo = self.client.store.get_mongo()
+        db = self.client.store.get_database()
 
         filter_dict = {
             "username": self.username,
@@ -511,7 +512,7 @@ class PoolBackend(BackendBase):
 
         fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
 
-        job_records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
+        job_records = db.get_info_for_jobs(ws_name, filter_dict, fields_dict)
 
         return job_records
 
@@ -519,15 +520,15 @@ class PoolBackend(BackendBase):
 
         # 3rd argument to script is the path of entry's log file
         if self.gen_for_windows:
-            cmd = "cp -v %3% {}".format(output_dir)
+            cmd = "cp -v %3 {}".format(output_dir)
         else:
             cmd = "cp -v $3 {}".format(output_dir)
         return cmd
 
    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
         # $3 is the path to the local log file for the node
         if self.gen_for_windows:
-            self.append(cmds, "xcopy %3% {}\\".format(dest_dir))
+            self.append(cmds, "xcopy %3 {}\\".format(dest_dir))
         else:
-            self.append(cmds, "mkdir -p {} && cp $3 {}".format(dest_dir, dest_dir))
+            self.append(cmds, "mkdir {} && cp $3 {}".format(dest_dir, dest_dir))
