@@ -10,7 +10,7 @@ here, check to see if you need to also change it in psm.py.
 '''
 
 #Sync the BUILD string changes with: setup.py, CHANGELOG.md
-BUILD = "version: 0.0.229, build: Sep-20-2020"
+BUILD = "version: 0.0.237, build: Jan-29-2021"
     
 INFO_CONTAINER_V1 = "xt-store-info"
 INFO_CONTAINER_V2 = "00-store-info"
@@ -21,6 +21,8 @@ INFO_DIR = "__info__"
 STORAGE_INFO_FILE = INFO_DIR + "/storage_info.json"
 NODE_PREFIX = "_"
 
+TRACEBACK = "Traceback (most recent call last)"
+
 # XT inserted metrics
 INDEX = "__index__"
 STEP_NAME = "__step_name__"
@@ -109,7 +111,7 @@ CONTROLLER_INNER_LOG = "./__after__/controller_inner.log"   # stdout capture fro
 CONTROLLER_SHELL = "~/.xt/cwd/run_controller.sh"
 CONTROLLER_BATCH = "~/.xt/cwd/run_controller.bat"
         
-APP_EXIT_MSG = "@__app_exit__:"
+APP_EXIT_MSG = "@__app_exit__"
 TEMP_SCRIPT = "$TEMP/xt_script"
 
 LOCAL_KEYPAIR_PRIVATE = "~/.ssh/xt_id_rsa"
