@@ -88,7 +88,7 @@ class XTConfig():
 
         # expand values containing a "$" id
         if isinstance(value, str) and "$" in value:
-            value = self.expand_system_symbols(value, group, name)
+            value = self.expand_system_symbols(value, name)
         elif isinstance(value, dict):
             for key, val in value.items():
                 if isinstance(val, str) and "$" in val:
@@ -132,7 +132,10 @@ class XTConfig():
             vault_creds = self.get_vault_creds()
 
             if not "url" in vault_creds:
-                errors.config_error("URL not specified for '{}' in [external-services] section of XT config file".format(vault_name))
+                vault_name = vault_creds["name"]
+
+                errors.config_error("URL not specified for '{}' in [external-services] section of XT config file".\
+                    format(vault_name))
 
             vault_url = vault_creds["url"]
             secret_name = utils.safe_value(vault_creds, "secret-name", "xt-keys")
@@ -151,7 +154,7 @@ class XTConfig():
                 # create on-demand
                 self.create_vault_if_needed()
 
-            assert section and prop_name
+            assert section 
             text = self.vault.get_secret(section)
 
         if text and "$username" in text:
@@ -240,7 +243,7 @@ class XTConfig():
         #self.expand_symbols_in_creds(service, service_name)
         return service
 
-    def get_compute_def(self, target_name): 
+    def get_target_def(self, target_name): 
         compute_def = self.get("compute-targets", target_name, suppress_warning=True)
 
         if not compute_def:
@@ -251,16 +254,18 @@ class XTConfig():
             # make box look like a target
             compute_def = {"service": "pool", "boxes": [target_name]}
 
-            # use setup from first box 
+            # use setup and docker from first box 
             if "setup" in box_info:
                 compute_def["setup"] = box_info["setup"]
+            if "docker" in box_info:
+                compute_def["docker"] = box_info["docker"]
 
         compute_def["name"] = target_name
         #self.expand_symbols_in_creds(target, target_name)
         return compute_def        
 
     def get_external_service_from_target(self, target_name):
-        target = self.get_compute_def(target_name)
+        target = self.get_target_def(target_name)
         
         if not "service" in target:
                 errors.config_error("'service' property must be defined for target={} in the XT config file".format(target))
@@ -292,20 +297,21 @@ class XTConfig():
         storage_creds["name"] = storage_name
         return storage_creds
 
-    def get_mongo_creds(self):
-        # validate MONGO service
+    def get_database_creds(self):
+        # validate database service
         ss_info = self.get_store_info()
-        mongo_name = utils.safe_value(ss_info, "mongo")
-        if not mongo_name:
-            errors.config_error("'mongo' property must be set in store '{}'".format(ss_info["name"]))
+        db_name = utils.safe_value(ss_info, "database")
+        if not db_name:
+            errors.config_error("'database' property must be set in store '{}'".format(ss_info["name"]))
 
-        # validate MONGO credentials
-        mongo_creds = self.get("external-services", mongo_name, suppress_warning=True)
-        if not mongo_creds:
-            errors.config_error("'{}' must be specified in [external-services] section of XT config file".format(mongo_name))
+        # validate database credentials
+        db_creds = self.get("external-services", db_name, suppress_warning=True)
+        if not db_creds:
+            errors.config_error("'{}' must be specified in [external-services] section of XT config file".format(db_name))
 
-        #self.expand_symbols_in_creds(mongo_creds, mongo_name)
-        return mongo_creds, mongo_name
+        #self.expand_symbols_in_creds(db_creds, db_name)
+        db_creds["name"] = db_name
+        return db_creds
 
     def get_vault_creds(self):
         # validate VAULT service
@@ -369,7 +375,8 @@ class XTConfig():
         providers = self.get("providers", provider_type)
 
         if not name in providers:
-            errors.config_error("{} provider='{}' not registered in XT config file".format(provider_class, service_name))
+            errors.config_error("{} provider='{}' not registered in XT config file".\
+                format(provider_type, name))
 
         code_path = providers[name]
         return utils.get_class_ctr(code_path)
@@ -379,7 +386,7 @@ class XTConfig():
         login_server = None
         docker_registry = None
 
-        compute_def = self.get_compute_def(target)
+        compute_def = self.get_target_def(target)
 
         if not docker_name:
             docker_name = utils.safe_value(compute_def, "docker")
@@ -480,18 +487,21 @@ def _merge_configs(config, overrides):
             config_data[section_name] = section_value
         else:
             # process dict
+            section = config_data[section_name]
+
             for key, value in section_value.items():
-                if isinstance(value, dict):
+                # note: entries in a dict like compute-targets should be overwritten in whole (not prop by prop)
+                if False:   # isinstance(value, dict):
                     # handle [section.subsection] 
                     if not key in config_data[section_name]:
-                        config_data[section_name][key] = {}
+                        section[key] = {}
                     for inner_key, inner_value in value.items():
                         #console.print("overridding: [{}.{}] {} = {}".format(section_name, key, inner_key, inner_value))
-                        config_data[section_name][key][inner_key] = inner_value
+                        section[key][inner_key] = inner_value
                 else:
                     # handle [section]
                     #console.print("overridding: [{}] {} = {}".format(section_name, key, value))
-                    config_data[section_name][key] = value    
+                    section[key] = value    
 
 def _apply_override(validator, config, schema, fn_schema, fn_override):
     
@@ -506,7 +516,7 @@ def _apply_override(validator, config, schema, fn_schema, fn_override):
     
 def _apply_basic_mode_rules(config):
     sc = os.getenv("XT_STORE_CREDS")
-    mc = os.getenv("XT_MONGO_CONN_STR")
+    mc = os.getenv("XT_DB_CREDS")
     if sc and mc:
         # we are running on compute node (launched by script)
         #console.print("XT: detected run on compute node; setting advanced mode ON")
@@ -534,16 +544,22 @@ def get_merged_config(create_if_needed=True, local_overrides_path=None, suppress
     # validate the default config file
     validator = Validator()
     validator.validate(schema, fn_schema, config.data, config.fn, True)
+    fn_complete = config.fn
 
     # is there a GLOBAL config?
     fn_global = os.path.expanduser(os.getenv("XT_GLOBAL_CONFIG", "~/.xt/xt_config.yaml"))
     if os.path.isfile(fn_global):    
         _apply_override(validator, config, schema, fn_schema, fn_global)
+        fn_complete = fn_global
 
     # is there a LOCAL config?
     fn_local = local_overrides_path if local_overrides_path else constants.CONFIG_FN
     if os.path.isfile(fn_local):    
         _apply_override(validator, config, schema, fn_schema, fn_local)
+        fn_complete = fn_local
+
+    # now that merging is complete, validate all references in the config data
+    validator.validate_references(config.data, fn_complete)
 
     # apply special BASIC MODE rules
     _apply_basic_mode_rules(config)
