@@ -533,12 +533,15 @@ class HyperparameterExplorer(object):
             console.print("downloading runs for JOB={}...".format(run_group_type))
             # files are at JOB LEVEL
             # read SWEEPS file
-            if not store.does_job_file_exist(run_group_type, hp_config_cloud_path):
+            if not store.does_job_file_exist(ws_name, run_group_type, hp_config_cloud_path):
                 errors.store_error("missing job hp_config file (job={}, fn={})".format(run_group_type, hp_config_cloud_path))
             store.download_file_from_job(ws_name, run_group_type, hp_config_cloud_path, local_config_file_path)
 
+            fields = {"job_id": 1, "run_name": 1, "log_records": 1}
+
             # read ALLRUNS info aggregated in JOB
-            allrun_records = store.get_all_runs(run_group_name, ws_name, run_group_type, batch_size=batch_size)
+            allrun_records = store.get_all_runs(run_group_name, ws_name, run_group_type, 
+                fields_dict=fields, batch_size=batch_size)
 
         console.diag("after downloading all runs")
         return local_config_file_path, allrun_records
