@@ -6,9 +6,11 @@
 import os
 import sys
 import json
+from textwrap import wrap
 import time
 import datetime
 import tempfile
+from xtlib import report_builder
 import numpy as np 
 
 from xtlib.client import Client
@@ -171,6 +173,9 @@ class ImplCompute(ImplBase):
             return self.monitor_with_jupyter(workspace, name)
 
         if job_helper.is_job_id(name):
+            if "/" in name:
+                name, node_index = name.split("/")
+                
             return self.monitor_job_node(workspace, name, sleep, node_index, log_name, escape) 
 
         if run_helper.is_run_name(name):
@@ -373,7 +378,8 @@ class ImplCompute(ImplBase):
             console.print("==> monitoring cancelled (control-c detected)")
             ch = single_char_input("do you want to cancel the job? (y/n): ")
             if ch == "y":
-                self.cancel_job(job_id, workspace)
+                max_workers = self.config.get("general", "max-run-workers")
+                self.cancel_job([job_id], workspace, max_workers, cancel=True, wrapup=True)
                 show_final_status = False
 
         if show_final_status and last_status_shown != simple_status:
@@ -431,6 +437,7 @@ class ImplCompute(ImplBase):
         box_name = None
         parent_name = None
         node_index = None
+        xt_cmdline = None
 
         records = self.store.get_run_log(ws, run_name)
 
@@ -539,10 +546,6 @@ class ImplCompute(ImplBase):
             box_name = box_name.upper()
 
             if not run_summary:
-                # if is_aml:
-                #     console.print("Azure ML:")
-                # else:
-                #     console.print("box {}:".format(box_name))
                 console.print(box_name + ":")
 
             if not results:
@@ -583,7 +586,7 @@ class ImplCompute(ImplBase):
     def get_runs_by_box(self, run_names, workspace=None):
         run_names, actual_ws = run_helper.parse_run_list(self.store, workspace, run_names)
 
-        mongo = self.store.get_mongo()
+        mongo = self.store.get_database()
         fields_dict = {"box_name": 1, "compute": 1, "ws_name": 1, "run_name": 1}
 
         filter_dict = {}
@@ -632,30 +635,100 @@ class ImplCompute(ImplBase):
     #---- CANCEL JOB command ----
     @argument("job-names", type="str_list", help="the names of the jobs to be cancelled")
     @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
+    @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers for this command")
+    @flag("cancel", default=1, help="call the backend service to cancel the job")
+    @flag("wrapup", default=1, help="wrapup the runs associated with the job")
     @example(task="cancel jobs job3 and job15", text="xt cancel job job3, job15")
     @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified jobs and their active or queued runs")
-    def cancel_job(self, job_names, workspace):
+    def cancel_job(self, job_names, workspace, max_workers, cancel, wrapup):
 
         for job_id in job_names:
-            console.print("cancelling {}:".format(job_id))
+            service_job_info, service_info_by_node, backend = \
+                job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
 
-            service_job_info, service_info_by_node, backend = job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
-            result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
+            if cancel:
+                console.print("cancelling {} (service: {}):".format(job_id, backend.get_name()))
 
-            for node_id, result in result_by_node.items():
-                console.print("  {}: {}".format(node_id, result))
+                try:
+                    result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
 
-            # wrap up runs associated with each node
-            wrapup_count = 0
+                    for node_id, result in result_by_node.items():
+                        console.print("  {}: {}".format(node_id, result))
+                except BaseException as ex:
+                    console.print("Exception during service cancel call: {}".format(ex))
+                    result_by_node = service_info_by_node
+                # wrap up runs associated with each node
 
-            for node_id, result in result_by_node.items():
-                runs = run_helper.wrapup_runs_by_node(self.store, workspace, job_id, node_id, result)
-                wrapup_count += len(runs)
+            if wrapup:
+                # whether or not the cancel succeeded, cleanup the runs
+                node_list = list(service_info_by_node.keys())
+                runs = run_helper.wrapup_runs_for_nodes(self.store, workspace, job_id, node_list, max_workers)
 
-            console.print("  runs wrapped-up: {}".format(wrapup_count))
+    # TODO: this will replace the 'view status' command
+    #---- LIST QUEUES command ----
+    @argument("target", help="the name of the compute target")
+    @option("queues", type="str_list", values=["queued", "running", "completed"], help="the queues to list")
+    @option("username", default="$general.username", help="the username to filter entries with")
+    @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
+    @example(task="list my all of my queued jobs on the target philly-rr1", text="list queues philly-rr1  --queues=queued ")
+    @command(kwgroup="list", kwhelp="displays the specified storage items", help="list the queue entries for the specified compute target")
+    def list_queues(self, target, username, queues, workspace):
+        backend = self.core.create_backend(target, username=username)
+        target_def = self.config.get_target_def(target)
+        builder = report_builder.ReportBuilder()
+
+        results_by_queue, columns = backend.get_queue_jobs(target_def, username, queues, workspace)
+
+        for queue, entries in results_by_queue.items():
+            console.print("\nqueue: {}".format(queue))
+
+            text, _ = builder.build_formatted_table(entries, columns, columns)
+            # indent all lines by 2 spaces
+            text = "  " + text.replace("\n", "\n  ")
+            console.print(text)
+
+    #---- CANCEL QUEUES command ----
+    @argument("target", help="the name of the compute target")
+    @option("queues", type="str_list", values=["queued", "running", "completed"], help="the queues to cancel")
+    @option("username", default="$general.username", help="the username to filter entries with")
+    @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
+    @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers for this command")
+    @flag("cancel", default=1, help="call the backend service to cancel the job")
+    @flag("wrapup", default=1, help="wrapup the runs associated with the job")
+    @example(task="cancel my all of my queued jobs on the target philly-rr1", text="cancel queues philly-rr1  --queues=queued ")
+    @command(kwgroup="cancel", help="cancel the queue entries for the specified compute target")
+    def cancel_queues(self, target, username, queues, workspace, max_workers, cancel, wrapup):
+        backend = self.core.create_backend(target, username=username)
+        target_def = self.config.get_target_def(target)
+
+        results_by_queue, columns = backend.get_queue_jobs(target_def, username, queues, workspace)
+        
+        for queue, jobs in results_by_queue.items():
+            cancel_results = []
+
+            for job in jobs:
+
+                if cancel:
+                    job_id = job["job_id"]
+                    console.print("cancelling {} (service: {}):".format(job_id, backend.get_name()))
+                    result = backend.cancel_job_from_service_dict(job, workspace)
+                    cancel_results.append(result)
+
+                if wrapup:
+                    job_id = job["job_id"]
+
+                    try:
+                        service_job_info, service_info_by_node, backend = \
+                            job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
+
+                        node_list = list(service_info_by_node.keys())
+                        runs = run_helper.wrapup_runs_for_nodes(self.store, workspace, job_id, node_list, max_workers)
+
+                    except BaseException as ex:
+                        console.print("exception during wrapup: {}".format(ex))
 
     #---- CANCEL ALL command ----
-    @argument("target", help="the name of the compute target whose runs will be cancelled")
+    @argument("target", help="the name of the compute target (or backend service) whose runs will be cancelled")
     @hidden("username", default="$general.username", help="the username to log as the author of this run")
     @option("workspace", default="$general.workspace", help="the workspace to cancel runs in")
     @example(task="cancel all runs for current user on Philly", text="xt cancel all philly")
@@ -665,7 +738,7 @@ class ImplCompute(ImplBase):
         backend = self.core.create_backend(target, username=username)
         cancel_results_by_box = {}
 
-        compute_info = self.config.get_compute_def(target)
+        compute_info = self.config.get_target_def(target)
         service_name = compute_info["service"]
 
         if service_name == "pool":
@@ -813,6 +886,7 @@ class ImplCompute(ImplBase):
     @hidden("env-vars", default="$general.env-vars", help="the environment variable to be passed to the run when it is launched")
     @hidden("evaluation-interval", default="$early-stopping.evaluation-interval", type=int, help="the frequencency (# of metric logs) for testing the policy")
     @option("experiment", default="$general.experiment", type=str, help="the name of the experiment to create this run under")
+    @option("fake-error-percent", default="$database.fake-error-percent", type=float, help="for testing only: the percentage of database calls to fail")
     @flag("fake-submit", help="when True, we skip creation of job and runs and the submit (used for internal testing)")
     @hidden("fn_generated_config", default="$hyperparameter-search.fn-generated-config", type=str, help="the name for the generated HP config file")
     @hidden("framework", default="$aml-options.framework", help="the name of the framework to be installed for the run (pytorch, tensorflow, chainer)")
@@ -825,7 +899,9 @@ class ImplCompute(ImplBase):
     @option("low-pri", type=bool, help="when true, use low-priority (preemptable) nodes for this job")
     @option("max-minutes", default="$hyperparameter-search.max-minutes", type=int, help="the maximum number of minutes the run can execute before being terminated")
     @hidden("max-seconds", type=int, default="$aml-options.max-seconds", help="the maximum number of seconds this run will execute before being terminated")
+    @option("max-run-delay", default="$database.max-run-delay", type=float, help="the maximum random delay applied to each run in the job")
     @option("max-runs", default="$hyperparameter-search.max-runs", type=int, help="the total number of runs across all nodes (for hyperparameter searches)")
+    @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers to use during run command")
     @hidden("maximize-metric", default="$general.maximize-metric", help="whether to minimize or maximize value of primary metric")
     @hidden("merge-batch-logs", type=int, default="$logging.merge-batch-logs", help="replaces STDOUT.txt and STDERR.txt with STDBOTH.txt")
     @hidden("mirror-dest", default="$logging.mirror-dest", help="the location where mirrored information is store (none, storage, grok)")
@@ -835,8 +911,9 @@ class ImplCompute(ImplBase):
     @hidden("model-share-path", default="$model.model-share-path", help="the model share name for the model")
     @flag("model-writable", default="$model.model-writable", help="when true, a mounted model path can be written to")
     @flag("jupyter-monitor", help="when True, a Jupyter notebook is created to monitor the run")
-    @option("monitor", default="$general.monitor", values=["new", "same", "none"], help="how to monitor primary run of the new job")
+    @option("monitor", default="$general.monitor", values=["new", "bg", "same", "none"], help="how to monitor primary run of the new job")
     @flag("multi-commands"   , help="the script file contains multiple run commands (one per line)")
+    @option("node-repeat", type=int, help="the number of times to repeat each hyperparameter set on each node")
     @option("nodes", type=int, help="the number of normal (non-preemptable) nodes to allocte for this run")
     @hidden("option-prefix", default="$hyperparameter-search.option-prefix", help="the prefix to be used for specifying hyperparameter options to the script")
     @option("parent-script", type=str, help="path of script used to initialize the target for repeated runs of primary script")
@@ -857,6 +934,7 @@ class ImplCompute(ImplBase):
     @flag("static-search",  default="$hyperparameter-search.static-search", help="if true, hyperparameter searches are performed at job submit time, when possible")
     @hidden("storage", default=None, type=str, help="name of storage service to be used for this run")
     @option("submit-logs", default=None, help="specifies a directory to which log files for the submit are saved")
+    @option("tags", default=None, type="tag_list", help="tags to be applied to the job, nodes, and runs")
     @option("target", default=None, type=str, help="one of the user-defined compute targets on which to run")
     @hidden("truncation-percentage", default="$early-stopping.truncation-percentage", type=float, help="(truncation only) percent of runs to cancel at each eval interval")
     @option("use-gpu", type=bool, default="$aml-options.use-gpu", help="when True, the gpu(s) on the nodes will be used by the run")
@@ -901,6 +979,11 @@ class ImplCompute(ImplBase):
         cmd = get_dispatch_cmd()
         args["xt_cmd"] = "xt " + cmd if cmd else ""
 
+        # backdoor way to set database options from a command
+        fake_error_percent = args["fake_error_percent"]
+        if fake_error_percent:
+            self.config.data["database"]["fake-error-percent"] = fake_error_percent
+
         with tempfile.TemporaryDirectory(prefix="import-") as temp_dir:
             runner = Runner(self.config, self.core, temp_dir)
 
@@ -915,7 +998,8 @@ class ImplCompute(ImplBase):
             self.monitor(name=job_id, workspace=workspace, escape=escape)
 
         elif monitor in ["bg", "new"]:
-            cmd = "xt --echo monitor {} --workspace={}".format(job_id, workspace)
+            store_name = self.config.data["store"]
+            cmd = "xt --echo --store={} monitor {} --workspace={}".format(store_name, job_id, workspace)
             noactivate = (monitor=="bg")
             process_utils.run_cmd_in_new_console(cmd, noactivate=noactivate)
 
