@@ -79,7 +79,8 @@ class ImplUtilities(ImplBase):
 
     #---- XT ROOT FLAGS/OPTIONS ----
     @flag("help", help="Shows an overview of XT command syntax")
-    @option("console", type=int, values=["none", "normal", "diagnostics", "detail"], default="$internal.console", help="sets the level of console output")
+    @option("console", values=["none", "normal", "timing", "diagnostics", "detail"], default="$internal.console", help="sets the level of console output")
+    @option("config", type=str, default="xt_config.yaml", help="selects the XT config file to be used")
     @option("store", type=str, default="$store", help="selects the store to be used from the stores list")
     @flag("stack-trace", default="$internal.stack-trace", help="Show stack trace when an exception is raised")
     @flag("quick-start", default="$general.quick-start", help="When true, XT startup time is reduced (experimental)")
@@ -120,6 +121,10 @@ class ImplUtilities(ImplBase):
         elif name == "prep":
             self.prep_machine_for_controller()
             
+        elif name == "config":
+            # already handled by xt_cmds.main()
+            pass
+
         else:
             errors.syntax_error("unrecognized root flag=" + name)
 
@@ -304,7 +309,7 @@ class ImplUtilities(ImplBase):
     def grok(self, store, browse):
 
         url = "https://grok.azurewebsites.net"
-        if team:
+        if store:
             url += "/#/teams/" + store
 
         browse = browse and pc_utils.has_gui()
@@ -484,7 +489,7 @@ class ImplUtilities(ImplBase):
                     # it is a property assignment
                     left, right = stripped.split(":", 1)
                     prop_name = left.strip()
-                    prop_value = right.split()[0] if right else ""
+                    prop_value = right.strip()   # .split()[0] if right else ""
 
                     # apply to props_stack
                     new_entry = (indent, prop_name)
@@ -638,7 +643,7 @@ class ImplUtilities(ImplBase):
         registry_creds = None
 
         if not env:
-            compute_def = self.config.get_compute_def(compute)
+            compute_def = self.config.get_target_def(compute)
             env = utils.safe_value(compute_def, "docker")
 
         if env and env != "none":
@@ -651,7 +656,7 @@ class ImplUtilities(ImplBase):
             # get REGISTRY credentials
             registry_creds = self.config.get("external-services", registry_name, suppress_warning=True)
             if not registry_creds:
-                config_error("'{}' must be specified in [external-services] section of XT config file".format(registry_name))
+                errors.config_error("'{}' must be specified in [external-services] section of XT config file".format(registry_name))
 
         return registry_creds
 
