@@ -6,6 +6,7 @@
 import json
 import time
 import fnmatch
+from typing import List
 
 from .console import console
 from .report_builder import ReportBuilder   
@@ -22,17 +23,21 @@ job_info_props = {"_id": 1, "compute": 1, "concurrent": 1, "exper_name": 1, "hol
     "schedule": 1, "search_type": 1, "search_style": 1, "username": 1, "xt_cmd": 1, "started": 1,
     "pool_info": 1, "service_job_info": 1, "ws_name": 1}
 
-job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_nodes": 1, "total_runs": 1, "tags": 1}
-
+job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_runs": 1, 
+    "running_nodes": 1, "total_runs": 1, "db_retries": 1, "storage_retries": 1,
+    "restarts": 1}
 
 def is_job_id(name):
     # compatible with version 1 (prefix-jobNNNN) and version 2 (jobNNNN)
     return "job" in name.lower()
 
-def expand_job_list(store, mongo, workspace, name_list, can_mix=True):
+def expand_job_list(store, db, workspace, name_list, can_mix=True):
     '''
     parse jobs, expand job ranges
     '''
+    actual_ws = None
+    job_list = None
+
     if name_list:
         first_name = name_list[0]
         if len(name_list)==1 and file_utils.has_wildcards(first_name):
@@ -42,7 +47,7 @@ def expand_job_list(store, mongo, workspace, name_list, can_mix=True):
             if workspace:
                 filter_dict["ws_name"] = workspace
 
-            all_names = mongo.get_job_names(filter_dict)
+            all_names = db.get_job_names(filter_dict)
             job_list = [jn for jn in all_names if fnmatch.fnmatch(jn, first_name)]
         else:            
             job_list, actual_ws = parse_job_list(store, workspace, name_list, can_mix=can_mix)
@@ -52,41 +57,32 @@ def expand_job_list(store, mongo, workspace, name_list, can_mix=True):
 
     return job_list, actual_ws
 
-def set_job_tags(store, mongo, name_list, tag_list, workspace, fd, clear):
-    job_list, actual_ws = expand_job_list(store, mongo, workspace, name_list, can_mix=True)
-
+def set_job_tags(store, db, name_list, tag_list, ws_name, tag_dict, clear):
+    job_list, actual_ws = expand_job_list(store, db, ws_name, name_list, can_mix=True)
+    
     if job_list:
-        # for job in job_list:
-        #     mongo.update_job_info(workspace, job, fd, clear=clear, upsert=False)   
 
-        if len(job_list)==1 and job_list[0] == "*":
-            # update ALL records in ws
-            all_ws_jobs = mongo.get_info_for_jobs(workspace, {"ws_name": workspace}, {"_id": 1})
-            job_list = [job["_id"] for job in all_ws_jobs]
-        else:
-            # convert simple job_name into id required by storage version
-            job_list = [store_utils.make_id(workspace, job) for job in job_list]
-
-            matched_count = mongo.update_job_stats_from_list(workspace, job_list, fd, clear=clear, upsert=False)
+        filter_dict = {"job_id": {"$in": job_list}}
+        matched_count = db.set_job_tags(ws_name, filter_dict, tag_dict, clear)
 
         if clear:
-            console.print("{} jobs, tags cleared: {}".format(matched_count, tag_list))
+            console.print("{} jobs updated, tags cleared: {}".format(matched_count, tag_list))
         else:
-            console.print("{} jobs, tags set: {}".format(matched_count, tag_list))
+            console.print("{} jobs updated, tags set: {}".format(matched_count, tag_list))
 
     else:
         console.print("no matching jobs found")
 
 
-def list_job_tags(store, mongo, ws_name, name_list, tag_list):
-    job_list, actual_ws = expand_job_list(store, mongo, ws_name, name_list, can_mix=True)
+def list_job_tags(store, db, ws_name, name_list, tag_list, sort):
+    job_list, actual_ws = expand_job_list(store, db, ws_name, name_list, can_mix=True)
 
     if job_list:
         filter_dict = {"job_id": {"$in": job_list}}
         #fields_dict = {"tags": 1}
-        fields_dict = {"job_stats": 1}
+        fields_dict = {"tags": 1}
 
-        records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
+        records = db.get_info_for_jobs(ws_name, filter_dict, fields_dict)
         for record in records:
             job = record["_id"]
             console.print("{}:".format(job))
@@ -94,7 +90,13 @@ def list_job_tags(store, mongo, ws_name, name_list, tag_list):
             if "tags" in record:
                 tags = record["tags"] 
                 tag_names = list(tags.keys())
-                tag_names.sort()
+
+                if sort == "ascending":
+                    tag_names.sort()
+                elif sort == "descending":
+                    tag_names.sort(reverse=True)
+                elif sort:
+                    raise Exception("Unrecognized sort value: {}".format(sort))
 
                 for tag in tag_names:
                     if tag_list and not tag in tag_list:
@@ -133,6 +135,12 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
     if filter_exp_list:
         builder.process_filter_list(fd, filter_exp_list, user_to_actual)
 
+    # filter by tag
+    if "tag" in args:
+        tag_name = args["tag"]
+        if tag_name:
+                fd["tags." + tag_name] = {"$exists": True}
+
     # filter by tags_all
     tags_all = args["tags_all"]
     if tags_all:
@@ -161,12 +169,14 @@ def get_job_property_dicts():
         "repeat": "repeat", "search": "search_type", "service_type": "pool_info.service", 
         "sku": "pool_info.sku", "runs": "run_count", "search_style": "search_style", "started": "started", "target": "compute", "username": "username", 
         "vc": "pool_info.vc", "vm_size": "pool_info.vm-size", "workspace": "ws_name",  
-        "hparams": "hparams", "tags": "tags",
+        "hparams": "hparams", "tags": "tags", 
+
 
         # dynamic properties (get updated at various stages of job)
         "status": "job_status", "running_nodes": "running_nodes", 
         "running_runs": "running_runs", "error_runs": "error_runs", "completed_runs": "completed_runs",
-        "total_runs": "total_runs"
+        "total_runs": "total_runs", "db_retries": "db_retries", "storage_retries": "storage_retries",
+        "restarts": "restarts"
         }
 
     std_cols_desc = {
@@ -195,18 +205,23 @@ def get_job_property_dicts():
         "workspace": "the workspace associated with the job",
 
         # dynamic properties (get updated at various stages of job)
+        "db_retries": "the total number of database retries performed by the runs of this node",
         "status": "one of: submitted, running, completed",
         "running_nodes": "the number of nodes in the running state",
         "running_runs": "the number of this job's runs that are current running",
         "completed_runs": "the number of runs that have completed (with or without errors)",
         "error_runs": "the number of runs that have terminated with an error",
+        "storage_retries": "the total number of storage retries performed by the runs of this node",
         "total_runs": "the total number of runs specified for the job",
+        "restarts": "the total number node restarts in this job",
     }
 
     return user_to_actual, std_cols_desc
 
 def get_list_jobs_records(store, config, args):
-    job_list = args["job_list"]
+    list_nodes = utils.safe_value(args, "list_nodes")
+
+    job_list = args["node_list"] if list_nodes else args["job_list"]
     pool = args["target"]
 
     if job_list and store_utils.STORAGE_FORMAT == "1":
@@ -218,9 +233,13 @@ def get_list_jobs_records(store, config, args):
     else:
         workspace = args["workspace"]
 
+    console.diag("before ensure_workspace_exists")
+
     if workspace:
         store.ensure_workspace_exists(workspace, flag_as_error=True)
 
+    console.diag("after ensure_workspace_exists")
+
     # get info about job properties
     user_to_actual, std_cols_desc = get_job_property_dicts()        
     actual_to_user = {value: key for key, value in user_to_actual.items()}
@@ -228,20 +247,22 @@ def get_list_jobs_records(store, config, args):
     builder = ReportBuilder(config, store)
 
     # get list of specified jobs
-    mongo = store.get_mongo()
-    job_list, actual_ws = expand_job_list(store, mongo, workspace, job_list)
+    db = store.get_database()
+    job_list, actual_ws = expand_job_list(store, db, workspace, job_list)
 
     # build a filter dict for all specified filters
     filter_dict = build_job_filter_dict(workspace, job_list, user_to_actual, builder, workspace, args)
 
-    # get the mongo records for the matching JOBS
+    # get the db records for the matching JOBS
     #console.print("gathering job data...", flush=True)
-    records, using_default_last, last = builder.get_mongo_records(mongo, filter_dict, workspace, "jobs", actual_to_user, args=args)
+    records, using_default_last, last = builder.get_db_records(db, filter_dict, workspace, "jobs", actual_to_user, args=args)
     return records, using_default_last, last, user_to_actual, builder
 
 
-
 def list_jobs(store, config, args, compare=False):
+
+    console.diag("job_helper.list_jobs")
+
     available = args["available"]
 
     if compare:
@@ -250,6 +271,8 @@ def list_jobs(store, config, args, compare=False):
     records, using_default_last, last, user_to_actual, builder \
         = get_list_jobs_records(store, config, args)
 
+    console.diag("after get_list_jobs_records")
+
     if available:
         user_to_actual, std_cols_desc = get_job_property_dicts()        
         hparams_cols = run_helper.extract_dotted_cols(records, "hparams")
@@ -294,7 +317,6 @@ def list_jobs(store, config, args, compare=False):
             else:
                 console.print("no matching jobs found")
 
-
 def extract_tag_cols(records):
     tag_dict = {}
 
@@ -333,6 +355,14 @@ def get_job_number(job):
         part = part[3:] 
     return int(part)
 
+def expand_job_range(store, ws_name, low, high, can_mix, job_list):
+    low = get_job_number(low)
+    high = get_job_number(high)
+
+    for jx in range(low, high+1):
+        jxx = "job" + str(jx)
+        actual_ws = parse_job_helper(store, jxx, job_list, ws_name, False, can_mix=can_mix)
+
 def parse_job_list(store, workspace, jobs, can_mix=False):
     actual_ws = workspace
     job_list = []
@@ -344,12 +374,7 @@ def parse_job_list(store, workspace, jobs, can_mix=False):
             if "-" in job:
                 # range specified
                 low, high = job.split("-")
-                low = get_job_number(low)
-                high = get_job_number(high)
-
-                for jx in range(low, high+1):
-                    jxx = "job" + str(jx)
-                    actual_ws = parse_job_helper(store, jxx, job_list, actual_ws, False, can_mix=can_mix)
+                expand_job_range(store, workspace, low, high, can_mix, job_list)
             else:
                 actual_ws = parse_job_helper(store, job, job_list, actual_ws, can_mix=can_mix)
         
@@ -375,7 +400,7 @@ def get_client_cs(core, ws_name, job_id, node_index):
     box_secret = None
 
     filter = {"_id": job_id}
-    jobs = core.store.mongo.get_info_for_jobs(ws_name, filter, None)
+    jobs = core.store.database.get_info_for_jobs(ws_name, filter, None)
     if not jobs:
         errors.store_error("unknown job_id: {}".format(job_id))
 
@@ -402,12 +427,12 @@ def get_client_cs(core, ws_name, job_id, node_index):
 def get_job_records(store, workspace, job_names, fields_dict=None):
     ''' return job records for specified job names'''
 
-    mongo = store.get_mongo()
+    db = store.get_database()
 
     filter_dict = {}
     filter_dict["job_id"] = {"$in": job_names}
 
-    job_records = mongo.get_info_for_jobs(workspace, filter_dict, fields_dict)
+    job_records = db.get_info_for_jobs(workspace, filter_dict, fields_dict)
 
     return job_records
     
@@ -420,7 +445,7 @@ def get_job_record(store, workspace, job_id, fields_dict = None):
     
 def get_job_info_with_backend(store, core, workspace, job_id, node_index):
     
-    # FYI: getting job record from storage: 2.75 secs, from mongo: 2.39 secs (mongo slightly faster)
+    # FYI: getting job record from storage: 2.75 secs, from db: 2.39 secs (db slightly faster)
     job_info = get_job_record(store, workspace, job_id, {"pool_info": 1, "username": 1, "service_info_by_node": 1})
 
     pool_info = job_info["pool_info"]
@@ -429,8 +454,8 @@ def get_job_info_with_backend(store, core, workspace, job_id, node_index):
     backend = core.create_backend(target, compute_def=pool_info, username=username)
 
     if not "service_info_by_node" in job_info:
-        # mongo v2 requires a separate call for this
-        service_info_by_node = store.mongo.get_service_info_by_node(workspace, job_id)
+        # db v2 requires a separate call for this
+        service_info_by_node = store.database.get_service_info_by_node(workspace, job_id)
         job_info["service_info_by_node"] = service_info_by_node
 
     return backend, job_info
@@ -453,7 +478,7 @@ def get_service_job_info(store, core, ws_name, job_id):
     service_info_by_node = job_info["service_info_by_node"]
 
     # # get SERVICE_INFO_BY_NODE
-    # service_info_by_node = store.mongo.get_service_info_by_node(ws_name, job_id)
+    # service_info_by_node = store.database.get_service_info_by_node(ws_name, job_id)
     # job_info["job_info"] = job_info
 
     target = job_info["pool_info"]["name"]
@@ -462,10 +487,10 @@ def get_service_job_info(store, core, ws_name, job_id):
     return service_job_info, service_info_by_node, backend
 
 # helper
-def download_logs_from_storage(ws_name: str, store, items: [str], job_id: str, node_index: int, store_path: str,
+def download_logs_from_storage(ws_name: str, store, items: List[str], job_id: str, node_index: int, store_path: str,
         dest_dir: str):
 
-    job_path = "node-{}/{}".format(node_index, store_path)
+    job_path = "nodes/node{}/{}".format(node_index, store_path)
     new_items = store.download_files_from_job(ws_name, job_id, job_path, dest_dir)
 
     for ni in new_items:
