@@ -84,18 +84,20 @@ class ReportBuilder():
         console.diag("after reportbuilder SORT")
 
     def expand_special_symbols(self, value):
-        value = value.strip()
 
-        if value == "$none":
-            value = {"$type": 10}
-        elif value == "empty":
-            value = ""
-        elif value == "$true":
-            value = True
-        elif value == "$false":
-            value = False
-        elif value == "$exists":
-            value = {"$exists": True}
+        if isinstance(value, str):
+            value = value.strip()
+
+            if value == "$none":
+                value = {"$type": 10}
+            elif value == "empty":
+                value = ""
+            elif value == "$true":
+                value = True
+            elif value == "$false":
+                value = False
+            elif value == "$exists":
+                value = {"$exists": True}
 
         return value
 
@@ -114,18 +116,15 @@ class ReportBuilder():
         for filter_exp in filter_exp_list:
             prop = filter_exp["prop"]
             op = filter_exp["op"]
-            raw_value = filter_exp["value"]
+            value = filter_exp["value"]
             
-            #print("prop=", prop, ", op=", op, ", raw_value=", raw_value)
-
-            value = self.expand_special_symbols(raw_value)
-
             # translate name, if needed
             if prop in report2filter:
                 prop = report2filter[prop]
 
-            value = utils.make_numeric_if_possible(value)
-            #console.print("prop, op, value=", prop, op, value)
+            if isinstance(value, str):
+                value = self.expand_special_symbols(value)
+                value = utils.make_numeric_if_possible(value)
 
             if op in ["=", "=="]:
                 filter_dict[prop] = value
@@ -143,7 +142,7 @@ class ReportBuilder():
                 filter_dict[prop] = {"$regex" : value}
             elif op == ":exists:":
                 filter_dict[prop] = {"$exists" : value}
-            elif op == ":mongo:":
+            elif op == ":database:":
                 # raw filter dict, but we need to translate quotes and load as json
                 value = value.replace("`", "\"")
                 value = json.loads(value)
@@ -185,10 +184,10 @@ class ReportBuilder():
                 lines.append("  {}".format(col))
 
         if tags_list:
-            tag_list.sort()
+            tags_list.sort()
             lines.append("")
             lines.append("Tags:")
-            for col in tag_list:
+            for col in tags_list:
                 lines.append("  {}".format(col))
 
         return lines
@@ -198,11 +197,13 @@ class ReportBuilder():
 
         for key in record.keys():
             if key in subs:
-                self.build_avail_list(col_dict, record[key], prefix=key + ".")
+                value = record[key]
+                if value:
+                    self.build_avail_list(col_dict, value, prefix=key + ".")
             else:
                 col_dict[prefix + key] = 1
 
-    def flatten_records(self, records, sort_col, need_alive, args):
+    def flatten_records(self, records, sort_col, need_alive, which, args):
         '''
         pull out the AVAILABLE or USER SPECIFIED columns, flattening nested props to their dotted names.
         '''
@@ -299,7 +300,7 @@ class ReportBuilder():
 
         return first, last
 
-    def get_mongo_records(self, mongo, filter_dict, workspace, which, actual_to_user,
+    def get_db_records(self, database, filter_dict, workspace, which, actual_to_user,
             col_dict=None, col_names_are_external=True, flatten_records=True, need_alive=False, args=None):
 
         started = time.time()
@@ -307,20 +308,31 @@ class ReportBuilder():
         first, last = self.get_first_last(args)
         skip = utils.safe_value(args, "skip")
 
+        list_nodes = utils.safe_value(args, "list_nodes")
+        if list_nodes:
+            which = "nodes"
+
         if last:
             using_default_last = True
         else:
             using_default_last = False
 
         reverse = utils.safe_value(args, "reverse")
-        # use MONGO to do all of the work (query, sort, first/last)
+        # use database to do all of the work (query, sort, first/last)
         sort_col = utils.safe_value(args, "sort", "name")
         sort_dir = 1
 
         if sort_col:
             if sort_col == "name":
-                # special sorting needed; we have created "run_num" field just for this purpose
-                sort_col = "run_num" if which == "runs" else "job_num"
+
+                # special sorting needed; we have created "xxx_num" fields just for this purpose
+                if which == "runs":
+                    sort_col = "run_num"
+                elif which == "jobs":
+                    sort_col = "job_num"
+                elif which == "nodes":
+                    sort_col = "node_num"
+
             elif not "." in sort_col:
                 # translate name of std col from user-friendly version to logged version
                 user_to_actual = {value: key for key, value in actual_to_user.items()}
@@ -352,21 +364,29 @@ class ReportBuilder():
         # if not col_dict:
         #     col_dict = {"log_records": 0}
 
-        console.diag("get_mongo_records: first={}, last={}, ws: {}, filter_dict: {}, col_dict: {}". \
+        console.diag("get_db_records: first={}, last={}, ws: {}, filter_dict: {}, col_dict: {}". \
             format(first, last, workspace, filter_dict, col_dict))
 
         count_runs = utils.safe_value(args, "count")
         buffer_size = utils.safe_value(args, "buffer_size", 50)
 
-        # here is where MONGO does all the hard work for us
+        started = time.time()
+
+        # here is where DATABASE SERVICE does all the hard work for us
         if which == "runs":
-            records = mongo.get_filtered_sorted_run_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+            records = database.get_filtered_sorted_run_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+                count_runs, buffer_size)
+        elif which == "jobs":
+            records = database.get_filtered_sorted_job_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+                count_runs, buffer_size)
+        elif which == "nodes":
+            records = database.get_filtered_sorted_node_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
                 count_runs, buffer_size)
         else:
-            records = mongo.get_filtered_sorted_job_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-                count_runs, buffer_size)
+            errors.internal_error("unrecognized value for which: {}".format(which))
 
-        round_trip_count = mongo.round_trip_count
+        elapsed = time.time() - started
+        console.diag("  query elapsed: {:.2f} secs".format(elapsed))
 
         console.diag("after full records retreival, len(records)={}".format(len(records)))
 
@@ -376,10 +396,10 @@ class ReportBuilder():
 
         if flatten_records:
             # pull out requested cols, flattening nested values to their dotted names
-            records = self.flatten_records(records, sort_col, need_alive, args)
+            records = self.flatten_records(records, sort_col, need_alive, which, args)
 
         if last:
-            # we had to reverse the sort done by mongo, so correct it here
+            # we had to reverse the sort done by database, so correct it here
             records.reverse()
             #self.sort_records(records, sort_col, reverse)
 
@@ -401,8 +421,10 @@ class ReportBuilder():
         for sr in records:
             if "metric_names" in sr:
                 # seed col_dict with ordered list of metrics reported
-                for name in sr["metric_names"]:
-                    col_dict["metrics." + name] = 1
+                names = sr["metric_names"]
+                if names:
+                    for name in names:
+                        col_dict["metrics." + name] = 1
 
             # build from log records
             self.build_avail_list(col_dict, sr)
@@ -435,6 +457,10 @@ class ReportBuilder():
         fn_export = args["export"]
         if fn_export:
             fn_ext = os.path.splitext(fn_export)[1]
+            if not fn_ext:
+                fn_ext = ".txt"
+                fn_export += fn_ext
+
             sep_char = "," if fn_ext == ".csv" else "\t"
 
             col_list = user_col_args.keys()
@@ -591,7 +617,8 @@ class ReportBuilder():
         '''
 
         time_col_names = ["created", "started", "ended", "time"]
-        duration_col_names = ["duration", "queued"]
+        duration_col_names = ["duration", "queued", "queue_duration", "prep_duration",
+            "app_duration", "post_duration"]
 
         if not avail_cols:
             avail_cols = list(records[0].keys()) if records else []
@@ -743,7 +770,7 @@ class ReportBuilder():
                     text += "\n{}:\n".format(group)
 
                 txt, rc = self.generate_report(col_infos, grecords, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                    col_space, duration_col_names, time_col_names, significance, max_fixed_length)
+                    col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision)
 
                 # indent report
                 txt = "  " + txt.replace("\n", "\n  ")
@@ -755,7 +782,7 @@ class ReportBuilder():
         else:
             # UNGROUPED REPORT
             text, row_count = self.generate_report(col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                col_space, duration_col_names, time_col_names, significance, max_fixed_length)
+                col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision)
 
         return text, row_count
 
@@ -764,7 +791,7 @@ class ReportBuilder():
 
         if highlight_exp == "$alive":
             status = utils.safe_value(rd, "status")
-            should = status in ["queued", "running"]
+            should = status in ["submitted", "queued", "running"]
 
         return should
 
@@ -807,7 +834,11 @@ class ReportBuilder():
                     value = "{}{:,.1f} hrs".format(plus, hrs)
                 else:
                     mins = hrs*60
-                    value = "{}{:,.1f} mins".format(plus, mins)
+                    if mins > 1:
+                        value = "{}{:,.1f} mins".format(plus, mins)
+                    else:
+                        secs = mins*60
+                        value = "{}{:,.1f} secs".format(plus, secs)
 
         return value
 
@@ -818,7 +849,7 @@ class ReportBuilder():
         return color
 
     def generate_report(self, col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-        col_space, duration_col_names, time_col_names, significance, max_fixed_length):
+        col_space, duration_col_names, time_col_names, significance, max_fixed_length, float_precision):
 
         # process COLUMN HEADERS
         text = ""
