@@ -20,6 +20,7 @@ from xtlib import impl_storage
 
 from xtlib.storage.store import Store, store_from_context
 from xtlib.console import console
+from xtlib.impl_shared import StoreOnDemand
 from xtlib.helpers.xt_config import get_merged_config
 from xtlib.helpers.key_press_checker import KeyPressChecker
 
@@ -51,6 +52,14 @@ class Run():
         self.user_requests = []
         self.closed = False
 
+        # TODO: find/remove any unneeded AML logic (leftover from XT original-approach AML support)
+        self.is_aml = None
+        self.aml_run = None
+        self.aml_logging = False
+
+        self.db_creds = None
+        self.store_creds = None
+
         if auto_close:
             atexit.register(self.close)
 
@@ -85,31 +94,28 @@ class Run():
 
         # load context, if present
         self.context = None
+        provider_code_path = None
 
         if self.run_name:
-            fn_context = os.path.abspath(constants.FN_RUN_CONTEXT)
-
-            if os.path.exists(fn_context):
-                json_context = file_utils.read_text_file(fn_context)
-
-                context_dict = json.loads(json_context)
-                self.context = utils.dict_to_object(context_dict)
+            self.context = utils.load_context_from_mrc()
+            if self.context:
                 if not supress_normal_output:
                     console.print("run context loaded: {}".format(self.context.run_name))
             else:
                 if not supress_normal_output:
-                    console.print("run context file not found: {}".format(fn_context))
+                    console.print("run context file not found")
 
-        mc = os.getenv("XT_MONGO_CONN_STR")
-        self.mongo_conn_str = utils.base64_to_text(mc)
+            db_creds64 = os.getenv("XT_DB_CREDS")
+            db_creds_json = utils.base64_to_text(db_creds64)
+            self.db_creds = json.loads(db_creds_json)
 
-        # convert store_creds from string to dict
-        sc = os.getenv("XT_STORE_CREDS")
-        self.store_creds = utils.base64_to_text(sc)
-        if self.store_creds:
-            self.store_creds = json.loads(self.store_creds)
+            # convert store_creds from string to dict
+            sc = os.getenv("XT_STORE_CREDS")
+            self.store_creds = utils.base64_to_text(sc)
+            if self.store_creds:
+                self.store_creds = json.loads(self.store_creds)
 
-        provider_code_path = os.getenv("XT_STORE_CODE_PATH")
+            provider_code_path = os.getenv("XT_STORE_CODE_PATH")
 
         run_cache_dir = None
         self.config = config
@@ -130,24 +136,25 @@ class Run():
 
         self.aml_logging = aml_logging
         self.is_aml_child = False    
-        self.is_aml = False   # TODO: remove need for this since we now treat AML runs normally
 
-        if True:    # self.run_name:
-            if not self.store_creds and not config:
-                # if store_creds not set, this app is running outside of XT control
-                # provide access to XT store for dev/test purposes
-                config = get_merged_config(suppress_warning=True)
-                
-            self.config = config
+        # try to create the Store object, even if we are not in a run
+        if not self.store_creds and not config:
+            # if store_creds not set, this app is running outside of XT control
+            # provide access to XT store for dev/test purposes
+            config = get_merged_config(suppress_warning=True)
+            
+        self.config = config
 
-            if self.context:
-                self.store = store_from_context(self.context)
-            else:
-                self.store = Store(self.store_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
-                    mongo_conn_str=self.mongo_conn_str, config=config)
+        self.store = StoreOnDemand(config)
 
-            # if not supress_normal_output:
-            #     console.print("XT logging enabled: ", self.run_name)
+        # if self.context:
+        #     self.store = store_from_context(self.context)
+        # else:
+        #     self.store = Store(self.store_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
+        #         db_creds=self.db_creds, config=config)
+
+        # if not supress_normal_output:
+        #     console.print("XT logging enabled: ", self.run_name)
 
         # distributed training support
         self.rank = None
@@ -164,10 +171,10 @@ class Run():
             # log stuff normally done by controller at start of run
             self.store.log_run_event(self.ws_name, self.run_name, "started", {})   
 
-            if self.store.mongo:
-                self.store.mongo.run_start(self.ws_name, self.run_name)
+            if self.store.database:
+                self.store.database.run_start(self.ws_name, self.run_name)
                 if self.context:
-                    self.store.mongo.job_run_start(self.ws_name, self.context.job_id)
+                    self.store.database.job_run_start(self.ws_name, self.context.job_id)
 
     def check_for_user_request(self):
 
@@ -212,13 +219,44 @@ class Run():
         # changes won't be pushed thru blobfuse mnt_output_dir, so write them to 
         # local_output_dir where we will use XT mirroring to push their changes to "mirroring" path 
         # in run's storage
+
+        # # debug blobfuse path problem
+        # fn = os.getenv("XT_OUTPUT_DIR") + "/test.txt"
+        # print("TESTING ability to write new file:", fn)
+        # file_utils.ensure_dir_exists(file=fn)
+        # with open(fn, "wt") as outfile:
+        #     outfile.write("testing...\n")
+        # print("test PASSED", flush=True)
+
+        # # debug blobfuse path problem
+        # fn =  os.getenv("XT_NODE_DIR") + "/test.txt"
+        # print("TESTING ability to write new file:", fn)
+        # file_utils.ensure_dir_exists(file=fn)
+        # with open(fn, "wt") as outfile:
+        #     outfile.write("testing...\n")
+        # print("test PASSED", flush=True)
+
         serial_num = random.randint(1,100000)
         log_dir = "{}/logs/{}".format(self.tensorboard_path, serial_num)
 
         # tensorboard: SummaryWriter will output to ./runs/ directory by default
         log_path = os.path.expanduser(log_dir)
-        self.train_writer = SummaryWriter(log_path + "/train")
-        self.test_writer = SummaryWriter(log_path + "/test")
+        
+        log_train_path = log_path + "/train"
+        file_utils.ensure_dir_exists(log_train_path)
+
+        # debug path problem
+        fn = log_train_path + "/test.txt"
+        print("writing fn=", fn)
+        with open(fn, "wt") as outfile:
+            outfile.write("testing...\n")
+
+        print("opening SummaryWriter on path=", log_train_path)
+        self.train_writer = SummaryWriter(log_train_path)
+
+        log_test_path = log_path + "/test"
+        file_utils.ensure_dir_exists(log_test_path)
+        self.test_writer = SummaryWriter(log_test_path)
 
         philly_path = os.getenv("PHILLY_JOB_DIRECTORY")
         if philly_path:
@@ -240,7 +278,13 @@ class Run():
             # avoid calling close twice (once by app, once by auto_close)
             return
 
-        if self.xt_logging and self.direct_run and self.store and self.context:
+        # careful not to trigger store method if not needed so that local script runs
+        # don't trigger need for authentication unnecessarily 
+        has_store = False
+        if self.store and self.store.actual_store is not None:
+            has_store = True
+
+        if self.xt_logging and self.direct_run and has_store and self.context:
             context = self.context
             status = "completed"
             exit_code = 0
@@ -255,11 +299,11 @@ class Run():
                 capture_files=context.after_upload, job_id=context.job_id, is_parent = True, node_id=node_id, 
                 run_index=None)
 
-        self.store.mongo.on_run_close()
+        if has_store and self.store.database:        
+            self.store.database.on_run_close(self.ws_name, self.run_name)
 
-        if self.store and self.store.mongo:        
-            print("RUN: mongo call stats")
-            self.store.mongo.print_call_stats()
+            print("RUN: database call stats")
+            self.store.database.print_call_stats()
 
         if self.train_writer:
             self.train_writer.close()
@@ -273,7 +317,7 @@ class Run():
             console.print("run.close(): uploading output directory to RUN storage")
             self.upload_output_dir()
 
-        if self.is_aml and self.store:
+        if self.is_aml and has_store:
             # partially log the end of the run
 
             # TODO: how to do this partial log for killed/error runs?
@@ -281,7 +325,7 @@ class Run():
             exit_code = 0
             hparams_dict = None
             metrics_rollup_dict = None
-            end_time = utils.get_time()
+            end_time = utils.get_arrow_time_str()
             log_records = []
 
             self.store.end_run(self.ws_name, self.run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
@@ -297,16 +341,16 @@ class Run():
 
     def log_hparam(self, name, value):
         if self.store and self.xt_logging:
-            self.store.log_run_event(self.ws_name, self.run_name, "hparams", {name: value}, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, "hparams", {name: value})
 
         if self.is_aml and self.aml_logging:
-            self.aml_run.log(name, value, description)
+            self.aml_run.log(name, value)
 
     def log_hparams(self, data_dict):
         #console.print("log_hparam, self.store=", self.store)
 
         if self.store and self.xt_logging:
-            self.store.log_run_event(self.ws_name, self.run_name, "hparams", data_dict, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, "hparams", data_dict)
 
         if self.is_aml and self.aml_logging:
             self.aml_run.log_row("hparams", **data_dict)
@@ -346,15 +390,15 @@ class Run():
 
         if self.store and self.xt_logging:
             #console.print("logging run_event for metrics...")
-            self.store.log_run_event(self.ws_name, self.run_name, "metrics", dd, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, "metrics", dd)
 
             if metric_names_changed:
                 ddx = {"metric_names": list(self.metric_names)}
                 # if not self.supress_normal_output:
                 #     console.print("updating metric_names: {}".format(ddx))
 
-                if self.store.mongo:
-                    self.store.mongo.update_run_info(self.ws_name, self.run_name, ddx)
+                if self.store.database:
+                    self.store.database.update_run_info(self.ws_name, self.run_name, ddx)
 
         if self.is_aml and self.aml_logging:
             for name, value in dd.items():
@@ -392,7 +436,7 @@ class Run():
 
     def log_event(self, event_name, data_dict):
         if self.store and self.xt_logging:
-            self.store.log_run_event(self.ws_name, self.run_name, event_name, data_dict, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, event_name, data_dict)
 
     def is_resuming(self):
         # return a bool using not not
@@ -408,14 +452,15 @@ class Run():
             self.store.create_run_file(self.ws_name, self.run_name, FN_CHECKPOINT_DICT, text)
 
             # also log the checkpoint
-            self.store.log_run_event(self.ws_name, self.run_name, "set_checkpoint", dict_cp, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, "set_checkpoint", dict_cp)
             return True
         return False
 
     def clear_checkpoint(self):
         if self.store and self.checkpoints_enabled and not self.is_aml:
+            dict_cp = {}    # should this be passed by caller?
             self.store.delete_run_files(self.ws_name, self.run_name, FN_CHECKPOINT_WILD)
-            self.store.log_run_event(self.ws_name, self.run_name, "clear_checkpoint", dict_cp, is_aml=self.is_aml)
+            self.store.log_run_event(self.ws_name, self.run_name, "clear_checkpoint", dict_cp)
             return True
         return False
 
@@ -431,7 +476,7 @@ class Run():
                 text = self.store.read_run_file(self.ws_name, self.resume_name, FN_CHECKPOINT_DICT)
                 dict_cp = json.loads(text)
                 # log that we retreived the checkpoint
-                self.store.log_run_event(self.ws_name, self.run_name, "get_checkpoint", dict_cp, is_aml=self.is_aml)
+                self.store.log_run_event(self.ws_name, self.run_name, "get_checkpoint", dict_cp)
 
         return dict_cp
 
