@@ -4,33 +4,36 @@
 #
 # run_helper.py: functions needed for processing run-related information
 import json
+import math
 import fnmatch
-from datetime import timedelta 
+from datetime import timedelta
+from os import fdopen
 
 from xtlib import qfe
 from xtlib import utils
 from xtlib import errors
 from xtlib import constants
-from xtlib import file_utils
 from xtlib import job_helper
-from xtlib.helpers import file_helper
+from xtlib import file_utils
 from xtlib import store_utils
-
 from xtlib.console import console
+from xtlib.helpers import file_helper
+from xtlib.helpers.feedbackParts import feedback as fb
 
 run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, "exper_name": 1,
     "from_ip": 1, "from_computer_name": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
     "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_index": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
     "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
-    "xt_build": 1, "xt_version": 1}
+    "xt_build": 1, "xt_version": 1, "cluster": 1, "vc":1, "service_run_id": 1}
 
 run_stats_props = {"status": 1, "start_time": 1, "last_time": 1, "exit_code": 1, "restarts": 1, 
-    "end_time": 1, "queue_duration": 1, "run_duration": 1, "end_id": 1, "tags": 1, "metrics_names": 1}
+    "end_time": 1, "queue_duration": 1, "run_duration": 1, "end_id": 1, "metric_names": 1,
+     "db_retries": 1, "storage_retries": 1}
 
-def expand_run_list(store, mongo, workspace, name_list):
+def expand_run_list(store, db, workspace, name_list):
     '''
     args:
-        - mongo: instance of our mongodb mgr
+        - db: instance of our database mgr
         - workspace: name of the workspace associated with name_list
         - name_list: a list of run sources (run_names, job_names, experiment_names)
 
@@ -54,39 +57,39 @@ def expand_run_list(store, mongo, workspace, name_list):
     if name_list:
         for entry in name_list:
             if is_run_name(entry):
-                actual_ws = expand_run_entry(store, mongo, workspace, run_list, entry)
+                actual_ws = expand_run_entry(store, db, workspace, run_list, entry)
             elif job_helper.is_job_id(entry):
-                actual_ws = expand_job_entry(store, mongo, workspace, run_list, entry)
+                actual_ws = expand_job_entry(store, db, workspace, run_list, entry)
             else:
-                actual_ws = expand_experiment_name(store, mongo, workspace, run_list, entry)
+                actual_ws = expand_experiment_name(store, db, workspace, run_list, entry)
 
     return run_list, actual_ws
 
-def expand_run_entry(store, mongo, workspace, run_list, name):
+def expand_run_entry(store, db, ws_name, run_list, name):
     
     if name in ["*", "run*"]:
         # return  "all records" indicator
         sub_list = ["*"]
-        actual_ws = workspace
+        actual_ws = ws_name
     elif "*" in name:
         #match wildcard to all run names in workspace
         re_pattern = utils.wildcard_to_regex(name)
         filter_dict = {"_id": {"$regex": re_pattern} }
 
-        records = mongo.get_info_for_runs(workspace, None, {"_id": 1})
+        records = db.get_info_for_runs(ws_name, None, {"_id": 1})
         sub_list = [rec["_id"] for rec in records]
-        actual_ws = workspace
+        actual_ws = ws_name
     else:            
-        sub_list, actual_ws = parse_run_list(store, workspace, [name])
+        sub_list, actual_ws = parse_run_list(store, ws_name, [name])
         
     run_list += sub_list
     return actual_ws
 
-def expand_job_entry(store, mongo, workspace, run_list, job_term):
+def expand_job_entry(store, db, ws_name, run_list, job_term):
     '''
     Arguments:
         store: an instance of Store (access to storage) 
-        mongo: an instance of Mongo (access to related mongoDB)
+        db: an instance of Mongo (access to related mongoDB)
         workspace: name of the workspace to search within
         run_list: a list of run names being accumulated
         job_term: a single job_id, or range of job_id's (job3-job5)
@@ -97,12 +100,12 @@ def expand_job_entry(store, mongo, workspace, run_list, job_term):
     from xtlib import job_helper
 
     # expand job name_entry into a list of job names
-    job_list, actual_ws = job_helper.expand_job_list(store, mongo, workspace, [job_term], can_mix=False)
+    job_list, actual_ws = job_helper.expand_job_list(store, db, ws_name, [job_term], can_mix=False)
 
     run_filter = {"job_id": {"$in": job_list}}
-    records = mongo.get_info_for_runs(workspace, run_filter, {"_id": 1, "run_num": 1})
+    records = db.get_info_for_runs(ws_name, run_filter, {"_id": 1, "run_num": 1})
     if records:
-        # records are not in any order from mongo, so sort by run_num
+        # records are not in any order from db, so sort by run_num
         records.sort(key=lambda r: r["run_num"])
 
         names = [run["_id"] for run in records]
@@ -110,53 +113,42 @@ def expand_job_entry(store, mongo, workspace, run_list, job_term):
 
     return actual_ws
 
-def expand_experiment_name(store, mongo, workspace, run_list, exper_name):
+def expand_experiment_name(store, db, ws_name, run_list, exper_name):
 
-    actual_ws = workspace
+    actual_ws = ws_name
 
     run_filter = {"exper_name": exper_name}
-    result = mongo.get_info_for_runs(workspace, run_filter, {"_id": 1})
+    result = db.get_info_for_runs(ws_name, run_filter, {"_id": 1})
     if result:
         names = [run["_id"] for run in result]
         run_list += names
 
     return actual_ws
 
-def set_run_tags(store, mongo, name_list, tag_list, workspace, fd, clear):
-    run_list, actual_ws = expand_run_list(store, mongo, workspace, name_list)
+def set_run_tags(store, db, name_list, tag_list, ws_name, tag_dict, clear):
+    run_list, actual_ws = expand_run_list(store, db, ws_name, name_list)
 
     if run_list:
-        matched_count = 0
 
-        if len(run_list)==1 and run_list[0] == "*":
-            # update ALL records in ws
-            all_ws_runs = mongo.get_info_for_runs(workspace, {"ws_name": workspace}, {"_id": 1})
-            run_list = [run["_id"] for run in all_ws_runs]
-        else:
-            # convert simple run_name into id required by storage version
-            run_list = [store_utils.make_id(workspace, run) for run in run_list]
-
-        matched_count = mongo.update_run_stats_from_list(workspace, run_list, fd, clear=clear, upsert=False)
-
-        # for run in run_list:
-        #     mongo.update_run_info(workspace, run, fd, clear=clear, upsert=False)  
+        filter_dict = {"run_name": {"$in": run_list}}
+        matched_count = db.set_run_tags(ws_name, filter_dict, tag_dict, clear)
 
         if clear:
-            console.print("{} runs, tags cleared: {}".format(matched_count, tag_list))
+            console.print("{} runs updated, tags cleared: {}".format(matched_count, tag_list))
         else:
-            console.print("{} runs, tags set: {}".format(matched_count, tag_list))
+            console.print("{} runs updated, tags set: {}".format(matched_count, tag_list))
     else:
         console.print("no matching runs found")
 
-def list_run_tags(store, mongo, ws_name, name_list, tag_list):
-    run_list, actual_ws = expand_run_list(store, mongo, ws_name, name_list)
+def list_run_tags(store, db, ws_name, name_list, tag_list, sort):
+    run_list, actual_ws = expand_run_list(store, db, ws_name, name_list)
 
     if run_list:
         filter_dict = {"run_name": {"$in": run_list}}
         #fields_dict = {"tags": 1}
-        fields_dict = {"run_stats": 1}
+        fields_dict = {"tags": 1}
 
-        records = mongo.get_info_for_runs(ws_name, filter_dict, fields_dict)
+        records = db.get_info_for_runs(ws_name, filter_dict, fields_dict)
         for record in records:
             run = record["_id"]
             console.print("{}:".format(run))
@@ -164,7 +156,13 @@ def list_run_tags(store, mongo, ws_name, name_list, tag_list):
             if "tags" in record:
                 tags = record["tags"] 
                 tag_names = list(tags.keys())
-                tag_names.sort()
+
+                if sort == "ascending":
+                    tag_names.sort()
+                elif sort == "descending":
+                    tag_names.sort(reverse=True)
+                elif sort:
+                    raise Exception("Unrecognized sort value: {}".format(sort))
 
                 for tag in tag_names:
                     if tag_list and not tag in tag_list:
@@ -193,15 +191,16 @@ def get_run_property_dicts():
         "node": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
         "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_index": "run_index", 
         "run_num": "run_num", "script": "script", "search": "search_type", "search_style": "search_style", 
-        "service_type": "service_type", "sku": "sku", "status": "status", "target": "compute", 
+        "service_type": "service_type", "sku": "sku", "started": "start_time", "status": "status", "target": "compute", 
         "username": "username", "vc": "vc", "workspace": "ws_name", "xt_build": "xt_build", 
-        "xt_version": "xt_version",
+        "xt_version": "xt_version", "cluster": "cluster", "vc": "vc", "service_run_id": "service_run_id",
 
         # embedded properties
         "hparams": "hparams", "metrics": "metrics", "tags": "tags",
 
         # run stats
         "ended": "end_time", "started": "start_time", "duration": "run_duration", "queued": "queue_duration",
+        "db_retries": "db_retries", "storage_retries": "storage_retries",
 
          # special info (not a simple property)
          "metric_names": "metric_names",
@@ -214,6 +213,7 @@ def get_run_property_dicts():
         "created": "the time when the run was created", 
         "cluster": "the name of the service cluster for the run", 
         "ended": "when the run execution ended",
+        "db_retries": "the total number of database retries performed by the runs of this node",
         "description": "the user specified description associated with the run",
         "duration": "how long the run has been executing",
         "ended": "when the run completed",
@@ -240,9 +240,11 @@ def get_run_property_dicts():
         "search": "the type of hyperparameter search used to start the run",
         "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
         "service_type": "the type of service of the compute target",
+        "service_run_id": "the run id assigned by the backend service",
         "sku": "the name of the service SKU (machine size) specified for the run",
         "started": "when the run started executing",
         "status": "the current status of the run",
+        "storage_retries": "the total number of storage retries performed by the runs of this node",
         "target": "the compute target the run was submitted to",
         "username": "the login name of the user who submitted the run",
         "vc": "the name of the virtual cluster for the run",
@@ -253,6 +255,67 @@ def get_run_property_dicts():
 
     return user_to_actual, std_cols_desc
         
+def build_filter_from_mixed_run_list(store, ws_name, run_list, fd):
+
+    exper_names = []
+    job_names = []
+    run_names = []
+
+    exper_fd = {}
+    job_fd = {}
+    run_fd = {}
+
+    # group into experiments, jobs, runs
+    for name in run_list:
+        if "-" in name:
+            low, high = name.split("-")
+            if is_run_name(low):
+                expand_run_range(store, ws_name, low, high, run_names)
+            elif job_helper.is_job_id(low):
+                job_helper.expand_job_range(store, ws_name, low, high, can_mix=False, job_list=job_names)
+            else:
+                errors.general_error("only runs and jobs can be used in ranges: {}".format(name))
+        elif is_run_name(name):
+            run_names.append(name)
+        elif job_helper.is_job_id(name):
+            job_names.append(name)
+        else:
+            # must be an experiment name
+            # TODO: validate experiment name using database query on jobs 
+            exper_names.append(name)
+                
+    # add EXPERIMENTS
+    if len(exper_names) == 1:
+        exper_fd["exper_name"] = exper_names[0]
+    elif len(exper_names) > 1:
+        exper_fd["exper_name"] = {"$in": exper_names}
+
+    # add JOBS
+    if len(job_names) == 1:
+        job_fd["job_id"] = job_names[0]
+    elif len(job_names) > 1:
+        job_fd["job_id"] = {"$in": job_names}
+
+    # add RUNS
+    if len(run_names) == 1:
+        run_fd["run_name"] = run_names[0]
+    elif len(run_names) > 1:
+        run_fd["run_name"] = {"$in": run_names}
+
+    # merge them together
+    fd_list = []
+    for fdx in [exper_fd, job_fd, run_fd]:
+        if fdx:
+            fd_list.append(fdx)
+
+    count = len(fd_list)
+    if count == 1:
+        # merge our only fdx into fd
+        fd.update(fd_list[0])
+    elif count > 1:
+        # combine with an OR
+        fd["$or"] = fd_list
+
 def build_filter_part(fd, args, arg_name, store_name):
     value = args[arg_name]
 
@@ -265,18 +328,14 @@ def build_filter_part(fd, args, arg_name, store_name):
         else:
             fd[store_name] = value
 
-def build_run_filter_dict(ws_name, run_list, user_to_actual, builder, args):
+def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, args):
     fd = {}
     option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", "outer", "username"]  
 
     explict_options = qfe.get_explicit_options()
 
     if run_list:
-        # filter by specified run names
-        # convert simple run_name into id required by storage vx
-        run_list = [store_utils.make_id(ws_name, run) for run in run_list]
-
-        fd["_id"] = {"$in": run_list}
+        build_filter_from_mixed_run_list(store, ws_name, run_list, fd)
 
     # filter by specified options
     for name in option_filters:
@@ -289,6 +348,12 @@ def build_run_filter_dict(ws_name, run_list, user_to_actual, builder, args):
         if filter_exp_list:
             builder.process_filter_list(fd, filter_exp_list, user_to_actual)
 
+    # filter by tag
+    if "tag" in args:
+        tag_name = args["tag"]
+        if tag_name:
+                fd["tags." + tag_name] = {"$exists": True}
+
     # filter by tags_all
     if "tags_all" in args:
         tags_all = args["tags_all"]
@@ -360,7 +425,7 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     if workspace:
         store.ensure_workspace_exists(workspace, flag_as_error=False)
 
-    mongo = store.get_mongo()
+    db = store.get_database()
 
     # get info about run properties
     user_to_actual, std_cols_desc = get_run_property_dicts()        
@@ -369,12 +434,12 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     builder = ReportBuilder(config, store)
 
     # get list of specified runs
-    pure_run_list, actual_ws = expand_run_list(store, mongo, workspace, run_list)
-    if run_list and not pure_run_list:
-        errors.general_error("no matching runs found")
+    # pure_run_list, actual_ws = expand_run_list(store, db, workspace, run_list)
+    # if run_list and not pure_run_list:
+    #     errors.general_error("no matching runs found")
 
     # build a filter dict for all specified filters
-    filter_dict = build_run_filter_dict(workspace, pure_run_list, user_to_actual=user_to_actual, 
+    filter_dict = build_run_filter_dict(store, workspace, run_list, user_to_actual=user_to_actual, 
         builder=builder, args=args)
 
     # if show_gathering:
@@ -383,19 +448,19 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     highight_exp = config.get("run-reports", "highlight") 
     need_alive = highight_exp == "$alive"
 
-    # get the mongo records for the matching RUNS
-    records, using_default_last, last = builder.get_mongo_records(mongo, filter_dict, workspace, which="runs", 
+    # get the db records for the matching RUNS
+    records, using_default_last, last = builder.get_db_records(db, filter_dict, workspace, which="runs", 
         actual_to_user=actual_to_user, col_dict=col_dict, col_names_are_external=col_names_are_external, 
         flatten_records=flatten_records, need_alive=need_alive, args=args)
 
     # remove key info from _id in records
     store_utils.simplify_records_id(records)
 
-    # remove key from pure_run_list
-    pure_run_list = [pr.split("/")[-1] for pr in pure_run_list]
+    # # remove key from pure_run_list
+    # pure_run_list = [pr.split("/")[-1] for pr in pure_run_list]
 
-    if preserve_order:
-        records = order_runs_by_user_list(records, pure_run_list)
+    # if preserve_order:
+    #     records = order_runs_by_user_list(records, pure_run_list)
 
     console.diag("end of: get_filtered_sorted_limit_runs")
 
@@ -468,19 +533,20 @@ def list_runs(store, config, args, compare=False):
             else:
                 console.print("no matching runs found")
 
-def get_run_records(store, workspace, run_names, fields_dict=None):
+def get_run_records(store, ws_name, run_names, fields_dict=None):
     ''' return run records for specified run names'''
 
-    mongo = store.get_mongo()
+    db = store.get_database()
 
     filter_dict = {}
     filter_dict["run_name"] = {"$in": run_names}
 
     if not fields_dict:
         # by default, get everything but the log records
-        fields_dict = {"log_records": 0}
+        #fields_dict = {"log_records": 0}
+        fields_dict = {"run_info": 1, "run_stats": 1}
 
-    run_records = mongo.get_info_for_runs(workspace, filter_dict, fields_dict)
+    run_records = db.get_info_for_runs(ws_name, filter_dict, fields_dict)
 
     return run_records
 
@@ -536,7 +602,18 @@ def correct_slash(name):
         name = name.replace("\\", "/")
     return name
 
-def parse_run_list(store, workspace, runs, validate=True):
+def expand_run_range(store, ws_name, low, high, run_names):
+    low, low_prefix = get_rightmost_run_num(low)
+    high, high_prefix = get_rightmost_run_num(high)
+
+    if low_prefix != high_prefix:
+        errors.syntax_error("for run name range, prefixes must match: {} vs. {}".format(low_prefix, high_prefix))
+
+    for rx in range(low, high+1):
+        rxx = low_prefix + str(rx)
+        actual_ws = parse_run_helper(store, ws_name, rxx, validate=True, actual_ws=ws_name, run_names=run_names)
+
+def parse_run_list(store, ws_name, runs, validate=True):
     run_names = []
     actual_ws = None
 
@@ -556,19 +633,11 @@ def parse_run_list(store, workspace, runs, validate=True):
             if "-" in run:
                 # parse run range
                 low, high = run.split("-")
-                low, low_prefix = get_rightmost_run_num(low)
-                high, high_prefix = get_rightmost_run_num(high)
-
-                if low_prefix != high_prefix:
-                    errors.syntax_error("for run name range, prefixes must match: {} vs. {}".format(low_prefix, high_prefix))
-
-                for rx in range(low, high+1):
-                    rxx = low_prefix + str(rx)
-                    actual_ws = parse_run_helper(store, workspace, rxx, validate, actual_ws, run_names)
+                expand_run_range(store, ws_name, low, high, run_names)                
             else:
-                actual_ws = parse_run_helper(store, workspace, run, validate, actual_ws, run_names)
+                actual_ws = parse_run_helper(store, ws_name, run, validate, actual_ws, run_names)
     else:
-        actual_ws = workspace
+        actual_ws = ws_name
         
     #console.print("actual_ws=", actual_ws)
     return run_names, actual_ws   
@@ -619,14 +688,15 @@ def validate_run_name(store, ws, run_name, error_if_invalid=True, parse_only=Fal
 
     run_name = run_name.lower()
     if not parse_only and not "*" in run_name:
-        if not store.mongo.does_run_exist(ws, run_name):
+        if not store.database.does_run_exist(ws, run_name):
             if error_if_invalid:
                 errors.store_error("run '{}' does not exist in workspace '{}'".format(run_name, ws))
             else:
                 return None, None, None
     return ws, run_name, ws + "/" + run_name
 
-def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=None, cleanup=True):
+def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=None, cleanup=True, 
+    alias_to_actual=None):
     '''
     Args:
         records: the set of dict records of a run log.
@@ -634,6 +704,7 @@ def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=
         merge: when True, merge all datasets into a single one
         metrics: list of specific metric names to extract
         cleanup: when True, restarts are detected and their older records are removed 
+        alias_to_actual: a dict of name/value pairs.  if defined, use to translate metric names (from name to value).
 
     Processing:
         We process all "metrics" event log records, grouping each by their property names.  Each group is a metric set.
@@ -644,6 +715,7 @@ def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=
     step_index = None
     next_step = None
     step_name = None
+    step_value = None
     
     # for merge
     last_record = {}
@@ -684,6 +756,15 @@ def build_metrics_sets(records, steps=None, merge=False, metrics=None, timebase=
             need_first_metric = False
 
         dd = log_dict["data"]
+
+        if alias_to_actual:
+            # translate metric names
+            key_list = list(dd.keys())
+            
+            for key in key_list:
+                if key in alias_to_actual:
+                    actual = alias_to_actual[key]
+                    utils.rename_dict_key(dd, key, actual)
         
         if step_name is None:
             if constants.STEP_NAME in dd:
@@ -839,7 +920,7 @@ def get_client_cs(core, ws, run_name):
     box_secret = None
 
     filter = {"_id": run_name}
-    runs = core.store.mongo.get_info_for_runs(ws, filter, {"run_logs": 0})
+    runs = core.store.database.get_info_for_runs(ws, filter, {"run_logs": 0})
     if not runs:
         errors.store_error("Unknown run: {}/{}".format(ws, run_name))
 
@@ -856,10 +937,7 @@ def get_client_cs(core, ws, run_name):
 
     return cs, box_secret
 
-def wrapup_runs_by_node(store, workspace, job_id, node_id, node_cancel_result):
-    '''
-    wrap up a run from azure batch.  run may have started, or may have completed.  
-    '''
+def get_job_context(store, job_id, workspace):
     # get job_record
     job_info = job_helper.get_job_record(store, workspace, job_id)
     ws_name = job_info["ws_name"]
@@ -868,7 +946,7 @@ def wrapup_runs_by_node(store, workspace, job_id, node_id, node_cancel_result):
     if store.does_job_file_exist(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT):
         text = store.read_job_file(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT)
     else:
-        # legacy job (need to download xt_code.zip and extract MRC file)
+        # need to download xt_code.zip and extract MRC file (why different for some jobs?)
         tmp_dir = file_utils.make_tmp_dir("unzip_code")
         fn_zip_local = "{}/{}".format(tmp_dir, constants.CODE_ZIP_FN)
         store_path = "before/code/{}".format(constants.CODE_ZIP_FN)
@@ -880,28 +958,96 @@ def wrapup_runs_by_node(store, workspace, job_id, node_id, node_cancel_result):
         text = file_utils.read_text_file(fn_mrc)
 
     mrc_data = json.loads(text)
+    return mrc_data
 
-    context_by_nodes = mrc_data["context_by_nodes"]
-    context_plus = context_by_nodes[node_id]
-    context = context_plus["runs"][0]
-    
-    # get runs that need wrapup in job
-    node_index = utils.node_index(node_id)
+def get_wrapup_runs_for_specified_nodes(store, ws_name, job_id, nodes):
 
+    node_indexes = [utils.node_index(node_id) for node_id in nodes]
+
+    filter_dict = {"job_id": job_id, "node_index": {"$in": node_indexes}}
+    fields_dict = {"status": 1, "run_index": 1, "run_name": 1, "node_index": 1 }
+
+    # get runs for specified nodes
+    runs = store.database.get_info_for_runs(ws_name, filter_dict, fields_dict)
+
+    # filter runs to those that need wrapping up
+    filtered_runs = []
     unwrapped = ["created", "queued", "spawning", "allocating", "running"]
-    # filter_dict = {"job_id": job_id, "node_index": node_index, "status":  {"$in": unwrapped}}
-
-    filter_dict = {"job_id": job_id, "node_index": node_index}
-    fields_dict = {"status": 1, "run_index": 1, "run_name": 1 }
-
-    runs = store.mongo.get_info_for_runs(ws_name, filter_dict, fields_dict)
 
     for run in runs:
-        status = run["status"]
+        # watch our for runs that never got promoted to "queued", so they are missing a status
+        status = utils.safe_value(run, "status", "created")
         if status in unwrapped:
-            wrapup_run_with_context(store, run, context)
+            filtered_runs.append(run)
 
-    return runs
+    return filtered_runs
+
+def get_wrapup_runs_for_all_nodes(store, ws_name, job_id, node_list, max_workers):
+
+    mrc_data = get_job_context(store, job_id, ws_name)
+    context_by_nodes = mrc_data["context_by_nodes"]
+
+    # gather the runs on worker threads
+    next_progress_num = 1
+    all_runs = []
+
+    from threading import Lock
+    worker_lock = Lock()
+
+    def thread_worker(nodes, store, ws_name, job_id):
+
+        runs = get_wrapup_runs_for_specified_nodes(store, ws_name, job_id, nodes)
+
+        with worker_lock:
+            nonlocal all_runs, next_progress_num
+
+            all_runs += runs
+
+            for node in nodes:
+                node_msg = "gathering runs from nodes: {}/{}".format(next_progress_num, len(node_list))
+                fb.feedback(node_msg, id="gather_msg")  
+                next_progress_num += 1
+
+    utils.run_on_threads(thread_worker, node_list, max_workers, [store, ws_name, job_id])
+    return all_runs, context_by_nodes
+
+def wrapup_runs_for_nodes(store, ws_name, job_id, nodes, max_workers):
+    '''
+    wrap up a run from azure batch.  run may have started, or may have completed.  
+    '''
+    fb.reset_feedback()
+
+    run_list, context_by_nodes = get_wrapup_runs_for_all_nodes(store, ws_name, job_id, nodes, max_workers)
+    if run_list:
+        # wrapup a subset of run_list on each worker thread
+        next_progress_num = 1
+
+        from threading import Lock
+        worker_lock = Lock()
+
+        def thread_worker(runs, store, context_by_nodes):
+
+            for run in runs:
+                node_index = run["node_index"]
+                node_id = utils.node_id(node_index)
+
+                context_plus = context_by_nodes[node_id]
+                context = context_plus["runs"][0]
+
+                wrapup_run_with_context(store, run, context)
+
+                with worker_lock:
+                    nonlocal next_progress_num
+                    node_msg = "wrapping up runs: {}/{}".format(next_progress_num, len(run_list))
+
+                    fb.feedback(node_msg, id="wrapup_msg")  
+                    next_progress_num += 1
+
+        utils.run_on_threads(thread_worker, run_list, max_workers, [store, context_by_nodes])
+    else:
+        fb.feedback("no runs needing wrapup found")
+
+    fb.feedback("done", is_final=True)
 
 def wrapup_run_with_context(store, run, context_dict):
     context = utils.dict_to_object(context_dict)
@@ -978,4 +1124,5 @@ def is_run_name(name):
         else:
             is_valid = part.isdigit()
 
-    return is_valid
\ No newline at end of file
+    return is_valid
+
