@@ -11,21 +11,21 @@ import time
 import json
 import psutil
 import logging
-from threading import Thread, Lock
+from threading import Lock
 
 from xtlib import utils
 from xtlib import errors
 from xtlib import file_utils
+from xtlib.utils import log_info, log_title
 
 from xtlib.console import console
-from xtlib.storage.store import store_from_context
 
 logger = logging.getLogger(__name__)
 
 class RunInfo():
     def __init__(self, run_name, workspace, cmd_parts, run_script, repeat, context, status, show_output=True, parent_name=None, 
-        parent_prep_needed=False, mirror_close_func=None, aml_run=None, node_id=None, run_index=None, 
-        mri=None, mri_entry=None, store=None):
+        parent_prep_needed=False, mirror_close_func=None, node_id=None, run_index=None, max_delay=0,
+        store=None):
 
         if isinstance(repeat, str):
             repeat = int(repeat)
@@ -59,17 +59,15 @@ class RunInfo():
         self.process_was_created = False
         self.mirror_worker = None
         self.mirror_close_func = mirror_close_func
-        self.aml_run = aml_run
-        self.is_aml = not(not aml_run)
         self.job_id = context.job_id
         self.exper_name = context.exper_name
         self.username = context.username
         self.run_as_parent = context.search_style != "single"
         self.node_id = node_id
         self.run_index = run_index
-        self.mri = mri
-        self.mri_entry = mri_entry
         self.killed_for_restart = False
+        self.max_delay = max_delay
+        self.delay_started = None
 
     def set_console_fn(self, console_fn):
         console_fn = os.path.expanduser(console_fn)
@@ -93,8 +91,9 @@ class RunInfo():
         self.update_recent_output(text_msg)
 
         # append to output file
-        with open(self.console_fn, "a") as tfile:
-            tfile.write(text_msg)
+        if self.console_fn:
+            with open(self.console_fn, "a") as tfile:
+                tfile.write(text_msg)
 
         # send output to attached clients
         if self.show_output:
@@ -162,7 +161,7 @@ class RunInfo():
 
     def wrapup_parent_prep_run(self):
         context = self.context
-        self.store.log_run_event(context.ws, self.run_name, "parent_script_completed", {"status": self.status})  # , is_aml=self.is_aml)
+        self.store.log_run_event(context.ws, self.run_name, "parent_script_completed", {"status": self.status})  
 
     def run_wrapup(self):
         '''wrap-up the run (logging, capture)'''
@@ -181,22 +180,17 @@ class RunInfo():
             # for "xt restart controller", do not log "cancelled" for runs
             # in spirit of a box that has just been preempted
 
-            #console.print("run_wrapup: self.is_aml=", self.is_aml)
-            print("is_parent={}, self.repeat={}".format(self.is_parent, self.repeat))
+            log_info("runinfo.is_parent", self.is_parent)
+            log_info("runinfo.repeat", self.repeat)
 
-            #log_info("run_info: calling WRAPUP_RUN")
+            log_info("calling WRAPUP_RUN", self.run_name)
 
             store.wrapup_run(context.ws, run_name, context.aggregate_dest, context.dest_name, 
                 status=self.status, exit_code=self.exit_code, primary_metric=context.primary_metric, 
                 maximize_metric=context.maximize_metric, report_rollup=context.report_rollup, rundir=self.rundir, 
                 after_files_list=context.after_files_list, after_omit_list=context.after_omit_list, log_events=context.log, 
                 capture_files=context.after_upload, job_id=self.job_id, is_parent=self.is_parent, node_id=self.node_id, 
-                run_index=self.run_index)
-
-            #if self.mri_entry and store.mongo:
-            #   store.mongo.mark_child_run_completed(self.mri_entry)
-            if self.mri and self.mri_entry:
-                self.mri.mark_child_run_completed(self.mri_entry)
+                run_index=self.run_index, start_time=self.started)
 
     def set_status(self):
         if self.killing:
@@ -215,12 +209,13 @@ class RunInfo():
 
             # wait for process to completely terminate
             if wait_if_needed:
-                console.print("waiting for process to terminate...")
+                log_info("waiting for terminate", self.run_name)
                 self.process.wait()
 
             presult = self.process.poll()
             exited = (presult != None)
-            console.print("process.poll() result=", presult, ", exited=", exited)
+            log_info("process.poll() result=", presult)
+            log_info("exited=", exited)
 
             if exited or wait_if_needed:
                 self.exit_code = self.process.wait()
