@@ -12,20 +12,22 @@ import arrow
 import shutil
 import numpy as np
 import logging
+from interface import implements
 
 from xtlib import utils
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
+from xtlib import run_helper
+from xtlib.storage.db_interface import DbInterface
 
 from xtlib.console import console
-import xtlib.storage_v1.mongo_run_index as mongo_run_index
 
 logger = logging.getLogger(__name__)
 
 MONGO_INFO = "__mongo_info__"
 
-class MongoDB():
+class MongoDB(implements(DbInterface)):
     '''
     We use MongoDB to provide fast query access to a large collection of run data.
 
@@ -34,6 +36,15 @@ class MongoDB():
 
     Currently, we experience severe capacity problems running approx. 500 runs at once 
     (across all users on team).  We need to find a way to scale up to larger limits.
+
+    MongoDB v1 organizes its data as follows:
+        - database: xtdb
+        - collections:
+            __mongo_info__  (single record about format version & paired storage account)
+            __jobs__        (document for each nested JOB_INFO, with id=job_id)            
+            ws_counters     (__job__ document and a document for each workspace)
+            <ws name>       (a collection exists for each workspace; documents are the nested RUN_INFO with id=run_name)             
+
     '''
     def __init__(self, mongo_conn_str, run_cache_dir):
         if not mongo_conn_str:
@@ -62,6 +73,10 @@ class MongoDB():
 
     #---- UTILS ----
 
+    # API call
+    def get_db_type(self):
+        return "mongo_v1"
+
     def get_service_name(self):
         _, rest = self.mongo_conn_str.split("//", 1)
         if ":" in rest:
@@ -83,13 +98,13 @@ class MongoDB():
                 # this will create the mongo database called "xtdb", if needed
                 self.mongo_db = self.mongo_client["xtdb"]
 
-    def get_mongo_info(self):
-        records = self.mongo_with_retries("get_mongo_info", lambda: self.mongo_db[MONGO_INFO].find({"_id": 1}, None), return_records=True)
+    def get_db_info(self):
+        records = self.mongo_with_retries("get_db_info", lambda: self.mongo_db[MONGO_INFO].find({"_id": 1}, None), return_records=True)
         record = records[0] if records and len(records) else None
         return record
 
-    def set_mongo_info(self, info): 
-        self.mongo_with_retries("set_mongo_info", lambda: self.mongo_db[MONGO_INFO].update( {"_id": 1}, info, upsert=True) )
+    def set_db_info(self, db_info): 
+        self.mongo_with_retries("set_mongo_info", lambda: self.mongo_db[MONGO_INFO].update( {"_id": 1}, db_info, upsert=True) )
         
     def delete_workspace_if_needed(self, ws_name):
         self.remove_cache(ws_name)
@@ -122,8 +137,9 @@ class MongoDB():
                 console.print("  zapping cache_dir=", cache_dir)
                 file_utils.zap_dir(cache_dir)
 
-    def set_workspace_counters(self, ws_name, next_run, next_end):
-        update_doc = { "_id": ws_name, "next_run": next_run, "next_end": next_end, "next_child" : {} }
+    #def set_workspace_counters(self, ws_name, next_run, next_end):
+    def set_workspace_counters(self, ws_name, next_job_number, next_end_id):
+        update_doc = { "_id": ws_name, "next_run": next_job_number, "next_end": next_end_id, "next_child" : {} }
 
         cmd = lambda: self.mongo_db.ws_counters.find_and_modify( {"_id": ws_name}, update=update_doc, upsert=True)
         self.mongo_with_retries("set_workspace_counters", cmd)
@@ -191,8 +207,8 @@ class MongoDB():
      
         return next_id
 
-    def get_next_job_id(self, default_next=1):
-        return self.get_next_sequential_job_id(default_next)
+    def get_next_job_id(self, default_next):
+        return self.get_next_sequential_job_id(default_next=default_next)
 
     def get_next_run_id(self, ws_name, default_next=1):
         return self.get_next_sequential_ws_id(ws_name, "next_run", default_next)
@@ -282,7 +298,7 @@ class MongoDB():
 
     #---- RUNS ----
 
-    def create_mongo_run(self, dd):
+    def create_db_run(self, dd):
         # create run document on Mongo DB
         # copy standard CREATE properties
         run_doc = copy.deepcopy(dd)
@@ -305,18 +321,18 @@ class MongoDB():
         ws_name = dd["ws"]
 
         cmd = lambda: self.mongo_db[ws_name].insert_one(run_doc)
-        self.mongo_with_retries("create_mongo_run", cmd, ignore_error=True)
+        self.mongo_with_retries("create_db_run", cmd, ignore_error=True)
 
-    def process_run_event(self, ws_name, run_name, log_record):
+    def process_run_event(self, ws_name, run_name, orig_dd):
 
         if self.add_log_records:
             # first, add log record to ws/run document
-            update_doc = { "$push": {"log_records": log_record} }
+            update_doc = { "$push": {"log_records": orig_dd} }
             self.mongo_with_retries("process_run_event", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc, upsert=True) )
 
         if self.update_run_stats:
-            event_name = log_record["event"]
-            data_dict = log_record["data"]
+            event_name = orig_dd["event"]
+            data_dict = orig_dd["data"]
             updates = {}
 
             if event_name == "hparams":
@@ -370,38 +386,31 @@ class MongoDB():
 
             self.update_run_info(ws_name, run_name, updates)
 
-    def update_run_info(self, ws_name, run_id, orig_dd, clear=False, upsert=True, update_primary=False):
+    # API call
+    def create_db_run(self, rd):
+        ws_name = rd["ws"]   # rd["ws_name"]
+        run_name = rd["run_name"]
 
+        self.update_run_info(ws_name, run_name, rd, update_primary=True)  # , new_run=True)
+        
+    def update_run_info(self, ws_name, run_name, orig_dd, hparams=None, metrics=None, update_primary=False):
+
+        clear = False
         dd = dict(orig_dd)
 
         if self.update_run_stats:
             if clear:
                 update_doc = { "$unset": dd}
             else:
-                dd["last_time"] = utils.get_time()
+                dd["last_time"] = utils.get_arrow_time_str()
                 update_doc = { "$set": dd}
 
             # update, create prop if needed
             self.mongo_with_retries("update_run_info", \
-                lambda: self.mongo_db[ws_name].update_one( {"_id": run_id}, update_doc, upsert=upsert) )
+                lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc, upsert=True) )
 
-
-    # def update_run_info(self, ws_name, run_name, dd):
-    #     #console.print("update_run_info: ws_name={}, run_name={}, dd={}".format(ws_name, run_name, dd))
-
-    #     if self.update_run_stats:
-    #         update_dd = copy.deepcopy(dd)
-    #         update_dd["last_time"] = utils.get_time()
-
-    #         update_doc = { "$set": update_dd}
-
-    #         #console.print("update_run_info: ws_name={}, run_name={}, update_doc={}".format(ws_name, run_name, update_doc))
-
-    #         # do a REPLACE DOC update operation
-    #         self.mongo_with_retries("update_run_info", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc) )
-
-    def does_run_exist(self, ws, run_name):
-        records = self.get_info_for_runs(ws, {"_id": run_name}, {"_id": 1})
+    def does_run_exist(self, ws_name, run_name):
+        records = self.get_info_for_runs(ws_name, {"_id": run_name}, {"_id": 1})
         exists = len(records) == 1
         return exists
 
@@ -429,13 +438,13 @@ class MongoDB():
         console.diag("after get_boxes_for_runs()")        
         return run_records
 
-    def get_filtered_sorted_run_info(self, ws_name, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-            count_runs, buffer_size):
+    def get_filtered_sorted_run_info(self, ws_name, filter_dict, fields_dict, sort_col=None, sort_dir=1, skip=None, 
+        first=None, count_runs=False, buffer_size=50):
 
         self.v2_to_v1_run_cols(filter_dict)
-        self.v2_to_v1_run_cols(col_dict)
+        self.v2_to_v1_run_cols(fields_dict)
 
-        records = self.get_filtered_sorted_container(ws_name, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+        records = self.get_filtered_sorted_container(ws_name, filter_dict, fields_dict, sort_col, sort_dir, skip, first, 
             count_runs, buffer_size)
 
         for record in records:
@@ -566,39 +575,7 @@ class MongoDB():
 
         return records
 
-    def update_run_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
-
-        if self.update_run_stats:
-            if clear:
-                update_doc = { "$unset": dd}
-            else:
-                update_doc = { "$set": dd}
-
-            update_filter = {"_id": {"$in": id_list}}
-
-            # update, create prop if needed
-            result = self.mongo_with_retries("update_run_stats_from_list", lambda: \
-                self.mongo_db[ws_name].update_many( update_filter, update_doc, upsert=upsert) )
-
-            return result
-
     #---- JOBS ----
-
-    def update_job_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
-
-        if self.update_job_stats:
-            if clear:
-                update_doc = { "$unset": dd}
-            else:
-                update_doc = { "$set": dd}
-
-            update_filter = {"_id": {"$in": id_list}}
-
-            # update, create prop if needed
-            result = self.mongo_with_retries("update_job_stats_from_list", \
-                lambda: self.mongo_db["__jobs__"].update_many( update_filter, update_doc, upsert=upsert) )
-            return result
-
     def does_jobs_exist(self, ws_name):
         # does document for ws_name exist?
         job_id = "job1"
@@ -664,7 +641,7 @@ class MongoDB():
             return cursor
 
         # here is where MONGO does all the hard work for us
-        cursor = self.mongo_with_retries("get_mongo_records", fetch)
+        cursor = self.mongo_with_retries("get_db_records", fetch)
 
         # warn user if returning a large amount of records
         #count_runs = utils.safe_value(args, "count")
@@ -705,36 +682,40 @@ class MongoDB():
         console.diag("after get_info_for_jobs()")        
         return job_records
 
-    def get_filtered_sorted_job_info(self, workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-            count_runs, buffer_size):
+    def get_filtered_sorted_job_info(self, ws_name, filter_dict, fields_dict, sort_col=None, sort_dir=1, skip=None, 
+        first=None, count_jobs=False, buffer_size=50):
 
         self.v2_to_v1_job_cols(filter_dict)
-        self.v2_to_v1_job_cols(col_dict)
+        self.v2_to_v1_job_cols(fields_dict)
 
-        records = self.get_filtered_sorted_container("__jobs__", filter_dict, col_dict, sort_col, 
-            sort_dir, skip, first, count_runs, buffer_size)
+        records = self.get_filtered_sorted_container("__jobs__", filter_dict, fields_dict, sort_col, 
+            sort_dir, skip, first, count_jobs, buffer_size)
 
         for record in records:
             self.v1_to_v2_job_cols(record)
 
         return records
 
-    def create_mongo_job(self, jd):
+    def create_db_job(self, jd):
         ws_name = jd["ws_name"]
         job_id = jd["job_id"]
 
         self.update_job_info(ws_name, job_id, jd)
         
-    def update_job_info(self, ws_name, job_id, dd, clear=False, upsert=True):
+    #def update_job_info(self, ws_name, job_id, dd, clear=False, upsert=True):
+    def update_job_info(self, ws_name, job_id, orig_dd, update_primary=False):
+
+        clear = False
 
         if self.update_job_stats:
             if clear:
-                update_doc = { "$unset": dd}
+                update_doc = { "$unset": orig_dd}
             else:
-                update_doc = { "$set": dd}
+                update_doc = { "$set": orig_dd}
 
             # update, create if needed
-            self.mongo_with_retries("update_job_info", lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc, upsert=upsert) )
+            self.mongo_with_retries("update_job_info", lambda: self.mongo_db["__jobs__"].\
+                update_one( {"_id": job_id}, update_doc, upsert=True) )
 
     def get_job_names(self, filter_dict=None):
         job_names = []
@@ -763,7 +744,7 @@ class MongoDB():
 
         return ws_name
 
-    def job_node_start(self, ws_name, job_id):
+    def job_node_start(self, ws_name, job_id, node_index, is_restart):
         '''
         A job's node has started running.  We need to:
             - increment the job's "running_nodes" property
@@ -835,10 +816,10 @@ class MongoDB():
             restarts = utils.safe_value(doc, "restarts")
 
             # if queue_duration is non-zero, we have started this run previously
-            run_restarted = bool(queue_duration) or restarts > 0
+            run_restarted = bool(queue_duration) or bool(restarts)
             if run_restarted:
-                # increment the "restarts" field of run_stats
-                restarts += 1
+                # increment the "restarts" field of run_stats (handle None case)
+                restarts = 1 if restarts is None else restarts + 1
                 console.print("===> restart #{} detected: {} <======".format(restarts, run_name))
 
             if create_time_str:
@@ -853,7 +834,7 @@ class MongoDB():
                 update={"$set": {"start_time": now_str, "queue_duration": queue_duration, "restarts": restarts} })
             self.mongo_with_retries("run_start", cmd)
 
-    def run_exit(self, ws_name, run_name):
+    def run_exit(self, ws_name, run_name, status, exit_code, db_retries=None, storage_retries=None, start_time=None):
         '''
         A run has finished running.  We need to:
             - set the run "run_duration" property to NOW - start_time
@@ -877,13 +858,6 @@ class MongoDB():
                 cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
                 self.mongo_with_retries("run_exit #2", cmd)
 
-    def build_child_runs_schedule_data(self, total_run_count, node_count):
-        active_runs = mongo_run_index.build_active_runs("static", total_run_count, node_count)
-        return active_runs, "active_runs"
-
-    def create_child_run_mgr(self, ws_name, job_id, parent_run_name, node_id):
-        return mongo_run_index.MongoRunIndex(self, job_id, parent_run_name, node_id)
-
     def get_next_run_name(self, ws_name, job_id, is_parent, total_run_count, node_index):
         # v1 run name takes its base from the workspace
         run_num = self.get_next_run_id(ws_name)
@@ -891,6 +865,148 @@ class MongoDB():
 
         return run_name
 
-    def on_run_close(self):
+    def get_child_name(self, entry, parent_run_name, first_run_index):
+        # get 1-relative index for node's child runs
+        child_num = 1 + (entry["run_index"] - first_run_index)
+        parent_num = run_helper.get_parent_run_number(parent_run_name)
+
+        child_name = "run{}.{}".format(parent_num, child_num)
+        return child_name
+
+    def on_run_close(self, ws_name, run_name):
         # nothing for v1 to do on this API call
+        # because: no metrics to flush
         pass
+
+    # API call
+    def create_workspace_if_needed(self, ws_name, db_name=None):
+        # nothing for v1 to do on this API call 
+        # because: collection created when first RUN_INFO doc is written
+        pass
+
+    # API call
+    def get_filtered_sorted_node_info(self, ws_name, filter_dict, fields_dict, sort_col=None, sort_dir=1, skip=None, 
+        first=None, count_jobs=False, buffer_size=50):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        records = []
+        return records
+
+    # API call
+    def get_info_for_nodes(self, ws_name, filter_dict, fields_dict):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        records = []
+        return records
+
+    # API call
+    def node_start(self, ws_name, job_id, node_index, node_restart, prep_start_str):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        pass
+
+    # API call
+    def node_end(self, ws_name, job_id, node_index, db_retries, storage_retries, app_start_str):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        pass
+
+    # API call
+    def node_post_end(self, ws_name, job_id, node_index, post_start_str):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        pass
+
+    # API call
+    def node_run_start(self, ws_name, job_id, node_index):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        pass
+
+    # API call
+    def node_run_end(self, ws_name, job_id, node_index, exit_code):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        pass
+
+    # API call
+    def update_collection(self, collection_name, ws_name, dd, is_flat=True, flat_exceptions=[]):
+        '''
+        for now, node_info documents are not supported on mongo v1.
+        '''
+        if collection_name in ["node_info", "node_stats"]:
+            pass
+        else:
+            errors.internal_error("unsupported collection for update_collection: {}".format(collection_name))
+    
+    # API call
+    def get_run_count(self, ws_name):
+        records = self.get_info_for_runs(ws_name, {"ws_name": ws_name}, {"_id": 1})
+        return len(records)
+
+    # API call
+    def update_job_run_stats(self, ws_name, job_id, sd):
+        '''
+        Processing:
+            update records in the RUN_STATS collection, using the specified name/values
+            in "stats_dict" for all runs in the specified "job_id".
+        '''
+        update_doc = { "$set": sd}
+
+        self.mongo_with_retries("update_job_run_stats", \
+            lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc, upsert=True) ) 
+    
+    # API call
+    def set_job_tags(self, ws_name, filter_dict, tag_dict, clear_tags):
+
+        modified = None
+
+        if self.update_job_stats:
+            if clear_tags:
+                update_doc = { "$unset": tag_dict}
+            else:
+                update_doc = { "$set": tag_dict}
+
+            # update, create prop if needed
+            result = self.mongo_with_retries("update_job_stats_from_list", \
+                lambda: self.mongo_db["__jobs__"].update_many( filter_dict, update_doc, upsert=False) )
+            modified = result.modified_count
+
+        return modified
+
+    # API call
+    def set_run_tags(self, ws_name, filter_dict, tag_dict, clear_tags):
+
+        modified = None
+
+        if self.update_run_stats:
+            if clear_tags:
+                update_doc = { "$unset": tag_dict}
+            else:
+                update_doc = { "$set": tag_dict}
+
+            # update, create prop if needed
+            result = self.mongo_with_retries("update_run_stats_from_list", lambda: \
+                self.mongo_db[ws_name].update_many( filter_dict, update_doc, upsert=False) )
+
+            modified = result.modified_count
+
+        return modified
+
+    # API call
+    def set_insert_buffering(self, value):
+        '''
+        Support the buffering of multiple insert_record() calls before
+        the result is written in a single backend database call.
+        '''
+        # not supported in mongo_v1
+        pass
+
+            
\ No newline at end of file
