@@ -68,7 +68,7 @@ class StoreBlobObjs():
         storage_name = self.provider.get_service_name()
         connected_mongo = mongo.get_service_name()
 
-        mongo_info = mongo.get_mongo_info()
+        mongo_info = mongo.get_db_info()
         paired_storage = utils.safe_value(mongo_info, "paired_storage")
         if paired_storage and storage_name != paired_storage:
             errors.combo_error("mongo paired with storage service='{}', but passed XT storage service='{}'".format(  \
