@@ -39,10 +39,10 @@ logger = logging.getLogger(__name__)
 
 # public function
 def store_from_context(context):
-    store = Store(context.store_creds, provider_code_path=context.store_code_path, mongo_conn_str=context.mongo_conn_str)
+    store = StoreV1(context.store_creds, provider_code_path=context.store_code_path, mongo_conn_str=context.mongo_conn_str)
 
     # set switches for mongo stats, logging, and scheduling
-    mongo = store.mongo
+    mongo = store.database
     mongo.update_job_stats = context.update_job_stats
     mongo.update_run_stats = context.update_run_stats
     mongo.add_log_records = context.add_log_records
@@ -72,15 +72,20 @@ class StoreV1():
 
     '''
     
-    def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
+    def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  db_creds=None, 
         provider_code_path=None, validate_creds=False):
         '''This is the constructor for the Store class. '''
 
         self.feedback_enabled = feedback_enabled
+        mongo_conn_str = None
+            
+        if not db_creds and config:
+            db_creds = config.get_database_creds()
 
-        if not mongo_conn_str and config:
-            mongo_creds, mongo_name = config.get_mongo_creds()
-            mongo_conn_str = mongo_creds["mongo-connection-string"]
+        if db_creds:
+            mongo_conn_str = db_creds["connection-string"]
+
+        self.db_creds = db_creds
 
         if config:
             if not storage_creds:
@@ -102,8 +107,8 @@ class StoreV1():
                 errors.service_error("Azure Storage service credentials not set correctly" + 
                     "; use 'xt config' to correct")
 
-        self.mongo = mongo_v1.MongoDB(mongo_conn_str, run_cache_dir) if mongo_conn_str else None
-        self.mongo_conn_str = mongo_conn_str
+        self.database = mongo_v1.MongoDB(mongo_conn_str, run_cache_dir) if mongo_conn_str else None
+        self.db_conn_str = mongo_conn_str
         self.run_cache_dir = run_cache_dir
         self.provider_code_path = provider_code_path
         self.storage_creds = storage_creds
@@ -114,7 +119,7 @@ class StoreV1():
         self.cap_stdout = None
         self.cap_stderr = None
 
-        self.helper.validate_storage_and_mongo(self.mongo)
+        self.helper.validate_storage_and_mongo(self.database)
 
     def init_store_utils_if_needed(self):
         ''' this function works by forcing creation of JIT store object, which calls init_store_utils()'''
@@ -123,17 +128,17 @@ class StoreV1():
     def get_name(self):
         return self.helper.get_name()
         
-    def get_mongo(self):
-        return self.mongo
+    def get_database(self):
+        return self.database
 
     def get_props_dict(self):
-        pd = {"storage_creds": self.storage_creds, "mongo_conn_str": self.mongo_conn_str, 
+        pd = {"storage_creds": self.storage_creds, "db_creds": self.db_creds,
             "run_cache_dir": self.run_cache_dir, "provider_code_path": self.provider_code_path}
         return pd
 
     @staticmethod
     def create_from_props_dict(pd):
-        return Store(**pd) 
+        return StoreV1(**pd) 
 
     def _error(self, msg):
         raise Exception("Error - {}".format(msg))
@@ -159,7 +164,7 @@ class StoreV1():
         result = self.helper.delete_share(share_name)
         if result:
             # remove associated summary cache
-            self.mongo.remove_cache(share_name)
+            self.database.remove_cache(share_name)
 
         return result
     
@@ -198,14 +203,14 @@ class StoreV1():
         result = self.helper.delete_workspace(ws_name)
         if result:
             # remove associated mongo info
-            self.mongo.delete_workspace_if_needed(ws_name)
+            self.database.delete_workspace_if_needed(ws_name)
 
         return result
     
     def log_workspace_event(self, ws_name, event_name, data_dict):
         ''' log the specifed event_name and key/value pairs in the data_dict to the workspace log file.
         '''
-        record_dict = {"time": utils.get_time(), "event": event_name, "data": data_dict}
+        record_dict = {"time": utils.get_arrow_time_str(), "event": event_name, "data": data_dict}
         rd_text = json.dumps(record_dict)
 
         # append to workspace log file
@@ -216,7 +221,7 @@ class StoreV1():
         '''
         if use_mongo:
             # use mongo for more faster workspace names 
-            names = self.mongo.get_workspace_names()
+            names = self.database.get_workspace_names()
         else:
             # storage is the ground truth for workspace names
             names = self.helper.get_workspace_names()
@@ -256,7 +261,7 @@ class StoreV1():
         '''
         #return self.helper.upload_file_to_workspace(ws_name, ws_fn, source_fn)
         wf = self.workspace_files(ws_name, use_blobs=True)
-        return wf.upload_file(wf_fn, source_n)
+        return wf.upload_file(ws_fn, source_fn)
 
     def upload_files_to_workspace(self, ws_name, ws_folder, source_wildcard):
         ''' upload the local files matching 'source_wildcard' to the workspace folder 'ws_folder' within the workspace 'ws_name'.
@@ -301,7 +306,7 @@ class StoreV1():
         return wf.does_file_exist(ws_fn)
 
     def get_job_workspace(self, job_id):
-        return self.mongo.get_job_workspace(job_id)
+        return self.database.get_job_workspace(job_id)
 
     # ---- EXPERIMENT ----
 
@@ -332,7 +337,7 @@ class StoreV1():
 
             items = list(set(items))
         else:
-            items = self.mongo.get_all_experiments_in_ws(ws_name)
+            items = self.database.get_all_experiments_in_ws(ws_name)
 
         return items
 
@@ -393,7 +398,7 @@ class StoreV1():
         ''' download the experiment files matching 'ws_wildcard' to the  folder 'dest_folder'.
         '''
         ef = self.experiment_files(ws_name, exper_name, use_blobs=True)
-        return ef.download_files(ws_wildcard, dest_folder)
+        return ef.download_files(exper_wildcard, dest_folder)
 
     def get_experiment_filenames(self, ws_name, exper_name, exper_wildcard=None):
         ''' return the name of all experiment files matching 'exper_wildcard'.
@@ -405,7 +410,7 @@ class StoreV1():
         ''' delete the experiment file 'filename'.
         '''
         ef = self.experiment_files(ws_name, exper_name, use_blobs=True)
-        return wf.delete_file(filename)
+        return ef.delete_file(filename)
 
     def does_experiment_file_exist(self, ws_name, exper_name, exper_fn):
         ''' return True if the experiment file 'exper_fn' exists.
@@ -444,7 +449,7 @@ class StoreV1():
         # allow for pending runs to write to our recycled workspace (deleted and then recreated)
         # this is a common problem with the current quicktest and the "xt-demo" workspace
         while True:
-            run_id = self.mongo.get_next_run_id(ws_name, default_next=default_next)
+            run_id = self.database.get_next_run_id(ws_name, default_next=default_next)
             run_name = "run{}".format(run_id)
 
             # ensure we are not somehow overwriting an existing run
@@ -460,7 +465,7 @@ class StoreV1():
     def _create_next_child_directory(self, ws_name, parent_name, child_name):
 
         if not child_name:
-            child_id = self.mongo.get_next_child_id(ws_name, parent_name, default_next=1)
+            child_id = self.database.get_next_child_id(ws_name, parent_name, default_next=1)
             console.print("mongo returned child_id=" + str(child_id))
 
             child_name = "{}.{}".format(parent_name, child_id)
@@ -506,18 +511,13 @@ class StoreV1():
 
     def start_child_run(self, ws_name, parent_run_name, exper_name=None, cmd_line_args=None, all_args=None, description=None, 
             from_ip=None, from_host=None, app_name=None, repeat=None, box_name="local", job_id=None, pool=None, 
-            node_index=None, username=None, aggregate_dest=None, path=None, is_aml=False, child_name=None, compute=None, 
+            node_index=None, username=None, aggregate_dest=None, path=None, child_name=None, compute=None, 
             service_type=None, search_type=None, sku=None, search_style=None, run_index=None):
         '''
         This is usually called from the XT COMPUTE box, so we CANNOT get from_ip and from_computer_name from the local machine.
         '''
 
-        if is_aml:
-            # child_num = self.mongo.get_next_child_id(ws_name, parent_run_name)
-            # child_name = parent_run_name + "_" + str(child_num)
-            child_name = aml_child_name
-        else:
-            child_name = self._create_next_child_directory(ws_name, parent_run_name, child_name=child_name)
+        child_name = self._create_next_child_directory(ws_name, parent_run_name, child_name=child_name)
 
         # we are called from controller, so our box_name is the name of this machine
         hostname = pc_utils.get_hostname()
@@ -531,7 +531,7 @@ class StoreV1():
 
         run_num = run_helper.get_run_number(child_name)
         script = os.path.basename(path) if path else None
-        create_time = utils.get_time()
+        create_time = utils.get_arrow_time_str()
 
         build_parts = constants.BUILD.split(",")
         xt_version = build_parts[0].split(":")[1].strip()
@@ -544,27 +544,26 @@ class StoreV1():
             "compute": compute, "service_type": service_type, "sku": sku, "search_style": search_style,
             "search_type": search_type,  "xt_build": xt_build, "xt_version": xt_version, "run_index": run_index}
 
-        self.mongo.create_mongo_run(dd)
+        self.database.create_db_run(dd)
 
         # log "created" event for child run
-        self.log_run_event(ws_name, child_name, "created", dd, is_aml=is_aml)
+        self.log_run_event(ws_name, child_name, "created", dd)
 
         # append "start" record to workspace summary log
-        dd["time"] = utils.get_time()
+        dd["time"] = utils.get_arrow_time_str()
         dd["event"] = "created"
         text = json.dumps(dd) + "\n"    
 
-        if not is_aml:
-            # append to summary file in run dir
-            self.append_run_file(ws_name, child_name, constants.RUN_SUMMARY_LOG, text)
+        # append to summary file in run dir
+        self.append_run_file(ws_name, child_name, constants.RUN_SUMMARY_LOG, text)
 
         if cmd_line_args:
-            self.log_run_event(ws_name, child_name, "cmd_line_args", {"data": cmd_line_args}, is_aml=is_aml)  
+            self.log_run_event(ws_name, child_name, "cmd_line_args", {"data": cmd_line_args})  
         if all_args:
-            self.log_run_event(ws_name, child_name, "all_args", {"data": all_args}, is_aml=is_aml)
+            self.log_run_event(ws_name, child_name, "all_args", {"data": all_args})
     
         # finally, add a "child_created" record to the parent
-        self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, is_aml=is_aml)
+        self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name})
         
         return child_name
 
@@ -590,20 +589,19 @@ class StoreV1():
     def start_run_core(self, ws_name, run_name, exper_name=None, description=None, username=None,
             box_name=None, app_name=None, repeat=None, is_parent=False, job_id=None, pool=None, node_index=None,
             aggregate_dest="none", path=None, aml_run_id=None, compute=None, service_type=None, search_type=None,
-            sku=None, search_style=None):
+            sku=None, search_style=None, tag_dict=None):
 
         console.diag("start_run: after create_next_run_directory")
 
         # log "created" event for run
         ip = pc_utils.get_ip_address()
         hostname = pc_utils.get_hostname()
-        is_aml = (aml_run_id is not None)
 
         # always log the true name of the box (since there can be multiple clients which would otherwise produce multiple "local"s)
         if box_name == "local":
             box_name = hostname
 
-        if exper_name and not is_aml:
+        if exper_name:
             # create the experiement, if it doesn't already exist
             if not self.does_experiment_exist(ws_name, exper_name):
                 self.create_experiment(ws_name, exper_name)
@@ -612,7 +610,7 @@ class StoreV1():
         run_guid = str(uuid.uuid4())
         run_num = run_helper.get_run_number(run_name)
         script = os.path.basename(path) if path else None
-        create_time = utils.get_time()
+        create_time = utils.get_arrow_time_str()
 
         build_parts = constants.BUILD.split(",")
         xt_version = build_parts[0].split(":")[1].strip()
@@ -628,20 +626,19 @@ class StoreV1():
         if aml_run_id:
             dd["aml_run_id"] = aml_run_id
 
-        self.mongo.create_mongo_run(dd)
+        self.database.create_db_run(dd)
 
-        self.log_run_event(ws_name, run_name, "created", dd, is_aml=is_aml)
+        self.log_run_event(ws_name, run_name, "created", dd)
 
         console.diag("start_run: after log_run_event")
 
         # append "start" record to workspace summary log
-        dd["time"] = utils.get_time()
+        dd["time"] = utils.get_arrow_time_str()
         dd["event"] = "created"
         text = json.dumps(dd) + "\n"
 
         # append to summary file in run dir
-        if not is_aml:
-            self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
+        self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
 
         console.diag("start_run: after append_workspace_file")
 
@@ -656,23 +653,22 @@ class StoreV1():
         return items
 
     def end_run(self, ws_name, run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-        aggregate_dest=None, dest_name=None, is_aml=False):
+        aggregate_dest=None, dest_name=None):
 
         if not end_time:
-            end_time = utils.get_time()
+            end_time = utils.get_arrow_time_str()
 
         self.log_run_event(ws_name, run_name, "ended", {"status": status, "exit_code": exit_code, \
-            "metrics_rollup": metrics_rollup_dict}, event_time=end_time, is_aml=is_aml)
+            "metrics_rollup": metrics_rollup_dict}, event_time=end_time)
 
-        if not is_aml:
-            # append "end" record to workspace summary log
-            end_record = {"ws_name": ws_name, "run_name": run_name, "time": end_time, "event": "end", 
-                "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict}
+        # append "end" record to workspace summary log
+        end_record = {"ws_name": ws_name, "run_name": run_name, "time": end_time, "event": "end", 
+            "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict}
 
-            text = json.dumps(end_record) + "\n"
+        text = json.dumps(end_record) + "\n"
 
-            # append to summary file in run dir
-            self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
+        # append to summary file in run dir
+        self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
 
     def delete_run(self, ws_name, run_name):
         return self.helper.delete_run(ws_name, run_name)
@@ -688,42 +684,37 @@ class StoreV1():
         return text, last_end_time
 
     def rollup_and_end_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
-        report_rollup, use_last_end_time=False, is_aml=False):
+        report_rollup, use_last_end_time=False):
 
-        #console.print("rollup_and_end_run: is_aml=", is_aml)
+        # if use_last_end_time:
+        #     end_time = last_end_time
+        # else:
+        #     end_time = utils.get_time()
 
-        if use_last_end_time:
-            end_time = last_end_time
-        else:
-            end_time = utils.get_time()
+        end_time = utils.get_arrow_time_str()
 
-        if is_aml:
-            log_records = []
-            hparams = {}
-            metrics = {}
-        else:
-            # write run to ALLRUNS file
-            if aggregate_dest and aggregate_dest != "none":
-                # convert entire run log to a single nested record
-                text, last_end_time = self.nest_run_records(ws_name, run_name)
+        # write run to ALLRUNS file
+        if aggregate_dest and aggregate_dest != "none":
+            # convert entire run log to a single nested record
+            text, last_end_time = self.nest_run_records(ws_name, run_name)
 
-                # append nested record to the specified all_runs file
-                if dest_name:
-                    if aggregate_dest == "experiment":
-                        self.append_experiment_file(ws_name, dest_name, constants.ALL_RUNS_FN, text)
-                    elif aggregate_dest == "job":
-                        self.append_job_file(ws_name, dest_name, constants.ALL_RUNS_FN, text)
+            # append nested record to the specified all_runs file
+            if dest_name:
+                if aggregate_dest == "experiment":
+                    self.append_experiment_file(ws_name, dest_name, constants.ALL_RUNS_FN, text)
+                elif aggregate_dest == "job":
+                    self.append_job_file(ws_name, dest_name, constants.ALL_RUNS_FN, text)
 
-            # LOG END RUN
-            log_records = self.get_run_log(ws_name, run_name)
-        
-            hparams = self._roll_up_hparams(log_records) 
-            metrics = self.rollup_metrics_from_records(log_records, primary_metric, maximize_metric, report_rollup) 
+        # LOG END RUN
+        log_records = self.get_run_log(ws_name, run_name)
+    
+        hparams = self._roll_up_hparams(log_records) 
+        metrics = self.rollup_metrics_from_records(log_records, primary_metric, maximize_metric, report_rollup) 
 
         self.end_run(ws_name, run_name, status, exit_code, hparams, metrics, 
-            end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name, is_aml=is_aml)
+            end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name)
 
-        self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
+        self.database.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
     def _roll_up_hparams(self, log_records):
         hparams_dict = {}
@@ -800,11 +791,9 @@ class StoreV1():
     def get_run_log(self, ws_name, run_name):
         return self.helper.get_run_log(ws_name, run_name)
 
-    def log_run_event(self, ws_name, run_name, event_name, data_dict=None, event_time=None, is_aml=False):
-        #console.print("log_run_event: ws_name={}, run_name={}, event_name={}".format(ws_name, run_name, event_name))
-
+    def log_node_event(self, ws_name, job_id, node_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_time()
+            event_time = utils.get_arrow_time_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
@@ -812,29 +801,59 @@ class StoreV1():
         record_dict = {"time": event_time, "event": event_name, "data": data_dict}
         #console.print("record_dict=", record_dict)
 
-        if not is_aml:
-            rd_text = json.dumps(record_dict)
-            # append to run log file
-            self.append_run_file(ws_name, run_name, RUN_LOG, rd_text + "\n")
+        rd_text = json.dumps(record_dict)
+        # append to node.log
+        node_path = "nodes/{}/node.log".format(node_id)
+        self.append_job_file(ws_name, job_id, node_path, rd_text + "\n")
+
+    def log_run_event(self, ws_name, run_name, event_name, data_dict=None, event_time=None, job_id=None):
+        #console.print("log_run_event: ws_name={}, run_name={}, event_name={}".format(ws_name, run_name, event_name))
+
+        if not event_time:
+            event_time = utils.get_arrow_time_str()
+
+        if data_dict and not isinstance(data_dict, dict):   
+            raise Exception("data_dict argument is not a dict: " + str(data_dict))
+
+        record_dict = {"time": event_time, "event": event_name, "data": data_dict}
+        #console.print("record_dict=", record_dict)
+
+        rd_text = json.dumps(record_dict)
+        # append to run log file
+        self.append_run_file(ws_name, run_name, RUN_LOG, rd_text + "\n")
 
         # log all backend types to mongo
-        self.mongo.process_run_event(ws_name, run_name, record_dict)
+        self.database.process_run_event(ws_name, run_name, record_dict)
 
     def get_job_names(self, filter_dict=None, fields_dict=None):
-        return self.mongo.get_job_names(filter_dict, fields_dict)
+        return self.database.get_job_names(filter_dict, fields_dict)
 
     def get_ws_runs(self, ws_name, filter_dict=None, include_log_records=False):
-        return self.mongo.get_ws_runs(ws_name, filter_dict, include_log_records)
+        return self.database.get_ws_runs(ws_name, filter_dict, include_log_records)
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=False, 
         fn_cache=None, batch_size=None):
-
-        return self.mongo.get_all_runs(aggregator_dest, ws_name, job_or_exper_name, filter_dict, fields_dict, use_cache,
+        '''
+        returns a list of all runs in the specified workspace.
+        '''
+        return self.database.get_all_runs(aggregator_dest, ws_name, job_or_exper_name, filter_dict, fields_dict, use_cache,
             fn_cache, batch_size=batch_size)
 
+    def download_all_runs_blobs(self, ws_name, aggregator_dest, job_or_exper_name, dest_fn, 
+        fn_cache=None, max_workers=1):
+        '''
+        download the all_runs.jsonl storage blob from specified job or experiment.
+        '''
+        if aggregator_dest == "job":
+            self.download_file_from_job(ws_name, job_or_exper_name, constants.ALL_RUNS_FN, dest_fn)
+        elif aggregator_dest == "exper":
+            self.download_file_from_experiment(ws_name, job_or_exper_name, constants.ALL_RUNS_FN, dest_fn)
+        else:
+            errors.general_error("unknown aggregator_dest: {}".format(aggregator_dest))
+
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
-        after_omit_list=None, node_id=None, run_index=None):
+        after_omit_list=None, node_id=None, run_index=None, start_time=None):
 
         #console.print("wrapup_run: rundir=", rundir, ", exit_code=", exit_code)
 
@@ -866,12 +885,12 @@ class StoreV1():
 
         # tell mongo RUNS that this run has completed
         console.diag("calling MONGO run_exit: ws={}, run_name={}".format(ws_name, run_name))
-        self.mongo.run_exit(ws_name, run_name)
+        self.database.run_exit(ws_name, run_name, status=None, exit_code=None, start_time=start_time)
 
         if not is_parent:
             # tell mongo this job has a completed run
             console.diag("calling MONGO job_run_exit: job_id={}".format(job_id))
-            self.mongo.job_run_exit(ws_name, job_id, exit_code)
+            self.database.job_run_exit(ws_name, job_id, exit_code)
 
     def copy_run_files_to_run(self, ws_name, from_run, run_wildcard, to_run, to_path):
         return self.helper.copy_run_files_to_run(ws_name, from_run, run_wildcard, to_run, to_path)
@@ -881,7 +900,7 @@ class StoreV1():
         pass
 
     def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
-        return self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
+        return self.database.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
     #---- LOW LEVEL ----
             
@@ -970,13 +989,13 @@ class StoreV1():
         if not default_next:
             default_next = 1
             
-        job_id = self.mongo.get_next_job_id(default_next=default_next)
+        job_id = self.database.get_next_job_id(default_next=default_next)
 
         job_name = "job{}".format(job_id)
         return job_name
 
     def get_job_secret(self, ws_name, job_id):
-        records = self.mongo.get_info_for_jobs( filter_dict={"_id": job_id}, fields_dict={"job_secret": 1} )
+        records = self.database.get_info_for_jobs( filter_dict={"_id": job_id}, fields_dict={"job_secret": 1} )
         value = records[0]["job_secret"] if len(records) else None
         return value
 
@@ -989,7 +1008,7 @@ class StoreV1():
 
         # update mongo-db for fast access
         dd = json.loads(text)
-        self.mongo.update_job_info(ws_name, job_id, dd)
+        self.database.update_job_info(ws_name, job_id, dd)
     
     def log_job_info(self, ws_name, job_id, dd):
         text = json.dumps(dd, indent=4)
@@ -997,7 +1016,7 @@ class StoreV1():
 
     def log_job_event(self, ws_name, job_id, event_name, data_dict=None, event_time=None):
         if not event_time:
-            event_time = utils.get_time()
+            event_time = utils.get_arrow_time_str()
 
         if data_dict and not isinstance(data_dict, dict):   
             raise Exception("data_dict argument is not a dict: " + str(data_dict))
