@@ -33,8 +33,6 @@ class XTVault():
         loaded = False
         self.authentication = authentication
 
-        #utils.debug_break()
-
         loaded = self._load_grok_creds()
         if not loaded:
             loaded = self._load_node_creds()
@@ -72,20 +70,20 @@ class XTVault():
     def _load_node_creds(self):
         loaded = False
 
-        store, mongo = get_store_and_mongo_creds_on_compute_node()
-        if store:
-            # creds are limited in this case to just Store access [storage + mongo]
+        store_creds, db_creds = get_store_and_db_creds_on_compute_node()
+        if store_creds:
+            # creds are limited in this case to just Store access [storage + db_creds]
             kv = {}
-            if store:
-                kv[store["name"]] = store["key"]
+            if store_creds:
+                kv[store_creds["name"]] = store_creds["key"]
 
-            if mongo:
-                kv[mongo["name"]] = mongo["key"]
+            if db_creds:
+                kv[db_creds["name"]] = db_creds["connection-string"]
 
             creds = json.dumps(kv)
             self.apply_creds(creds)
 
-            console.print("init_creds: using compute node ENV VAR settings: {}".format(dd))
+            console.print("init_creds: using compute node ENV VAR settings")
             loaded = True
 
         return loaded
@@ -94,6 +92,7 @@ class XTVault():
 
         # use normal Key Value
         from azure.keyvault.secrets import SecretClient
+        from azure.identity import DefaultAzureCredential
 
         if authentication == "auto":
             authentication = "browser" if pc_utils.has_gui() else "device-code"
@@ -113,7 +112,8 @@ class XTVault():
             errors.syntax_error("unrecognized authentication type '{}'".format(authentication))
 
         new_creds = True
-        outer_token = credential.get_token()
+        scopes = ""
+        outer_token = credential.get_token(scopes)
         token = outer_token.token
 
         # expires = outer_token[1]
@@ -179,43 +179,22 @@ class XTVault():
         return upn
 
 # flat functions
-def get_store_and_mongo_creds_on_compute_node():
+def get_store_and_db_creds_on_compute_node():
 
-    sd = None
-    md = None
+    store_creds = None
+    db_creds = None
 
-    sc = os.getenv("XT_STORE_CREDS")
-    sc = utils.base64_to_text(sc)
+    store_creds64 = os.getenv("XT_STORE_CREDS")
+    if store_creds64:
+        store_creds_json = utils.base64_to_text(store_creds64)
+        store_creds = json.loads(store_creds_json)
 
-    mc = os.getenv("XT_MONGO_CONN_STR")
-    mc = utils.base64_to_text(mc)
-    #print("init_cred: sc={}, mc={}".format(sc, mc))
+    db_creds64 = os.getenv("XT_DB_CREDS")
+    if db_creds64:
+        db_creds_json = utils.base64_to_text(db_creds64)
+        db_creds = json.loads(db_creds_json)
 
-    if sc:   # mongo is now optional (tracking mode)
-        # XT client on compute node 
-
-        # cleanup (for testing from client)
-        sc = sc.replace('\\"', '"')
-
-        # print("sc=", sc)
-        sc_data = json.loads(sc)
-        store_name = sc_data["name"]
-        store_key = utils.safe_value(sc_data, "key")
-
-        sd = {"name": store_name, "key": store_key}
-
-        if mc:
-            mc = mc.replace('\\"', '"')
-            if mc.startswith('"'):
-                mc = mc[1:-1]
-
-            # sample mc: mongodb://xt-sandbox-cosmos:kBOWLQrseZ
-            prefix, mc_rest = mc.split("://", 1)
-            mc_name, _ = mc_rest.split(":", 1)
-            
-            md = {"name": mc_name, "key": mc}
-
-    return sd, md
+    return store_creds, db_creds
 
 # def get_user_photo(token):
 #     import requests
